<?php

namespace Controller;

use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request;

use Zeedhi\Framework\DataSource\DataSet;

class Menu extends \Zeedhi\Framework\Controller\Simple {
    
    const ERROR_MESSAGE = 'E';
    // -- Homologação
    const URL_SERVIDOR_IMAGEM = 'http://midia.teknisa.com/files/';
    // -- Produção
    // const URL_SERVIDOR_IMAGEM = 'http://midia.teknisa.com/files/';
    
    protected $util;
    protected $menuService;
    
    public function __construct(\Util\Util $util, \Service\Menu $menuService) {
        $this->util            = $util;
        $this->menuService     = $menuService;
    }

    /*  
        Retorna o cardápio de um caixa (caixa/loja/filial) no momento em que a rotina é consultada. 
    */
    public function getMenuByPos(Request $request, Response $response) {
        $inputJSON = file_get_contents('php://input');
        $input = json_decode($inputJSON, true);
        $this->util->beginTransaction();
        try {
            if (!isset($input)) {
                throw new \Exception('Não foram recebidos parâmetros');
            }
            
            $data   = array(); // dados a serem retornados

            $params = $input;
            $subsidiaryCode = $params['subsidiaryCode'];
            $storeCode      = $params['storeCode'];
            $posCode        = $params['posCode'];
            $consumerType   = $params['consumerType'];
            $maskedHour     = $params['hour'];
            $hour           = str_replace(':','',$maskedHour);
            
            // verifica a existência da filial
            if (!$this->util->subsidiaryExists($subsidiaryCode)) {
                throw new \Exception('Filial inexistente.');
            }
            
            // verifica a existência da loja relacionada à filial
            if (!$this->util->storeExists($subsidiaryCode, $storeCode)) {
                throw new \Exception('Loja inexistente.');
            }
            
            // verifica a existência do caixa relacionado à loja
            if (!$this->util->posExists($subsidiaryCode, $storeCode, $posCode)) {
                throw new \Exception('Caixa inexistente.');
            }
            
            // busca pela tabela de preços da loja
            $return = $this->menuService->getActivePrincingTable($subsidiaryCode, $storeCode);
            
            // verifica a existência de tabela de preços cadastrada e ativa
            if (!$return['error']) {
                // dados necessários para pesquisar o preço de cada item do cardápio
                $pricingTableCode = $return['CDTABEPREC'];
                $pricingTableDate = $return['DTINIVGPREC'];
                // array com o preço de todos os produtos da loja, independente de grupos ou subgrupos
                $productPrices    = array();
                
                // busca o preço de cada produto da filial segundo o tipo do consumidor
                $result = $this->menuService->getPrices($subsidiaryCode, $pricingTableCode, $pricingTableDate, $consumerType, $hour, false);

                if (!$result['error']) {
                    $productPrices = $result['data'];
                } else {
                    $message = $result['message'];
                    $response->addMessage(new Message($message, self::ERROR_MESSAGE));
                }
                
                // busca os grupos de produtos do cardápio do caixa
                $return = $this->menuService->getGroups($subsidiaryCode, $posCode);
                
                if (!$return['error']) {
                    $groups = $return['data'];
                    
                    foreach ($groups as $group) {
                        $groupImage = $group['DSENDEIMG'] != null ? self::URL_SERVIDOR_IMAGEM.$group['DSENDEIMG'] : null;
                        $groupData = array(
                            'groupName'         => $group['NMGRUPO'],
                            'groupImage'        => $groupImage,
                            'groupCode'         => $group['NRBUTTON'],
                            'groupDescription'  => $group['DSITMENUVENDA'],
                            'products'          => array(),
                            'groupIcon'         => $group['DSENDEIMGAUX'],
                            'subGroups'         => array()
                        );
                        
                        $groupParams = array(
                            'NRPGCONFTELA' => $group['NRPGCONFTELA'],
                            'NRBUTTON'     => $group['NRBUTTON']
                        );
                        
                        // busca os produtos de cada grupo
                        $return = $this->menuService->getGroupProducts($subsidiaryCode, $posCode, $groupParams, $storeCode);
                        if (!$return['error']) {
                            $products = $return['data'];
                            
                            foreach ($products as $product) {
                                if(isset($productPrices[$product['CDPRODUTO']])){
                                    $productImage = $product['DSENDEIMG'] != null ? self::URL_SERVIDOR_IMAGEM.$product['DSENDEIMG'] : null;
                                    $groupProductData = array(
                                        'barCode' => $product['CDBARPRODUTO'],
                                        'productCode' => $product['CDPRODUTO'],
                                        'productName' => $product['NMPRODUTO'],
                                        'productDescription' => $product['DSPRODVENDA'] == null ? "" : $product['DSPRODVENDA'],
                                        'hasComboProducts' => $product['IDTIPOCOMPPROD'] == '3',
                                        'productImage' => $productImage,
                                        'price'       => 0.0
                                    );
                                    $hasObservations = $this->menuService->checkProductObservations($product['CDPRODUTO'], $subsidiaryCode, $storeCode);
                                    
                                    $groupProductData['hasObservations'] = $hasObservations;
                                    if (isset($product['CDPRODUTO'])) {
                                        $price = $productPrices[$product['CDPRODUTO']]['VRPRECITEM'];
                                        if (!empty($price)) {
                                            // arredonda o valor com precisão de duas casas decimais
                                            $price = round(floatval($price), 2);
                                            $groupProductData['price'] = $price;
                                            
                                            // só adiciona produtos sem problemas com preços
                                            array_push($groupData['products'], $groupProductData);
                                        } else {
                                            // $groupProductData['price'] = $this->util->float('0.00');
                                            
                                            // $message = 'O preço do produto '.$product['CDPRODUTO'].' está zerado';
                                            // $response->addMessage(new Message($message, self::ERROR_MESSAGE));
                                        }
                                    } else {
                                        // $message = 'Não há preço cadastrado para o produto '.$product['CDPRODUTO'];
                                        // $response->addMessage(new Message($message, self::ERROR_MESSAGE));
                                    }
                                }
                            }

                        } else {
                            // Erro ao buscar produtos do grupo
                            $message = $return['message'];
                            $response->addMessage(new Message($message, self::ERROR_MESSAGE));
                        }
                        // busca os subgrupos de cada grupo
                        $return = $this->menuService->getSubGroups($subsidiaryCode, $posCode, $groupParams);
                        
                        if (!$return['error']) {
                            $subGroups = $return['data'];
                            // --- temporário
                            $subGroupData = array();
                            // ---
                            foreach ($subGroups as $subGroup) {
                                $subGroupData = array(
                                    'name'     => $subGroup['NMSUBGRUPO'],
                                    'products' => array()
                                );
                                
                                $subGroupParams = array(
                                    'NRPGCONFTELA' => $subGroup['NRPGCONFTELA'],
                                    'NRBUTTON'     => $subGroup['NRBUTTON']
                                );
                                
                                // busca os produtos de cada subgrupo
                                $return = $this->menuService->getSubGroupProducts($subsidiaryCode, $posCode, $groupParams, $subGroupParams, $storeCode);
                                
                                if (!$return['error']) {
                                    $products = $return['data'];
                                    
                                    foreach ($products as $product) {
                                        $productImage = $product['DSENDEIMG'] != null ? self::URL_SERVIDOR_IMAGEM.$product['DSENDEIMG'] : null;
                                        $subgroupProductData = array(
                                            'barCode' => $product['CDBARPRODUTO'],
                                            'productCode' => $product['CDPRODUTO'],
                                            'productName' => $product['NMPRODUTO'],
                                            'productDescription' => $product['DSPRODVENDA'] == null ? "" : $product['DSPRODVENDA'],
                                            'hasComboProducts' => $product['IDTIPOCOMPPROD'] == '3',
                                            'productImage' => $productImage, 
                                            'price' => 0.0
                                        );
                                        
                                        $hasObservations = $this->menuService->checkProductObservations($product['CDPRODUTO'], $subsidiaryCode, $storeCode);
                                        $subgroupProductData['hasObservations'] = $hasObservations;
                                        
                                        $price = $productPrices[$product['CDPRODUTO']]['VRPRECITEM'];
                                        if (!empty($price)) {
                                            $subgroupProductData['price'] = $this->util->float($price);
                                        } else {
                                            $subgroupProductData['price'] = $this->util->float('0.00');
                                        }
                                        array_push($subGroupData['products'], $subgroupProductData);
                                    }
                                } else {
                                    // Erro ao buscar produtos do subgrupo
                                    $message = $return['message'];
                                    $response->addMessage(new Message($message, self::ERROR_MESSAGE));
                                }
                            }
                            
                            array_push($groupData['subGroups'], $subGroupData);
                        } else {
                            // Erro ao buscar subgrupos do grupo
                            $message = $return['message'];
                            $response->addMessage(new Message($message, self::ERROR_MESSAGE));
                        }
                        
                        array_push($data, $groupData);
                    } 
                    
                    $response->addDataSet(new DataSet('', $data));
                } else {
                    // Erro ao buscar grupos de produtos
                    $message = $return['message'];
                    $response->addMessage(new Message($message, self::ERROR_MESSAGE));
                }
            } else {
                $message = $return['message'];
                $response->addMessage(new Message($message, self::ERROR_MESSAGE));
            }
            $this->util->commit();
        } catch (\Exception $e) {
            $this->util->rollBack();
            $message = 'Erro ao buscar cardápio do caixa. '.$e->getMessage();
            $response->addMessage(new Message($message, self::ERROR_MESSAGE));
        }
    }
    
    public function getProductObservations(Request $request, Response $response){
        $inputJSON = file_get_contents('php://input');
        $input = json_decode($inputJSON, true);
        $this->util->beginTransaction();
        try {
            if (!isset($input)) {
                throw new \Exception('Não foram recebidos parâmetros');
            }
            
            $data   = array(); // dados a serem retornados

            $params = $input;
            
            $subsidiaryCode = $params['subsidiaryCode'];
            $storeCode      = $params['storeCode'];
            $consumerType   = $params['consumerType'];
            $productCode    = $params['productCode'];
            $maskedHour     = $params['hour'];

            $hour           = str_replace(':','',$maskedHour);
            
            // verifica a existência da filial
            if (!$this->util->subsidiaryExists($subsidiaryCode)) {
                throw new \Exception('Filial inexistente.');
            }
            
            // verifica a existência da loja relacionada à filial
            if (!$this->util->storeExists($subsidiaryCode, $storeCode)) {
                throw new \Exception('Loja inexistente.');
            }

            // busca pela tabela de preços da loja
            $return = $this->menuService->getActivePrincingTable($subsidiaryCode, $storeCode);
            $productPrices = array();
            // verifica a existência de tabela de preços cadastrada e ativa
            if (!$return['error']) {
                // dados necessários para pesquisar o preço de cada item do cardápio
                $pricingTableCode = $return['CDTABEPREC'];
                $pricingTableDate = $return['DTINIVGPREC'];
             
                // busca o preço de cada produto da filial segundo o tipo do consumidor
                $return = $this->menuService->getPrices($subsidiaryCode, $pricingTableCode, $pricingTableDate, $consumerType, $hour, false);

                if(!$return['error']){
                    $productPrices = $return['data'];
                }
            }
                
            $returnGroups = $this->menuService->getObservationGroups($subsidiaryCode, $storeCode);           
            if(!$returnGroups['error']){
                $observationGroups = $returnGroups['data'];
                $groupList = array();
                foreach($observationGroups as $group){
                    $groupCode = $group['CDGRPOCOR'];
                    $groupData = array(
                        'obsGroupCode' => $group['CDGRPOCOR'],
                        'obsGroupName' => $group['DSGRPOCOR'],
                        'minQuantity'  => $group['NRMINOCORR'],
                        'maxQuantity'  => $group['NRMAXOCORR']
                    );
                    $returnObservations = $this->menuService->getObservationsByProduct($groupCode, $productCode, $subsidiaryCode, $storeCode);
                    $observationList = array();
                    if(!$returnObservations['error']){
                        $observations = $returnObservations['data'];
                        foreach($observations as $observation){
                            $observationData = array(
                                'obsCode' => $observation['CDOCORR'],
                                'obsName' => $observation['DSOCORR'],
                                'price'   => null
                            );     

                            if($observation['CDPRODUTO'] != null){
                                $prodObsCode = $observation['CDPRODUTO'];
                                if(isset($productPrices[$prodObsCode])){
                                    $price = $productPrices[$prodObsCode]['VRPRECITEM'];
                                    if (!empty($price)) {
                                        $observationData['price'] = $this->util->float($price);
                                    }   
                                }
                            }
                            array_push($observationList, $observationData);
                        }

                        // Ordenação de observações por ordem numérica e alfanumérica
                        usort($observationList, function($a, $b) {
                            $a = $a['obsCode'];
                            $b = $b['obsCode'];
                            $reA = "/[^a-zA-Z]/g";
                            $reN = "/[^0-9]/g";
                            $aA = str_replace($reA, "", $a);
                            $bA = str_replace($reA, "", $b);
                            if ($aA === $bA) {
                                $aN = intval(str_replace($reN, "", $a));
                                $bN = intval(str_replace($reN, "", $b));
                                return $aN === $bN ? 0 : $aN > $bN ? 1 : -1;
                            } else {
                                return $aA > $bA ? 1 : -1;
                            }
                        });
                        
                        // usort($observationList, function($a, $b) {
                        //     if($a['price'] < $b['price'])
                        //         return 1;
                        //     return -1;
                        // });
                    }
                    if(!empty($observationList)){
                        $groupData['observations'] = $observationList;
                        array_push($groupList, $groupData);
                    }
                }
                $response->addDataSet(new DataSet('', $groupList));
            }else{
                $message = $returnGroups['message'];
                $response->addMessage(new Message($message, self::ERROR_MESSAGE));
            }
            $this->util->commit();
        } catch (\Exception $e) {
            $this->util->rollBack();
            $message = 'Erro ao buscar observacoes do produto. '.$e->getMessage();
            $response->addMessage(new Message($message, self::ERROR_MESSAGE));
        }
    }

    public function getProductCombos(Request $request, Response $response){
        $inputJSON = file_get_contents('php://input');
        $input = json_decode($inputJSON, true);
        $this->util->beginTransaction();
        try {
            if (!isset($input)) {
                throw new \Exception('Não foram recebidos parâmetros');
            }
            
            $data   = array(); // dados a serem retornados

            $params = $input;
            $subsidiaryCode = $params['subsidiaryCode'];
            $storeCode      = $params['storeCode'];
            $posCode        = $params['posCode'];
            $consumerType   = $params['consumerType'];
            $productCode    = $params['productCode'];
            $maskedHour     = $params['hour'];
            $hour           = str_replace(':','',$maskedHour);
            
            // verifica a existência da filial
            if (!$this->util->subsidiaryExists($subsidiaryCode)) {
                throw new \Exception('Filial inexistente.');
            }
            
            // verifica a existência da loja relacionada à filial
            if (!$this->util->storeExists($subsidiaryCode, $storeCode)) {
                throw new \Exception('Loja inexistente.');
            }
            
            // verifica a existência do caixa relacionado à loja
            if (!$this->util->posExists($subsidiaryCode, $storeCode, $posCode)) {
                throw new \Exception('Caixa inexistente.');
            }

            // busca pela tabela de preços da loja
            $return = $this->menuService->getActivePrincingTable($subsidiaryCode, $storeCode);
            $productPrices = array();
            $productConsumerPrices = array();
            // verifica a existência de tabela de preços cadastrada e ativa
            if (!$return['error']) {
                // dados necessários para pesquisar o preço de cada item do cardápio
                $pricingTableCode = $return['CDTABEPREC'];
                $pricingTableDate = $return['DTINIVGPREC'];
             
                // busca o preço de cada produto da filial segundo o tipo do consumidor
                $return = $this->menuService->getPrices($subsidiaryCode, $pricingTableCode, $pricingTableDate, $consumerType, $hour, true);
                if(!$return['error']){
                    $productConsumerPrices = $return['data'];
                }
                // busca o preço de cada produto da filial segundo o tipo do consumidor
               /*  $consumerType = 'T';
                $return = $this->menuService->getPrices($subsidiaryCode, $pricingTableCode, $pricingTableDate, $consumerType, $hour);
                if(!$return['error']){
                    $productPrices = $return['data'];
                } */
            }

            $returnGroups = $this->menuService->getComboGroupsBySubsidiary($subsidiaryCode, $productCode);
            if(!$returnGroups['error']){
                
                $comboGroups = $returnGroups['data'];
                $useSubsidiaryGroups = true;
                if(empty($comboGroups)){
                    $useSubsidiaryGroups = false;
                    $returnGroups = $this->menuService->getComboGroupsByProduct($productCode);
                    
                    if(!$returnGroups['error']){
                        $comboGroups = $returnGroups['data'];
                    }else{
                        $message = $returnGroups['message'];
                        $response->addMessage(new Message($message, self::ERROR_MESSAGE));
                    }   
                }


                $groupList = array();

                foreach($comboGroups as $comboGroup){
                   
                    $comboGroupData = array(
                        'groupName'  => $comboGroup['NMGRUPROMOC'],
                        'groupImage' => null,
                        'quantity'   => $comboGroup['QTPRGRUPPROMOC'],
                        'products'   => array()
                    );

                    $comboGroupCode = $comboGroup['CDGRUPROMOC'];

                    if($useSubsidiaryGroups){
                        
                        $return = $this->menuService->getComboProductsBySubsidiary($subsidiaryCode, $storeCode, $comboGroupCode, $productCode);
                    }else{
                        $return = $this->menuService->getComboProductsByProduct($subsidiaryCode, $storeCode, $comboGroupCode, $productCode, $pricingTableCode);
                    }

                    $comboProductsList = array();
                    if(!$return['error']){
                        $comboProducts = $return['data']; 
                        foreach($comboProducts as $comboProduct){
                            $comboProductData = array(
                                'barCode'            => $comboProduct['CDBARPRODUTO'],
                                'productCode'        => $comboProduct['CDPRODUTO'],
                                'productName'        => $comboProduct['NMPRODUTO'],
                                'productDescription' => $comboProduct['DSPRODVENDA'],
                                'hasComboProducts'   => false,
                                'productImage'       => $comboProduct['DSENDEIMG'] != null ? self::URL_SERVIDOR_IMAGEM.$comboProduct['DSENDEIMG'] : null,
                                'price'              => 0
                            );
                            $currentProductCode = $comboProduct['CDPRODUTO'];

                            $hasObservations = $this->menuService->checkProductObservations($currentProductCode, $subsidiaryCode, $storeCode);

                            $comboProductData['hasObservations'] = $hasObservations;

                            $usePercentDiscount = $comboProduct['IDPERVALORDES'] == 'P';
                            $discountValue      = $this->util->float($comboProduct['VRDESPRODPROMOC']);
                            $productPrice = 0;


                            if(isset($productConsumerPrices[$currentProductCode])){
                                $productPrice = $productConsumerPrices[$currentProductCode]['VRPRECITEM'];
                                if (!empty($productPrice) && $discountValue !== 0) {
                                    if($usePercentDiscount){
                                        $productPrice = $this->util->float($productPrice) - ($this->util->float($productPrice) * ($discountValue)/100);
                                    }else{
                                        $productPrice = $this->util->float($productPrice) - $discountValue;    
                                    }
                                }

                                $applyDiscount = $productConsumerPrices[$currentProductCode]['IDPERVALORPR'];
                                $disAcr = $productConsumerPrices[$currentProductCode]['IDDESCACREPR'];

                                if($disAcr !== null){
                                    if($disAcr == 'D'){
                                        if($applyDiscount == 'V'){
                                            $productPrice = $productPrice - $this->util->float($productConsumerPrices[$currentProductCode]['VRPRECODIA']); 
                                        }else{
                                            $productPrice =  $productPrice - ($productPrice * $this->util->float($productConsumerPrices[$currentProductCode]['VRPRECODIA'])/100); 
                                            
                                        }
                                    
                                    }else{
                                        if($applyDiscount == 'V'){
                                            $productPrice += $this->util->float($productConsumerPrices[$currentProductCode]['VRPRECODIA']); 
                                        }else{
                                            $productPrice =  $productPrice + ($productPrice * $this->util->float($productConsumerPrices[$currentProductCode]['VRPRECODIA'])/100); 
                                        }
                                    }
                                } 
                            }
                            $comboProductData['price'] = $this->util->float($productPrice);
                            if($comboProductData['price'] < 0.01){
                                $comboProductData['price'] = 0.01;
                            }
                            array_push($comboProductsList, $comboProductData);
                        }

                        if(!empty($comboProductsList)){
                            $comboGroupData['products'] = $comboProductsList;
                            array_push($groupList, $comboGroupData);
                        }
                    }else{
                        $message = $return['message'];
                        $response->addMessage(new Message($message, self::ERROR_MESSAGE));
                    } 
                }

                $response->addDataSet(new DataSet('', $groupList));
            }else{
                $message = $returnGroups['message'];
                $response->addMessage(new Message($message, self::ERROR_MESSAGE));
            }
            $this->util->commit();
        } catch (\Exception $e) {
            $this->util->rollBack();
            $message = 'Erro ao buscar combos do produto. '.$e->getMessage();
            $response->addMessage(new Message($message, self::ERROR_MESSAGE));
        }
    }
    
    public function getWaitingTime(Request $request, Response $response){
        try{
            $inputJSON = file_get_contents('php://input');
            $input = json_decode($inputJSON, true);
            if (isset($input['subsidiaryCode'])) $subsidiaryCode = $input['subsidiaryCode'];
            else throw new \Exception('Filial não informada.');

            if (isset($input['storeCode'])) $stores = $input['storeCode'];
            else throw new \Exception('Lojas não informadas.');
            
            $storeCode = array();

            foreach($stores as $store){
                $returnWaitingTime = $this->menuService->getWaitingTime($subsidiaryCode, $store);
                if($returnWaitingTime['data'] == false || empty($returnWaitingTime['data']['VRTMPMEDPROD']) ) $returnWaitingTime = $this->menuService->getStoreWaitingTime($subsidiaryCode, $store);
                $time = $returnWaitingTime['data'];
                $storeCode[$store] = empty($time['VRTMPMEDPROD']) ? $time['VRTMPESPLJ'] : $time['VRTMPMEDPROD'];
                $storeCode[$store] = $this->util->convertToHoursMins($storeCode[$store], '%02d:%02d:00');
            }
            $response->addDataSet(new DataSet('', array('storeCode' => $storeCode)));            
        } catch (\Exception $e) {
            $message = 'Erro ao recuperar o tempo de espera. ' . $e->getMessage();
            $response->addMessage(new Message($message, self::ERROR_MESSAGE));
        }
    }
}