<?php

namespace Controller;

use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request;

use Zeedhi\Framework\DataSource\DataSet;

class Order extends \Zeedhi\Framework\Controller\Simple {
    
    // --- Messages 
    const ERROR_MESSAGE     = 'E';
    const GENERAL_MESSAGE   = 'A';
    // --- Address
    const PRINCIPAL_ADDRESS = 'P';
    // --- Table/Positions 
    const REQUEST_ACCOUNT   = 'S';
    const BUSY              = 'O';
    const PAID              = 'P';

    const SCHEDULED_ORDER         = 'A';
    const PENDING_SCHED_ORDER     = 'N';
    const NOT_PENDING_SCHED_ORDER = 'S';

    const MAX_CONTROL_FILE_UPDATE_TIME = 60;
    
    protected $util;
    protected $orderService;
    protected $geocodeURL;
    protected $printMode;
    protected $impressaoUtil;
    protected $menuService;
    protected $requestTimeout;
    
    public function __construct(\Util\Util $util, \Service\Order $orderService, $geocodeURL,
                                \Zeedhi\Framework\DependencyInjection\InstanceManager $instanceManager,
                                \Odhen\API\Lib\ImpressaoUtil $impressaoUtil,
                                \Service\Menu $menuService) {
        date_default_timezone_set($instanceManager->getParameter('TIMEZONE'));
        $this->util           = $util;
        $this->orderService   = $orderService;
        $this->geocodeURL     = $geocodeURL;
        $this->printMode      = $instanceManager->getParameter('PRINT_MODE');
        $this->requestTimeout = $instanceManager->getParameter('REQUEST_TIMEOUT');
        $this->systemPath        = $instanceManager->getParameter('SYSTEM_PATH');
        $this->logPath        = $instanceManager->getParameter('LOG_PATH');
        $this->impressaoUtil  = $impressaoUtil;
        $this->menuService    = $menuService;
    }
    /*
        Salva uma venda delivery/toGo
    */ 
    public function saveDeliveryOrder(Request $request, Response $response) {
        $inputJSON = file_get_contents('php://input');
        $input = json_decode($inputJSON, true);
        $timeout = $this->requestTimeout;
        try{
            if (!isset($input)) throw new \Exception('Não foram recebidos parâmetros');
        }catch(\Exception $e){
            $message = 'Erro ao inserir comanda delivery. ' . $e->getMessage();
            $response->addMessage(new Message($message, self::ERROR_MESSAGE));
            return;
        }
        $data = array();
        $params = $input;
        try{
            $taaSaleId = isset($input['taaSaleId']) ? $input['taaSaleId'] : null;
            if(!empty($taaSaleId)){
                $orderById = $this->orderService->getOrderById($taaSaleId);
                if(!empty($orderById)){
                    if($this->util->databaseIsOracle())
                        $format = 'Y-m-d H:i:s';
                    else
                        $format = 'Y-m-d H:i:s.u';
                    
                    $orderTime = \DateTime::createFromFormat($format, $orderById['DTHRABERMESA']);
                    $orderTime = $orderTime->format($format);
                    throw new \Exception("Pedido $taaSaleId já existente no banco de dados. Pedido feito em $orderTime");
                }
            }
        }catch(\Exception $e){
            $response->addMessage(new Message($e->getMessage(), self::GENERAL_MESSAGE));
            $response->addDataSet(
                new DataSet(
                    '', 
                    array(
                        array(
                            'saleId' => $orderById['NRCOMANDAEXT'],
                            'saleCode' => $orderById['NRCOMANDA']
                        )
                    )
                )
            );
            return;
        }
        $order = $params['order'];
        $fileName = date("Ymd-H-i-s")."-"; 
        $pagerId = isset($order['pagerId']) ? $order['pagerId'] : null;
        $fileName .= empty($pagerId) ? $order['subsidiaryCode'] : $pagerId;
        $this->util->saveLogOrder($inputJSON,'Order/'.date("Y/m/d"),$fileName);
        $this->util->beginTransaction();
        try{
            $subsidiaryCode = $order['subsidiaryCode'];
            $comandavenSeq    = isset($order['saleCode']) ? $order['saleCode'] : null;
            $dscomanda        = isset($order['orderId']) ? $order['orderId'] : null;
            $nrmesa           = isset($order['tableNumber']) ? $order['tableNumber'] : null;
            $tableData = $this->orderService->getTableData($nrmesa, $subsidiaryCode);
            $comandavenData = $this->orderService->getComandaData($comandavenSeq, $subsidiaryCode);
            if (empty($comandavenData) && empty($tableData)) {
                $vendarestSeq = $this->orderService->generateVendarestSeq($subsidiaryCode);
                $comandavenSeq = $this->orderService->generateComandavenSeq($subsidiaryCode);
            }
            $this->util->commit();
        }catch(\Exception $e){
            $this->util->rollBack();
            $message = 'Erro ao gerar novo codigo. '.$e->getMessage();
            $this->util->saveLogOrder($message,'Error/'.date("Y/m/d"),$fileName);
            $response->addMessage(new Message($message, self::ERROR_MESSAGE));
            return;
        }
        $now = strtotime("now");
        
        $this->util->beginTransaction();
        try {
            $paymentOnlyWithCashback = false;
            $consumerCode     = !empty($order['consumerCode']) ? $order['consumerCode'] : null;
            $clientCode       = $order['clientCode'];
            $billParams       = $order['orderDescription'];
            $cashBack         = isset($billParams['cashBack']) ? $billParams['cashBack'] : null;
            $orderType        = $billParams['orderType'];
            $orderDiscount    = isset($billParams['orderDiscount']) ? $billParams['orderDiscount'] : 0;
            $holdTime         = isset($billParams['holdTime']) ? $billParams['holdTime'] : 0;
            $productsValue    = $billParams['productsValue'];
            $remaingValue     = $billParams['productsValue'];
            $toTravel         = !empty($billParams['toTravel']);
            $driveThru        = !empty($billParams['driveThru']);
            $orderDescription = isset($billParams['description']) ? $billParams['description'] : null;
            $sellerCode       = isset($order['sellerCode']) ? $order['sellerCode'] : null;
            $closeByApi       = isset($order['closeByApi']) ? $order['closeByApi'] : false;
            $items            = $order['items'];
            
            $closeByApi = !empty($closeByApi) ? 'S' : 'N';
            
            $realItemsQuant   = 0;
            for($i = 0; $i < count($items); $i++){
                $realItemsQuant += $this->divideItems($items, $items[$i]);
            }
            unset($item);
            $standardTable = null;
            if(!empty($nrmesa)){
                $this->orderService->validateTable($nrmesa);
            }

            if(!empty($cashBack)) $this->validateCashBack($clientCode, $consumerCode, $cashBack);
            /*
                Para que seja entendido e o código abaixo não seja retirado até o #~#, é necessário entender(tentar) a lógica utilizada.
                
                Para que uma venda seja válida, TODO item tem que ter um valor de pelo menos 0.01 centavo.
                
                Para saber qual valor máximo de desconto(DESCONTO+CASHBACK) que o pedido pode ter, basta descobrir a quantidade de items existentes
                no pedido e dividir por 100, o resultado de (qtdItems/100) irá retornar a quantidade de items representados em centavos.
                
                SE o valor do CASHBACK + DESCONTO - quantidade de items em centavos for > que o valor de todos os produtos
                o CASHBACK é somado ao DESCONTO já existente, isso valida casos como: produtos custam 50 reais, temos 20 reais
                de desconto, 15 reais de cashback, e 7 items, então o máximo de cashback possível pra esse pedido é 
                50 - 20 - 7/100 que daria 29.93, 29.93 é o valor máximo de cashback possível
                SENÃO respeitar a regra acima, o CASHBACK a ser aplicado é recuperado da seguinte forma: VALORPRODUTOS - DESCONTO
                - quantidade de items em centavos.
                DESCONTO recebe DESCONTO + valor encontrado acima.
                e o CASHBACK quer será utilizado para salvar no campo VRDESCFID da tabela COMANDAVEN que informa quanto de valor
                cashback foi utilizado no pedido, é dado por CASHBACK que será aplicado + quantidade de itens em centavos.
                EXEMPLO: 12 ITEMS com valor total de 50 reais, desconto de 20 reais, com cashback de 60. este caso não irá respeitar
                a primeira regra, portanto: 50 - 20 - 12/100 = 29.88 ESTE é o valor máximo de CASHBACK que será aplicado,
                pode ser valido por 29.88 + 20 = 29.88 sobrando assim 0.01 centavo pra cada item.
                E o valor que será salvo na VRDESCFID é basicamente os 29.88 + 0.12 que dá 30. Este seria o valor de cashback
                utilizado caso pudéssemos zerar uma venda. 50-20-30 = 0.
            */

            $items = $this->recalculateComboDiscount($subsidiaryCode, $items);

            $realItemsQuant = $this->util->div($realItemsQuant, 100);
            if(!empty($cashBack)){
                if($this->util->add($cashBack, $orderDiscount) < $productsValue){
                    $orderDiscount = $this->util->add($orderDiscount, $cashBack);
                }else{
                    $paymentOnlyWithCashback = true;
                    $cashToApply = $this->util->subtract($this->util->subtract($productsValue, $orderDiscount), $realItemsQuant);
                    $orderDiscount = $this->util->add($orderDiscount, $cashToApply);
                    $userCashBack = $cashBack;
                    $cashBack = $this->util->add($cashToApply, $realItemsQuant);
                    
                    
                    if($userCashBack < $this->util->add($cashBack, $billParams['deliveryFee'])){
                        $cashBack = $userCashBack;
                    }else{
                        $cashBack = $this->util->add($cashBack, $billParams['deliveryFee']);
                    }
                }
            }
            /* #~# */
            $remaingDiscount = $orderDiscount;
            // --- Order items
            // --- Order payments
            $payments      = $billParams['payments'];
            $giftCards     = isset($billParams['giftCards']) ? $billParams['giftCards'] : array();
            if(!empty($payments) || !empty($giftCards) || !empty($cashBack))
                $this->validatePayment($billParams['totalValue'], $payments, $giftCards, $cashBack);
            // --- Consumer description
            $consumerDescription = isset($order['consumerId']) ? $order['consumerId'] : null;
            // --- Pager code
            
            // verifica a existência da filial
            if (!$this->util->subsidiaryExists($subsidiaryCode)) {
                throw new \Exception('Filial inexistente.');
            }
            if(!empty($cashBack) && (empty($clientCode) && empty($consumerCode))){
                throw new \Exception('Não pode ser utilizado saldo fidelidade sem consumidor identificado.');
            }
            if(!empty($clientCode) && !empty($consumerCode)){
                if(!$this->util->consumerExists($clientCode, $consumerCode)){
                    throw new \Exception('Consumidor inexistente.');
                }    
            }else{
                $defaultClient = $this->util->getDefaultClient($subsidiaryCode);
                
                if(!empty($defaultClient))
                    $clientCode = $defaultClient;
                else 
                    throw new \Exception('Cliente padrão inexistente.');
            }
            
            $orderType = $this->validateOrderType($orderType, $toTravel);
            $storeCode     = null;
            $sellerPos     = null;
            $idposobsped   = null;
            $idlugarmesa   = null;
            if(isset($order['storeCode'])){
                $storeCode = $order['storeCode']; 
                $storeReturn = $this->util->getStoreDetails($subsidiaryCode,$storeCode);
                if(!empty($storeReturn)){
                    if(empty($standardTable)) $standardTable = $storeReturn['NRMESA'];
                    $idposobsped = $storeReturn['IDPOSOBSPED'];
                    $idlugarmesa = $storeReturn['IDLUGARMESA'];
                }
                
            }
            if(isset($order['posCode'])){
                $posCode   = $order['posCode'];
                $sellerPos = $posCode;
            }
            
            if(empty($sellerCode)) $sellerReturn = $this->orderService->getStandardSeller($subsidiaryCode);
            else $sellerReturn = $this->orderService->getVendorInfo($sellerCode);
            
            if(!$sellerReturn['error']){
                
                $standardSeller = $sellerReturn['data'];
                $sellerCode = $standardSeller['CDVENDEDOR'];
                $operatorCode = $standardSeller['CDOPERADOR'];
                if($storeCode == null)
                    $storeCode = $standardSeller['CDLOJA'];
                if($sellerPos == null)
                    $sellerPos = $standardSeller['CDCAIXA'];
                if($standardTable == null)
                    $standardTable = $standardSeller['NRMESA'];
                if($idposobsped == null)
                    $idposobsped = $standardSeller['IDPOSOBSPED'];
                if($idlugarmesa == null)
                    $idlugarmesa = $standardSeller['IDLUGARMESA'];

                $orderExists = false;
                
                $table = !empty($nrmesa) ? $nrmesa : $standardTable;
                
                if(!empty($dscomanda))
                    $oderByDscomanda = $this->orderService->getOrderDscomanda($dscomanda, $subsidiaryCode);

                $tableData = $this->orderService->getTableData($nrmesa, $subsidiaryCode);
                $comandavenData = $this->orderService->getComandaData($comandavenSeq, $subsidiaryCode);
                if (!empty($comandavenData)) {
                    $vendarestSeq = $comandavenData['NRVENDAREST'];
                    $comandavenSeq = $comandavenData['NRCOMANDA'];
                    $orderExists = true;
                }else if (!empty($tableData)) {
                    $vendarestSeq = $tableData['NRVENDAREST'];
                    $comandavenSeq = $tableData['NRCOMANDA'];
                    $orderExists = true;
                }else if (!empty($oderByDscomanda)) {
                    $vendarestSeq = $oderByDscomanda['NRVENDAREST'];
                    $comandavenSeq = $oderByDscomanda['NRCOMANDA'];
                    $orderExists = true;
                } else {
                    // $vendarestSeq = $this->orderService->generateVendarestSeq($subsidiaryCode);
                    $returnVendarest = $this->insertVendaRest($subsidiaryCode,$vendarestSeq,$storeCode,$sellerCode,$operatorCode,$clientCode,$consumerCode, $table);
                    if ($returnVendarest['error']) 
                        $response->addMessage(new Message($returnVendarest['message'], self::ERROR_MESSAGE)); 
                    // $comandavenSeq = $this->orderService->generateComandavenSeq($subsidiaryCode);
                    if(!empty($nrmesa))
                        $this->orderService->updateTableStatus($subsidiaryCode, $storeCode, $nrmesa);
                }
                
                if($paymentOnlyWithCashback === true){
                    $cdtiporece = $this->util->getMoney();
                    $movcaixadlvSeq    = str_pad(1, 10, '0', STR_PAD_LEFT);
                    $paymentValue = $this->util->add($realItemsQuant, $billParams['deliveryFee']);
                    $this->insertMovCaixaDlv($subsidiaryCode,$vendarestSeq,$movcaixadlvSeq,'E',$clientCode,$consumerCode,$cdtiporece,$paymentValue, null, 1);                    
                }

                if($this->isToGo($orderType)){
                    $toGo = 'S';
                    $subsidiaryAddress = $this->util->getSubsidiaryAddress($subsidiaryCode, self::PRINCIPAL_ADDRESS);
                    if(!empty($subsidiaryAddress)){
                        $cityCode      = $subsidiaryAddress['CDMUNICIPIO'];
                        $state         = $subsidiaryAddress['SGESTADO'];
                        $country       = $subsidiaryAddress['CDPAIS'];   
                    }else{
                        $cityCode      = null;
                        $state         = null;    
                        $country       = null;
                    }    
                    $zipCode           = null;
                    $streetAddress     = null;
                    $neighborhood      = null;
                    $number            = null;
                    $reference         = null;
                    $phone             = null;
                    $cellphone         = null;
                    $addressLine2      = null;
                    $fullStreetAddress = null;
                    $deliveryFee       = 0;
                }else{
                    $toGo           = 'N';
                    $addressDetails = $billParams['deliveryAddress'];
                    $state          = $addressDetails['state'];
                    $cityCodeExt    = $addressDetails['city'];
                    $cityCode       = $this->util->getCityCode($cityCodeExt, $state);
                    $zipCode        = $addressDetails['zipCode'];
                    $streetAddress  = $addressDetails['streetAddress'];
                    $neighborhood   = $addressDetails['neighborhood'];
                    $number         = $addressDetails['number'];
                    $reference      = $addressDetails['reference'];
                    $phone          = $addressDetails['phone'];
                    $cellphone      = $addressDetails['cellphone'];
                    $addressLine2   = $addressDetails['addressLine2'];
                    $deliveryFee    = (double)$billParams['deliveryFee'];
                    $fullStreetAddress = $streetAddress.', '.$number;
                    $country        = '0055';
                    $phoneToUpdate  = !empty($phone) ? $phone : $cellphone;
                    $this->util->updateConsumerPhone($clientCode, $consumerCode, $phoneToUpdate);
                }
                $idcontroprod = $this->getIdcontroprod($storeCode,$subsidiaryCode);
                $IDSTCOMANDA = '3';
                if($idcontroprod === 'S')
                    $IDSTCOMANDA = '1';
                
                if(empty($taaSaleId)){
                    $orderToken = sha1($subsidiaryCode.$vendarestSeq.$comandavenSeq.$clientCode.$consumerCode);
                }else{
                    $orderToken = $taaSaleId;
                }
                $scheduledDate = null;
                $isScheduled   = $billParams['isScheduled'];
                $scheduledDate = $isScheduled ? $billParams['scheduledDate'] : $scheduledDate;
                $IDSTCOMANDA   = $isScheduled ? self::SCHEDULED_ORDER : $IDSTCOMANDA; 
                $syncScheduled = $isScheduled ? self::PENDING_SCHED_ORDER : self::NOT_PENDING_SCHED_ORDER;
                $kdsChangeStatusOnExp = '0';
                
                $taaOrder = strpos($orderType,'TAA') !== false;
                if($this->printMode == 'EXPEDICAO'){
                    if($toGo == 'S' || $taaOrder) $kdsChangeStatusOnExp = '1';
                }else if($this->printMode == 'INICIO'){
                    if($taaOrder) $kdsChangeStatusOnExp = '2';
                    else if($toGo == 'S') $kdsChangeStatusOnExp = '1';
                }
                
                if(empty($dscomanda)){
                    if($orderType == "MES_MOB" || $orderType == "CMD_MOB"){
                        $dscomanda = "PKR_$comandavenSeq";
                        $toGo = 'N';
                    }else{
                        $dscomanda = "DLV_$comandavenSeq";
                    }
                }else{
                    $dscomanda = str_pad($dscomanda, 10, 0, STR_PAD_LEFT);
                }
                
                if ($orderExists === false){
                    $returnComandaven  = $this->insertComandaVen($subsidiaryCode,$vendarestSeq,$comandavenSeq,$storeCode,$IDSTCOMANDA,$deliveryFee,$country,$state,$cityCode,$zipCode,$neighborhood, $fullStreetAddress, $addressLine2, $reference, $toGo, $orderToken, $syncScheduled, $scheduledDate, $kdsChangeStatusOnExp, $consumerDescription, $orderType, $cashBack, $orderDescription, $dscomanda, $closeByApi);
                    if ($returnComandaven['error']) {
                        $message = $returnComandaven['message'];
                        $response->addMessage(new Message($message, self::ERROR_MESSAGE));
                    }

                }
                // --- Save payments
                if(!empty($payments)){
                    $savePaymentsreturn = $this->savePayments($payments,$subsidiaryCode,$vendarestSeq,$clientCode,$consumerCode,(double)$billParams['changeValue']);
                    if($savePaymentsreturn['error']){
                        $message = $savePaymentsreturn['message'];
                        throw new \Exception($message);
                    }
                }
                // --- Save gift cards
                if(!empty($giftCards)){
                    $DTABERCAIX = $this->util->getPosDate($subsidiaryCode, $sellerPos);
                    $totalValue = $billParams['totalValue'] - $cashBack;
                    $giftCardsResponse = $this->saveGiftCards($giftCards, $subsidiaryCode, $vendarestSeq, $sellerPos, $DTABERCAIX, $totalValue);
                    if($giftCardsResponse['error']){
                        $message = $giftCardsResponse['message'];
                        throw new \Exception($message);
                    }
                }
                    
                // --- save items
                if(count($items) > 0)
                    $itemsSeq = $this->orderService->generateItemsSeq($subsidiaryCode);
                $orderCPF = $billParams['orderCPF'];
                $itemNum  = 0;
                $positionNumber = '01';  
                
                $toTravelData = array();
                $driveThruData = array();
                // if(!empty($toTravel)) $toTravelData = $this->getToTravelData();
                if(!empty($driveThru)) $driveThruData = $this->getDriveThruData();
                    
                if(!$taaOrder && !empty($pagerId)){
                    $pagerId = substr($pagerId, 0,4);
                    $pagerId = 'A'.$pagerId;
                }
                $sumValue = 0; 
                $totalDiscount = $orderDiscount;
                while($orderDiscount > 0){
                    $sumValue = 0;
                    foreach ($items as &$item) {
                        $this->shareDiscount($item, $orderDiscount, $productsValue, $remaingValue, $remaingDiscount, false);
                    }
                    unset($item);
                    $remaingValue = $productsValue;
                    foreach ($items as $item) {
                        $sumValue += $this->getDiscountValue($item);
                    }
                    $orderDiscount = $this->util->subtract($totalDiscount, $sumValue);
                    $remaingDiscount = $orderDiscount;
                    unset($item);
                }
                $quantItems = $this->orderService->getQuantItens($subsidiaryCode, $vendarestSeq, $comandavenSeq);
                foreach($items as &$item)
                    $this->saveItem($item, $subsidiaryCode, $vendarestSeq, $comandavenSeq, $itemsSeq, $positionNumber, $idcontroprod, $sellerPos, $storeCode, $sellerCode, $orderCPF, $itemNum, null, $toTravelData, $pagerId, $holdTime, $driveThruData, $orderType, false, $quantItems);
                unset($item);    

                if(empty($consumerCode) && !empty($giftCards)){
                    $this->orderService->updateConsumerVendarest($giftCards[0]['clientCode'], $giftCards[0]['consumerCode'], $vendarestSeq, $subsidiaryCode);
                }
                $elapsedTime = strtotime("now");
                if(($elapsedTime - $timeout) >= $now){
                    throw new \Exception("Tempo limite excedido.");
                }
                $this->util->commit();
                if(!empty($consumerDescription) && !empty($pagerId)) $pagerId.=" - $consumerDescription";
                $printItems = $this->formatItemsToPrint($items);
                if($toGo == 'S'){
                    $toGoString = 'PEDIDO PARA RETIRADA NA LOJA';
                }else{
                    $toGoString = 'PEDIDO PARA ENTREGA DELIVERY';
                }
                $this->printOrder($subsidiaryCode, $comandavenSeq, $storeCode, $printItems, $sellerCode, $table, $pagerId, $toGoString, $taaOrder);
                
                $response->addDataSet(
                    new DataSet(
                        '', 
                        array(
                            array(
                                'saleId' => $orderToken,
                                'saleCode' => $comandavenSeq
                            )
                        )
                    )
                );
            }else{
                $message = $sellerReturn['message'];
                $response->addMessage(new Message($message, self::ERROR_MESSAGE));
                $this->util->rollBack();
            }
        } catch (\Exception $e) {
            $this->util->rollBack();
            $message = 'Erro ao inserir comanda delivery. '.$e->getMessage();
            $this->util->saveLogOrder($message,'Error/'.date("Y/m/d"),$fileName);
            
            $response->addMessage(new Message($message, self::ERROR_MESSAGE));
        }
        
    }
    private function validateOrderType($orderType, $toTravel){
        if($orderType == 'DLV') $orderType = 'DLV_APP';
        else if($orderType == 'TAA'){
            if($toTravel) $orderType = 'TGO_TAA';
            else $orderType = 'BAL_TAA';
        }else if($orderType == 'MES'){
            $orderType = 'MES_MOB';
        }else if($orderType == 'CMD'){
            $orderType = 'CMD_MOB';
        }
        if($orderType === 'BGR_APP') $orderType = 'BAL_APP';
        if($orderType === 'BGR_TAA') $orderType = 'BAL_TAA';
        return $orderType;
    }

    public function formatItemsToPrint($items){
        $formatedItems = array();
        foreach ($items as $item) {
            $formatedItem = array();
            $childItems = array();
            if(!empty($item['childItems'])){
                $formatedItem['NMPRODPROMOCAO'] = $item['productName'];
                $formatedItem['IDTIPOCOMPPROD'] = '3';
                $formatedItem['ORDEMIMP'] = 0;
                $childItems = $this->formatItemsToPrint($item['childItems']);
            }else{
                $formatedItem['IDIMPPROMOCAO'] = 'N';
                $formatedItem['IDTIPOCOMPPROD'] = null;
            }

            $formatedItem['CDPRODUTO']      = $item['productCode'];
            $formatedItem['NRLUGARMESA']    = 1;
            $formatedItem['CDPRODUTO']      = $item['productCode'];
            $formatedItem['NMPRODUTO']      = $item['productName'];
            $formatedItem['QTPRODCOMVEN']   = $item['quantity'];
            $formatedItem['ATRASOPROD']     = null;
            $TXPRODCOMVEN = '';
            if(!empty($item['observations']))
                foreach ($item['observations'] as $obs) 
                    $TXPRODCOMVEN .= " {$obs['obsName']}\n   *  ";

            $formatedItem['TXPRODCOMVEN'] = substr($TXPRODCOMVEN, 0,-4);
            array_push($formatedItems, $formatedItem);
            if(!empty($childItems)) $formatedItems = array_merge($formatedItems,$childItems);
        }
        return $formatedItems;

    }
    
    public function isToGo($orderType){
        if(strpos($orderType,'TGO') !== false) return true;
        if(strpos($orderType,'BGR') !== false) return true;
        if(strpos($orderType,'MES') !== false) return true;
        if(strpos($orderType,'BAL') !== false) return true;
        if(strpos($orderType,'CMD') !== false) return true;
        if(strpos($orderType,'THR') !== false) return true;
        return false;
    }
    private function getToTravelData(){
        $toTravelObservation = $this->util->getToTravelObservation();
        return array(
            'obsGroupCode'  => $toTravelObservation['CDGRPOCOR'],
            'obsCode'       => $toTravelObservation['CDOCORR'],
            'obsName'       => $toTravelObservation['DSOCORR'],
            'price'         => 0,
            'quantity'      => 1
        );
    }
    
    public function getDriveThruData(){
        $driveThruObservation = $this->util->getDriveThruObservation();
        return array(
            'obsGroupCode'  => $driveThruObservation['CDGRPOCOR'],
            'obsCode'       => $driveThruObservation['CDOCORR'],
            'obsName'       => $driveThruObservation['DSOCORR'],
            'price'         => 0,
            'quantity'      => 1
        );
    }
    
    public function saveGiftCards($giftCards, $subsidiaryCode, $vendarestSeq, $sellerPos, $DTABERCAIX, $totalValue){
        $savedValue = $this->util->getPaymentValue($subsidiaryCode, $vendarestSeq);
        
        $this->validateConsumerType($giftCards);
        $paymentValue = bcsub($totalValue, $savedValue,2);
        $paymentCode = $this->util->getCreditPayment();
        if(count($giftCards) == 1){
            $giftCard = $giftCards[0];
            $clientCode = $giftCard['clientCode'];
            $consumerCode = $giftCard['consumerCode'];
            if($giftCard['value'] < $paymentValue)
                throw new \Exception("Valor insuficiente para realização do pagamento com cartão presente.");
        }else{
            $destinyCard = array_shift($giftCards);
            $clientCode = $destinyCard['clientCode'];
            $consumerCode = $destinyCard['consumerCode'];
            $availableValue = $destinyCard['value'];
            foreach ($giftCards as $giftCard) 
                $availableValue = bcadd($availableValue, $giftCard['value'],2);
            if($availableValue < $paymentValue) 
                throw new \Exception("Valor insuficiente para realização do pagamento com cartão presente.");
            $extratoCons = new \SimpleXMLElement('<ExtratoCons/>');
            $xmlFields = $this->getXMLFields();
            foreach ($giftCards as $giftCard) {
                $registro = $extratoCons->addChild('Registro');
                foreach ($xmlFields as $field) 
                    $registro->addChild($field);
                $registro->cdcliente    = $giftCard['clientCode'];
                $registro->cdconsumidor = $giftCard['consumerCode'];
                $registro->cdfamilisald = $giftCard['cdfamilisald'];
                $registro->dtmovextcons = date('d/m/Y H:i:s');
                $registro->nrseqmovext  = '001';
                $registro->cdtiporece   = '001';
                $registro->cdfilial     = $subsidiaryCode;
                $registro->cdcaixa      = $sellerPos;
                $registro->dsopeextcons = "Transf. Saldo:".$destinyCard['cardNumber']." - ". $destinyCard['clientCode'];
                $registro->vrmovextcons = $giftCard['value'];
                $registro->vrsaldconext = 0;
                $registro->idtpmovext   = 'D';
                $registro->idimpextrato = 'S';
                $detiny = $extratoCons->addChild('Registro');
                foreach ($xmlFields as $field) 
                    $detiny->addChild($field);
                $detiny->cdcliente    = $destinyCard['clientCode'];
                $detiny->cdconsumidor = $destinyCard['consumerCode'];
                $detiny->cdfamilisald = $destinyCard['cdfamilisald'];
                $detiny->dtmovextcons = date('d/m/Y H:i:s');
                $detiny->nrseqmovext  = '001';
                $detiny->cdtiporece   = '001';
                $detiny->cdfilial     = $subsidiaryCode;
                $detiny->cdcaixa      = $sellerPos;
                $detiny->dsopeextcons = "Transf. Saldo:".$giftCard['cardNumber']." - ". $giftCard['clientCode'];
                $detiny->vrmovextcons = $giftCard['value'];
                $detiny->vrsaldconext = 0;
                $detiny->idtpmovext   = 'C';
                $detiny->idimpextrato = 'S';
                sleep(1);
            }
            $xmlToSend = $extratoCons->asXML();
            $params = array(
                'P_XMLEXTCONS' => $xmlToSend    
            );  
            $this->util->callExtratocons($params, $subsidiaryCode, "InsereDados");
        }
        $movcaixadlvSeq = $this->orderService->generateMovcaixadlvSeq($subsidiaryCode, $vendarestSeq);
        $movementType = 'E';
        $returnMovcaixadlv = $this->insertMovCaixaDlv($subsidiaryCode, $vendarestSeq, $movcaixadlvSeq, $movementType, $clientCode, $consumerCode, $paymentCode, $paymentValue, null, 1);
        if($returnMovcaixadlv['error'])
            return $this->returnPattern(true,$returnMovcaixadlv['message']);
    }
    
    public function validateConsumerType($giftCards){
        $cards = $giftCards;
        foreach ($giftCards as &$giftCard)
            $giftCard['TIPOCONS'] = $this->util->getTipoCons($giftCard['clientCode'], $giftCard['consumerCode']);
        unset($giftCard);
        $consType = $giftCards[0]['TIPOCONS'];
        foreach ($giftCards as $giftCard) 
            if(!in_array($consType, $giftCard)) throw new \Exception("Os tipos de consumidor devem ser iguais."); 
        unset($giftCard);
    }
    
    public function getXMLFields(){
        return array("cdcliente","cdconsumidor","cdfamilisald","dtmovextcons","nrseqmovext","cdtiporece","cdfilial","cdcaixa","dsopeextcons","vrmovextcons","vrsaldconext","idtpmovext","nrdeposicons","nrseqmovcaixa","idimpextrato");
    }

    public function savePayments($payments,$subsidiaryCode,$vendarestSeq,$clientCode,$consumerCode, $change){
        $sequential = 1;
        foreach($payments as $key=>$payment){
            $paymentCode   = $payment['paymentCode'];
            $quantParcel   = isset($payment['quantParcel']) ? $payment['quantParcel'] : 1;
            $this->util->validatePaymentCode($paymentCode);
            $nsu           = isset($payment['nsu']) ? $payment['nsu'] : null;
            $paymentValue  = $this->util->removeDecimal($payment['value']);
            $paymentId     = $this->util->getPaymentId($paymentCode);
            if($paymentId === "9"){
                $returnVerify = $this->verifyDailyCreditLimit($paymentValue, $clientCode, $consumerCode);
                if($returnVerify['error']){
                    return $this->returnPattern(true,$returnVerify['message']);
                }
            }
            $movcaixadlvSeq = $this->orderService->generateMovcaixadlvSeq($subsidiaryCode, $vendarestSeq);
            $movementType      = 'E';
            $returnMovcaixadlv = $this->insertMovCaixaDlv($subsidiaryCode,$vendarestSeq,$movcaixadlvSeq,$movementType,$clientCode,$consumerCode,$paymentCode,$paymentValue, $nsu, $quantParcel);
            if($returnMovcaixadlv['error'])
                return $this->returnPattern(true,$returnMovcaixadlv['message']);
        }
        // --- save change
        if($change > 0){
            $paymentCode       = $this->util->getMoney();
            $movcaixadlvSeq = $this->orderService->generateMovcaixadlvSeq($subsidiaryCode, $vendarestSeq);
            $movementType      = 'S';
            $returnMovcaixadlv = $this->insertMovCaixaDlv($subsidiaryCode,$vendarestSeq,$movcaixadlvSeq,$movementType,$clientCode,$consumerCode,$paymentCode,$change, $nsu, 1);
            if($returnMovcaixadlv['error'])
                return $this->returnPattern(true,$returnMovcaixadlv['message']);
        }
        return $this->returnPattern(false,'');
    }

    public function divideItems(&$items, &$item){
        if(!empty($item['childItems'])){
            $childQuant = 0;
            foreach ($item['childItems'] as &$childItem) {
                $childQuant += $this->divideItems($item['childItems'], $childItem);
            }
            if(!empty($item['observations'])){
                foreach ($item['observations'] as $observation) {
                    if(!empty($item['price'])){
                        $childQuant += 1;   
                    }
                }
            }
            $item['NRLUGARMESA'] = 1;
            unset($childItem);
            if($item['quantity'] > 1){
                $fatherToPush = $item['quantity'];
                $item['quantity'] = 1;
                for($i = 1; $i < $fatherToPush; $i ++){
                    array_push($items, $item);
                }
            }
            return $childQuant;
        }else if($item['quantity']){
            $realQuantity = 1;
            $qtToSave = $item['quantity'];
            $item['quantity'] = 1;
            $item['NRLUGARMESA'] = 1;
            $item['IDIMPPROMOCAO'] = 'N';
            for($i = 1; $i < $qtToSave; $i ++){
                array_push($items, $item);
            }
            if(!empty($item['observations'])){
                foreach ($item['observations'] as $observation) {
                    if(!empty($observation['price'])){
                        $realQuantity += 1;
                    }
                }
            }
            return $realQuantity;
        }
    }

    public function shareDiscount(&$item, $orderDiscount , $productsValue, &$remaingValue, &$remaingDiscount, $isObs){
        if(!empty($item['observations'])){
            foreach ($item['observations'] as &$observation) {
                $this->shareDiscount($observation, $orderDiscount , $productsValue, $remaingValue, $remaingDiscount, true);
            }
            unset($observation);
        }
        if(!empty($item['childItems'])){
            foreach ($item['childItems'] as &$childItem) {
               $this->shareDiscount($childItem, $orderDiscount , $productsValue, $remaingValue, $remaingDiscount, false);
            }
            unset($childItem);
        }else{
            $productQuantity    = $item['quantity'];
            if($isObs){
                $productValue = !empty($item['price']) ? $item['price'] : 0;
                $item['discount'] = !empty($item['discount']) ? $item['discount'] : 0;
                $productDiscount = $item['discount'];
            }else{
                $productValue    = $item['unitValue'];
                $productDiscount = $item['discount'];
            }
            if($productValue != 0){
                /*echo  ("\n-----------------------------------------------------");
                echo  ("\nProduct Value ".$productValue);  
                  echo  ("\nProduct Discount ".$productDiscount);
                    echo  ("\nRemaing Value ".$remaingValue);   
                 echo  ("\nRemaing Discount ".$remaingDiscount); 
                 if($isObs){
                     echo  ("\n============================\nobsName: {$item['obsName']}\nPrice: {$item['price']}\nQuantity: {$item['quantity']}\nDiscount: {$item['discount']}");
                 }else{
                    echo  ("\n============================\nproductName: {$item['productName']}\nUnitValue: {$item['unitValue']}\nQuantity: {$item['quantity']}\nDiscount: {$item['discount']}");
                 }*/
                if(!isset($item['elegible']) || !empty($item['elegible'])){
                    if($this->util->removeDecimal($remaingValue) == $this->util->removeDecimal($productValue)){
                        if($this->util->removeDecimal($productValue) == $this->util->removeDecimal($remaingDiscount)){
                            $productDiscount = $remaingDiscount - 0.01; 
                           $remaingDiscount = 0.01;    
                        }else $productDiscount = $remaingDiscount;  
                    }else if($remaingDiscount > 0 && $remaingDiscount < 0.01){
                        $remaingDiscount = 0.01;   
                        $productDiscount = 0.01;
                    }else{
                        $productDiscount = $this->util->removeDecimal(((($this->util->subtract($productValue, $productDiscount)) * $productQuantity) *$orderDiscount) / $productsValue);
                    }    
                    $actualValue = $this->util->removeDecimal($productValue - $item['discount']); 
                    if($productDiscount === 0.00 &&  $actualValue > $this->util->removeDecimal($remaingDiscount)){
                        $productDiscount = $remaingDiscount;  
                    }else if($productDiscount === 0.00 && $actualValue < $this->util->removeDecimal($remaingDiscount)){
                        $productDiscount = $this->util->subtract($productValue, $item['discount']) - 0.01;   
                        // $remaingDiscount = $this->util->subtract($remaingDiscount, $productDiscount);    
                    }else if($productDiscount === 0.00 && $actualValue == $this->util->removeDecimal($remaingDiscount)){    
                        $productDiscount = $remaingDiscount - 0.01;
                        $remaingDiscount = 0.01;
                    }    
                    $remaingValue -= $productValue;  
                    // echo "\nProduct Value: $productValue";
                    if($productDiscount >= $productValue){    
                        $productDiscount = $productValue - 0.01;   
                        $remaingDiscount = $this->util->subtract($remaingDiscount, $productDiscount);
                    }else{    
                        // echo "\n_______________Product Discount: $productDiscount Remaing Discount: $remaingDiscount _______________";
                        $remaingDiscount =  $this->util->subtract($this->util->removeDecimal($remaingDiscount), $this->util->removeDecimal($productDiscount)) * $item['quantity'];
                        // echo "\n######Remaing Discount: $remaingDiscount######";
                    }    
                    $item['discount'] += $this->util->removeDecimal($productDiscount); 
                    $item['elegible'] = $this->util->removeDecimal($productValue - 0.01) > $item['discount'];
                      /*if($isObs){
                             echo  ("\n============================\nobsName: {$item['obsName']}\nPrice: {$item['price']}\nQuantity: {$item['quantity']}\nDiscount: {$item['discount']}\n============================");
                         }else{
                            echo  ("\n============================\nproductName: {$item['productName']}\nUnitValue: {$item['unitValue']}\nQuantity: {$item['quantity']}\nDiscount: {$item['discount']}\n============================");
                         }
                    echo  ("\nProduct Value ".$productValue);  
                  echo  ("\nProduct Discount ".$productDiscount);
                    echo  ("\nRemaing Value ".$remaingValue);   
                 echo  ("\nRemaing Discount ".$remaingDiscount);    
                echo  ("\n-----------------------------------------------------"); */   
                }
            }
            // var_dump($item);
          }
    }

    public function getDiscountValue($item){
        $discount = 0;
        // echo "\nDesconto inicial: $discount Item: {$item['productName']}";
        if(!empty($item['observations'])){
            $observations  = $item['observations'];
            foreach ($observations as $observation) {
                $discount = $this->util->add($discount, $observation['discount']);
                // echo "\nValor: $discount Observacao: {$observation['obsName']}";
            }
        }
        if(!empty($item['childItems'])){
            $childItems  = $item['childItems'];
            foreach ($childItems as $childItem){
                // var_dump("ANTES DA DOIDERA: $discount");
                // echo "\n";
                $discount = $this->util->add($discount, $this->getDiscountValue($childItem));
                // var_dump($childItem);
                // var_dump("DOIDER: $discount");
                
            }
        }else{
            $discount = $this->util->add($discount, $item['discount']);
            // echo "\nItem sem filho: {$item['productName']} Desconto: $discount";
        }
        /*if(isset($item['productName'])){
            echo "\nDesconto: $discount\nItem: {$item['productName']}";
        }else{
            echo "\nDesconto: $discount\nObs Item: {$item['obsName']}";
        }
        echo "\n";*/
        return $discount;
    }
    
    public function saveItem(&$item, $subsidiaryCode, $vendarestSeq, $comandavenSeq, $itemsSeq, $positionNumber, $idcontroprod, $sellerPos, $storeCode, $sellerCode, $orderCPF, &$itemNum, $comboProductCode = null, $toTravelData, $pagerId = null, $holdTime, $driveThruData, $orderType, $toTravel, &$quantItems){
        if(!empty($item['childItems'])){
            if(!empty($item['note'])) $item['childItems'][0]['note'] .= '; '.$item['note'];
            $childItems       = $item['childItems'];
            $comboProductCode = $item['productCode'];
            $comboQuantity    = $item['quantity'];
            $toTravel = !empty($item['toTravel']) ? $item['toTravel'] : null;
            for($save = 0; $save < $comboQuantity; $save++){
                $itemNum++;
                foreach($childItems as &$childItem)
                    $this->saveItem($childItem, $subsidiaryCode, $vendarestSeq, $comandavenSeq, $itemsSeq, $positionNumber, $idcontroprod, $sellerPos, $storeCode, $sellerCode, $orderCPF, $itemNum, $comboProductCode, $toTravelData, $pagerId, $holdTime, $driveThruData, $orderType, $toTravel, $quantItems);    
            }
        }else{
            
            $note = !empty($item['note']) ? $item['note'] : null;
            $realProductQuantity  = $item['quantity'];
            $item['observations'] = $item['observations'] === null ? array() : $item['observations'];
            $toTravel = isset($item['toTravel']) && $item['toTravel'] !== null ? $item['toTravel'] : $toTravel;
            $idorigemvenda = null;
            if(!empty($toTravel) && substr($orderType, 0, 3) !== 'TGO' && substr($orderType, 0, 3) !== 'DLV' && substr($orderType, 0, 3) !== 'THR'){
                $idorigemvenda = 'TGO_'.substr($orderType, 0, 3);
                // $toTravelData = $this->getToTravelData();
            }
            /*if(!empty($toTravelData)){
                $alreadyHasToTravel = false;
                foreach($item['observations'] as $observation){
                    if($observation['obsCode'] === $toTravelData['obsCode'] && $observation['obsGroupCode'] === $toTravelData['obsGroupCode'])
                        $alreadyHasToTravel = true;
                }
                if(!$alreadyHasToTravel)
                    array_push($item['observations'],$toTravelData);
            } */
            if(!empty($driveThruData)){
                $alreadyHasDriveThru = false;
                foreach($item['observations'] as $observation){
                    if($observation['obsCode'] === $driveThruData['obsCode'] && $observation['obsGroupCode'] === $driveThruData['obsGroupCode'])
                        $alreadyHasDriveThru = true;
                }
                if(!$alreadyHasDriveThru)
                    array_push($item['observations'],$driveThruData);
            } 
            for($save = 0; $save < $realProductQuantity; $save++){
                if($realProductQuantity < 1)
                    $productQuantity = $realProductQuantity;
                else
                    $productQuantity = 1;
                $productCode        = $item['productCode'];
                $productValue       = $item['unitValue'];
                $productDiscount    = $item['discount'];
                $productAcr         = 0;
                $itcomandavenSeq    = $quantItems;
                $quantItems++;
                $productDescription = '';
                if(!empty($item['observations']) || !empty($note))
                    $productDescription = $this->setProductDescription($note, $item['observations']);

                $idstprcomven = $this->util->getTheThing($subsidiaryCode, $storeCode, $productCode, $idcontroprod);
                $returnItcomandaven = $this->insertItComandaVen($subsidiaryCode,$vendarestSeq,$comandavenSeq,$itcomandavenSeq,$productCode,$productQuantity,$productValue,$productDescription,$idstprcomven,$productDiscount,$storeCode,$productAcr,$itemNum,$itemsSeq,$sellerPos,$comboProductCode,$sellerCode,$orderCPF, $positionNumber,$pagerId, $holdTime, $idorigemvenda, $note);

                if($returnItcomandaven['error']){
                    throw new \Exception($returnItcomandaven['message']);
                }
                foreach($item['observations'] as $observation){
                    $obsCode      = $observation['obsCode'];
                    $obsGroupCode = $observation['obsGroupCode'];
                    $obsValue     = $observation['price'];
                    $obsQuantity  = $observation['quantity'];
                    $obsDiscount  = !empty($observation['discount']) ? $observation['discount']: 0;
                    if(!$obsCode)
                        throw new \Exception("Código de observação inválido!");
                    if(!$obsGroupCode)
                        throw new \Exception("Código de grupo de observação inválido");
                    $returnObsitcomandaven = $this->insertObsItComandaVen($subsidiaryCode,$vendarestSeq,$comandavenSeq, $itcomandavenSeq, $obsGroupCode, $obsCode);
                    $observationProduct    = $this->util->getObservationInfo($obsGroupCode, $obsCode);
                    if(!empty($observationProduct)){
                        $idstprcomven       = $this->util->getTheThing($subsidiaryCode, $storeCode, $observationProduct['CDPRODUTO'], $idcontroprod);
                        $itcomandavenSeqObs = $quantItems;
                        $quantItems++;
                        $returnItcomandaven = $this->insertItComandaVen($subsidiaryCode,$vendarestSeq,$comandavenSeq,$itcomandavenSeqObs,$observationProduct['CDPRODUTO'],$obsQuantity,$obsValue,'',$idstprcomven, $obsDiscount, $storeCode,0,$itemNum,$itemsSeq,$sellerPos,null,$sellerCode,$orderCPF, $positionNumber,$pagerId, $holdTime, $idorigemvenda, null);
                        if($returnItcomandaven['error']){
                            throw new \Exception($returnItcomandaven['message']);
                        }
                    }                    
                    if($returnObsitcomandaven['error']){
                       throw new \Exception($returnObsitcomandaven['message']);
                    }
                }
            }    
        }
    }

    private function setProductDescription($note, $observations){
        if(!empty($note)) $txprodcomven = ' '.$note;
        else $txprodcomven = ' ';
        
        foreach($observations as $observation){ 
            $obsName = $observation['obsName'];
            if($txprodcomven == null){
                $txprodcomven =  $obsName;
            }else{
                $txprodcomven .= '; '.$obsName; 
            }
        }
        return $txprodcomven;
    }

    public function insertVendaRest($subsidiaryCode,$vendarestSeq,$storeCode,$sellerCode,$operatorCode,$clientCode,$consumerCode, $standardTable){
        
        $params = array(
            'CDFILIAL'     => $subsidiaryCode,
            'NRVENDAREST'  => $vendarestSeq,
            'CDLOJA'       => $storeCode,
            'NRMESA'       => $standardTable,
            'CDVENDEDOR'   => $sellerCode,
            'CDOPERADOR'   => $operatorCode,
            'NRPESMESAVEN' => '1',
            'CDCLIENTE'    => $clientCode,
            'CDCONSUMIDOR' => $consumerCode,
            'IDPEDIDOPAGO' => 'S'
        );

        return $this->orderService->insertVendaRest($params);
    }

    protected function insertComandaVen($subsidiaryCode,$vendarestSeq,$comandavenSeq,$storeCode,$IDSTCOMANDA,$deliveryFee,$country,$state,$cityCode,$zipCode,$neighborhood, $fullStreetAddress, $addressLine2, $reference, $toGo, $orderToken, $syncScheduled, $scheduledDate, $kdsChangeStatusOnExp, $consumerDescription, $orderType, $cashBack, $orderDescription, $dscomanda, $closeByApi){
        $params = array(
            'CDFILIAL'        => $subsidiaryCode,
            'NRVENDAREST'     => $vendarestSeq,
            'NRCOMANDA'       => $comandavenSeq,
            'CDLOJA'          => $storeCode,
            'IDSTCOMANDA'     => $IDSTCOMANDA,
            'DSCOMANDA'       => $dscomanda,
            'SGSEXOCON'       => 'M',
            'TXMOTIVCANCE'    => " ",
            'VRACRCOMANDA'    => $deliveryFee,
            'CDPAIS'          => $country,
            'SGESTADO'        => $state,
            'CDMUNICIPIO'     => $cityCode,
            'NRCEPCONSCOMAND' => $zipCode,
            'CDBAIRRO'        => null,
            'DSBAIRRO'        => $neighborhood,
            'DSENDECONSCOMAN' => $fullStreetAddress,
            'DSCOMPLENDCOCOM' => $addressLine2,
            'DSREFENDCONSCOM' => $reference,
            'IDORGCMDVENDA'   => $orderType,
            'IDRETBALLOJA'    => $toGo,
            'NRCOMANDAEXT'    => $orderToken,
            'IDSINCAGENDA'    => $syncScheduled,
            'DTHRAGENDADA'    => $scheduledDate,
            'IDIMPCUPEXP'     => $kdsChangeStatusOnExp,
            'DSCONSUMIDOR'    => $consumerDescription,
            'VRDESCFID'       => $cashBack,
            'DSOBSCOMANDA'    => $orderDescription,
            'IDFECHAMENTOAPI' => $closeByApi
        );
        return $this->orderService->insertComandaven($params);
    }

    public function insertMovCaixaDlv($subsidiaryCode,$vendarestSeq,$movcaixadlvSeq,$movementType,$clientCode,$consumerCode,$paymentCode,$paymentValue, $nsu, $quantParcel){
        
        $params = array(
            'CDFILIAL'        => $subsidiaryCode,
            'NRVENDAREST'     => $vendarestSeq,
            'NRSEQMOVDLV'     => $movcaixadlvSeq,
            'IDTIPOMOVIVEDLV' => $movementType,
            'CDCLIENTE'       => $clientCode,
            'CDCONSUMIDOR'    => $consumerCode,
            'CDTIPORECE'      => $paymentCode,
            'VRMOVIVENDDLV'   => $paymentValue,
            'CDNSUTEFDLV'     => $nsu,
            'QTPARCRECEB'     => $quantParcel
        );

        return $this->orderService->insertMovCaixaDlv($params);
    }

    protected function insertItComandaVen($subsidiaryCode,$vendarestSeq,$comandavenSeq,$itcomandavenSeq,$productCode,$productQuantity,$productValue,$productDescription,$idstprcomven,$productDiscount,$storeCode,$productAcr,$itemNum,$itemsSeq,$sellerPos,$comboProductCode,$sellerCode,$orderCPF, $positionNumber,$pagerId, $holdTime, $idorigemvenda, $note){
        if($comboProductCode == null){
            $prodSeq = null;
        }else{
            $prodSeq = str_pad($itemNum,3,'0',STR_PAD_LEFT); 
        }
        $params  = array(
            'CDFILIAL'       => $subsidiaryCode,
            'NRVENDAREST'    => $vendarestSeq,
            'NRCOMANDA'      => $comandavenSeq,
            'NRPRODCOMVEN'   => str_pad($itcomandavenSeq, 6, '0', STR_PAD_LEFT),
            'CDPRODUTO'      => $productCode,
            'QTPRODCOMVEN'   => $productQuantity,
            'VRPRECCOMVEN'   => $productValue,
            'TXPRODCOMVEN'   => $productDescription,
            'IDSTPRCOMVEN'   => $idstprcomven,
            'VRDESCCOMVEN'   => $productDiscount,
            'NRLUGARMESA'    => $positionNumber,
            'IDPRODIMPFIS'   => 'N',
            'CDLOJA'         => $storeCode,
            'VRACRCOMVEN'    => $productAcr,
            'NRSEQPRODCOM'   => $prodSeq,
            'NRSEQPRODCUP'   => $itemsSeq,
            'VRPRECCLCOMVEN' => 0,
            'CDCAIXACOLETOR' => $sellerPos,
            'CDPRODPROMOCAO' => $comboProductCode,
            'CDVENDEDOR'     => $sellerCode,
            'NRINSCRCONS'    => $orderCPF,
            'CDSENHAPED'     => $pagerId,
            'NRATRAPRODCOVE' => $holdTime,
            'IDORIGEMVENDA'  => $idorigemvenda,
            'DSOBSPEDDIGCMD' => $note
        );
        return $this->orderService->insertItComandaVen($params);
    }

    public function getIdcontroprod($storeCode,$subsidiaryCode){
        return $this->util->getIdcontroprod($storeCode,$subsidiaryCode);
    }

    protected function insertObsItComandaVen($subsidiaryCode,$vendarestSeq,$comandavenSeq, $itcomandavenSeq, $obsGroupCode, $obsCode){
        
        $params = array(
            'CDFILIAL'     => $subsidiaryCode,
            'NRVENDAREST'  => $vendarestSeq,
            'NRCOMANDA'    => $comandavenSeq,
            'NRPRODCOMVEN' => str_pad($itcomandavenSeq, 6, '0', STR_PAD_LEFT),
            'CDGRPOCOR'    => $obsGroupCode,
            'CDOCORR'      => $obsCode    
        );
        return $this->orderService->insertObsItComandaVen($params);
    }

    public function isValid(Request $request, Response $response){
        $inputJSON = file_get_contents('php://input');
        $this->util->saveLog($inputJSON,'Order','isValid');
        $input = json_decode($inputJSON, true);
        try {
            if (!isset($input)) {
                throw new \Exception('Não foram recebidos parâmetros');
            }
            $data   = array(); // dados a serem retornados

            $params = $input;
             // --- Order
            $order              = $params['order'];
            $clientCode         = $order['clientCode'];
            $consumerCode       = $order['consumerCode'];
            $balanceGroupCode   = $order['balanceGroup'];
            $subsidiaryCode     = $order['subsidiaryCode'];
            $qualidade          = isset($params['Qualidade']) ? $params['Qualidade'] : false;

            if(!empty($clientCode) && !empty($consumerCode)){
                // verifica se existe cliente cadastrado com essas informações
                if(!$this->util->consumerExists($clientCode, $consumerCode)){
                    throw new \Exception('Consumidor inexistente.');
                }    
            }else{
                $defaultClient = $this->util->getDefaultClient($subsidiaryCode);
                
                if(!empty($defaultClient))
                    $clientCode = $defaultClient;
                else 
                    throw new \Exception('Cliente padrão inexistente.');
            }

            $storeCode      = $order['storeCode'];
            $items          = $order['items'];
            $products       = array();
            $obsProducts    = array();

            $totalValue = 0;

            foreach($items as $item){
                $products[$item['productCode']] =  $item['productName'];
                if(!empty($item['childItems'])){
                    $childItems = $item['childItems'];
                    foreach($childItems as $childItem){
                        $totalValue =+ $this->util->float(($childItem['unitValue'] * $childItem['quantity']) - $childItem['discount']);
                        $products[$childItem['productCode']] =  $childItem['productName'];
                        if(!empty($childItem['observations'])){
                            $observations = $childItem['observations']; 
                            foreach($observations as $observation){
                                if(isset($observation['price'])){
                                    $totalValue =+ $this->util->float($observation['price'] * $observation['quantity']);
                                    $obsGroupCode = $observation['obsGroupCode'];
                                    $obsCode      = $observation['obsCode'];
                                    $obsProdCode  = $this->util->getObservationAsProduct($obsGroupCode, $obsCode);
                                    if($obsProdCode !== null){
                                        $obsProducts[$obsProdCode] = $observation['obsName'];
                                    }
                                }
                            }
                        }
                    }
                }else{
                    $totalValue =+ $this->util->float(($item['unitValue'] * $item['quantity']) - $item['discount']);
                }
                if(!empty($item['observations'])){
                    $observations = $item['observations']; 
                    foreach($observations as $observation){
                        if(isset($observation['price'])){
                            $totalValue =+ $this->util->float($observation['price'] * $observation['quantity']);
                            $obsGroupCode = $observation['obsGroupCode'];
                            $obsCode      = $observation['obsCode'];
                            $obsProdCode  = $this->util->getObservationAsProduct($obsGroupCode, $obsCode);
                            if($obsProdCode !== null){
                                $obsProducts[$obsProdCode] = $observation['obsName'];
                            }
                        }
                    }
                }
            }
            $data = array();
            foreach($products as $productCode=>$productName){
                if(!$this->util->productIsAvailable($subsidiaryCode, $storeCode, $productCode)){
                    array_push($data,$productName);
                }
            }

            foreach($obsProducts as $productCode=>$productName){
                if(!$this->util->productIsAvailable($subsidiaryCode, $storeCode, $productCode)){
                    array_push($data,$productName);
                }
            }
            if(!empty($data)){
                $message = 'Produtos indisponíveis';
                $response->addDataSet(new DataSet('', $data));
                $response->addMessage(new Message($message, self::ERROR_MESSAGE));
            }

            if(!empty($clientCode) && !empty($consumerCode)){
                $balanceValue = $this->util->getBalanceValue($clientCode, $consumerCode, $balanceGroupCode);
                $consumerBalanceInfo = $this->util->getConsumerBalanceInfo($clientCode, $consumerCode);
                $belowBalanceLimit = false;
                if(!empty($consumerBalanceInfo['VRLIMCREDCONS'])){
                    $belowBalanceLimit = ($this->util->float($totalValue) - $this->util->float($balanceValue)) < $consumerBalanceInfo['VRLIMCREDCONS']; 
                }

                if($balanceValue !== null && $consumerBalanceInfo['IDVERSALDCON'] == 'S' && $qualidade){
                    if($this->util->float($totalValue) > $this->util->float($balanceValue) && $belowBalanceLimit){
                        $message = 'Saldo insuficiente';
                        $response->addDataSet(new DataSet('', $data));
                        $response->addMessage(new Message($message, self::ERROR_MESSAGE));
                    }
                }
            }
        }catch(\Exception $e){
            $message = 'Erro ao validar venda. Erro: '.$e->getMessage();
            $response->addMessage(new Message($message, self::ERROR_MESSAGE));
        }
    }
    
    public function validateTableValue(Request $request, Response $response){
        $inputJSON = file_get_contents('php://input');
        //$this->util->saveLog($inputJSON,'Order','validateTableValue');
        $input = json_decode($inputJSON, true);
        $this->util->beginTransaction();
        try {
            if (!isset($input)) {
                throw new \Exception('Não foram recebidos parâmetros');
            }
            
            $data   = array(); // dados a serem retornados

            $params         = $input;
            $checkinId      = $params['checkinId'];
            $usesServiceTax = $params['usesServiceTax'];
            $paymentValue   = $this->util->removeDecimal($params['paymentValue']);
            $paymentCode    = $params['paymentCode'];
            $positions      = $params['positions'];
            $currentValue   = 0;
            /*
             * Verifica se o checkinId é válido
             */
            $validCheckinId = $this->orderService->isValidCheckinId($checkinId);

            if($validCheckinId['error']){
                throw new \Exception($validCheckinId['message']);
            }

            /*
             * Obtém CDFILIAL E NRVENDAREST partindo de um IDCHECKIN
             */

            $resultCheckinMesPos = $this->getTableInfoByCheckin($checkinId);
            /* $resultCheckinMesPos = $this->orderService->getResultCheckinMesPos($checkinId);
            if($resultCheckinMesPos['error']){
                throw new \Exception($resultCheckinMesPos['message']);
            }
            $resultCheckinMesPos = $resultCheckinMesPos['data'];
            */
            $CDFILIAL       = $resultCheckinMesPos['CDFILIAL'];
            $NRVENDAREST    = $resultCheckinMesPos['NRVENDAREST']; 
            $NRCOMANDA      = $resultCheckinMesPos['NRCOMANDA'];
            $CDLOJA         = $resultCheckinMesPos['CDLOJA'];
            $CDCLIENTE      = $resultCheckinMesPos['CDCLIENTE'];
            $CDCONSUMIDOR   = $resultCheckinMesPos['CDCONSUMIDOR'];
            /*
            * Obtém NRMESA
            */
            $resultGetNrmesa = $this->orderService->getNrmesa($CDFILIAL, $NRVENDAREST);
            if($resultGetNrmesa['error']){
                throw new \Exception($resultGetNrmesa['message']);
            }

            $paymentId = $this->util->getPaymentId($paymentCode);
            if($paymentId === "9"){
                $returnVerify = $this->verifyDailyCreditLimit($paymentValue, $CDCLIENTE, $CDCONSUMIDOR);
                if($returnVerify['error']){
                    throw new \Exception($returnVerify['message']);
                }
            }

            $resultGetNrmesa = $resultGetNrmesa['data'];
            $NRMESA = $resultGetNrmesa['NRMESA'];
            foreach($positions as $position){
                
                $positionNumber = $position['positionNumber'];
                $productsValue = 0;
                $orderItems = $this->util->getPositionItems($CDFILIAL, $NRVENDAREST, $NRCOMANDA, $positionNumber);
                foreach($orderItems as $orderItem){
                    $priceDetails = $this->getConsumerItemPrice($CDFILIAL,$CDCLIENTE,$orderItem['CDPRODUTO'],$CDLOJA,$CDCONSUMIDOR);
                    $discount     = $this->getNewDiscount($priceDetails, $orderItem);
                    $newUnitPrice = !empty($priceDetails['newPrice']) ? $priceDetails['newPrice'] : $orderItem['VRPRECCOMVEN'];
                    if($newUnitPrice - $discount < 0.01)
                        $productsValue += 0.01;
                    else
                        $productsValue += $newUnitPrice - $discount;
                    $productsValue = bcdiv($productsValue, 1, 2);
                }
                $currentValue   += $productsValue;
            } 
            $serviceTax = 0;
            if($usesServiceTax){
                $storeServiceTax  = $this->util->getStoreServiceTax($CDFILIAL,$CDLOJA); 
                $serviceTax       = $this->setServiceTax($storeServiceTax, $paymentValue);
            }

            $serviceTax   = $this->util->removeDecimal($serviceTax);
            $currentValue = $this->util->removeDecimal($currentValue);
            $currentValue = $this->util->removeDecimal($serviceTax + $currentValue);
            $paymentValue = $this->util->removeDecimal($paymentValue);
            $data = array();
            $isValid = $currentValue == $paymentValue;
            $responseObject = array(
                'isValid' => $isValid
            );
            array_push($data, $responseObject);
            $response->addDataSet(new DataSet('', $data));
            $this->util->commit();
        }catch(\Exception $e){
            $this->util->rollBack();
            $message = 'Erro ao validar pagamento da mesa. Erro: '.$e->getMessage();
            $response->addMessage(new Message($message, self::ERROR_MESSAGE));
        }
    }

    private function setServiceTax($storeServiceTax, $paymentValue){
        $storeRequiresTax = $storeServiceTax['IDCOMISVENDA'] == 'S';
        $servTaxAsProduct = $storeServiceTax['IDTRATTAXASERV'] == 'P';
        $servTaxProduct   = $storeServiceTax['CDPRODTAXASERV'];
        $servTaxValue     = $storeServiceTax['VRCOMISVENDA'];
        if(($storeRequiresTax) && (!empty($servTaxValue)) && (($servTaxAsProduct && !empty($servTaxProduct)) || (!$servTaxAsProduct))){
           return $paymentValue/$servTaxValue;
        }
        return 0;
    }

    public function closeTableBill(Request $request, Response $response) {
        $inputJSON = file_get_contents('php://input');
        $this->util->saveLog($inputJSON,'Order','closeTableBill');
        $params = json_decode($inputJSON, true);
        $this->util->beginTransaction();
        try {
            if(!isset($params)) {
                throw new \Exception('Não foram recebidos parâmetros');
            }

            $checkinId      = $params['checkinId'];
            $saveServiceTax = $params['usesServiceTax']; 
            /*
             * Verifica se o checkinId é válido
             */
            $validCheckinId = $this->orderService->isValidCheckinId($checkinId);

            if($validCheckinId['error']){
                throw new \Exception($validCheckinId['message']);
            }   
            /*
             * Obtém CDFILIAL E NRVENDAREST partindo de um IDCHECKIN
             */
            $resultCheckinMesPos = $this->getTableInfoByCheckin($checkinId);

            $CDFILIAL       = $resultCheckinMesPos['CDFILIAL'];
            $NRVENDAREST    = $resultCheckinMesPos['NRVENDAREST']; 
            $CDLOJA         = $resultCheckinMesPos['CDLOJA'];
            $CDCLIENTE      = $resultCheckinMesPos['CDCLIENTE'];
            $CDCONSUMIDOR   = $resultCheckinMesPos['CDCONSUMIDOR'];
            $idcontroprod   = $this->getIdcontroprod($CDLOJA,$CDFILIAL);
            if($saveServiceTax){
                $storeServiceTax = $this->util->getStoreServiceTax($CDFILIAL,$CDLOJA);
            }else{
                $storeServiceTax = array();
            }
            /*
             * Obtém CDVENDEDOR, CDCAIXA E CDLOJA partindo de um CDFILIAL
             */
            $resultParavendAndVendedor = $this->orderService->getResultParavendAndVendedor($CDFILIAL);                
            if($resultParavendAndVendedor['error']){
                throw new \Exception($resultParavendAndVendedor['message']);
            }

            $resultParavendAndVendedor = $resultParavendAndVendedor['data'];

            $CDVENDEDOR = $resultParavendAndVendedor['CDVENDEDOR'];
            $CDCAIXA    = $resultParavendAndVendedor['CDCAIXA'];
   
            /*
             * Obtém NRCOMANDA
             */

            $resultGetNrcomanda = $this->orderService->getNrcomanda($NRVENDAREST, $CDFILIAL);
            if($resultGetNrcomanda['error']){
                throw new \Exception($resultGetNrcomanda['message']);
            }

            $resultGetNrcomanda = $resultGetNrcomanda['data'];
               
            $NRCOMANDA = $resultGetNrcomanda['NRCOMANDA'];
                
            /*
            * Obtém NRMESA
            */
            $resultGetNrmesa = $this->orderService->getNrmesa($CDFILIAL, $NRVENDAREST);
            if($resultGetNrmesa['error']){
                throw new \Exception($resultGetNrmesa['message']);
            }

            $resultGetNrmesa = $resultGetNrmesa['data'];
            $NRMESA = $resultGetNrmesa['NRMESA'];
            $positions = $params['positions'];
            

            foreach($positions as $positionToClose){
                $return = $this->getProductsToRecalculate($CDFILIAL, $NRMESA, $positionToClose['positionNumber'], $CDLOJA);

                if(!$return['error']){
                    $productsToRecalculate = $return['data'];
                    $productsData = $this->recalculateProductsPrice($productsToRecalculate,$CDFILIAL, $NRMESA, $positionToClose['positionNumber'], $CDLOJA, $CDCLIENTE, $CDCONSUMIDOR, true);
                }

                $return = $this->getCombosToRecalculate($CDFILIAL, $NRMESA, $positionToClose['positionNumber'], $CDLOJA);
                if(!$return['error']){
                    $combosToRecalulate = $return['data'];
                    $combosData = $this->recalculateCombos($combosToRecalulate, $CDFILIAL, $CDLOJA, $CDCLIENTE, $CDCONSUMIDOR, true);
                }
                $positionNumber = $positionToClose['positionNumber'];
                if(isset($positionToClose['positionCPF']))
                    $positionCPF    = $positionToClose['positionCPF'];
                else
                    $positionCPF  = null;
                
                $position       = $positionNumber;
                $payments       = $positionToClose['payments'];
                $paymentsValue  = $this->util->removeDecimal($positionToClose['paymentsValue']);
                /*
                 * Altera o valor de positionNumber para T 
                 *      --> O status da mesa é alterado posteriormente
                 */

                if(empty($positionNumber)) { 
                    $position = 'T';   
                }else{
                    $lastPosition = $this->util->getLastPosition($CDFILIAL, $NRVENDAREST, $NRCOMANDA, $positionNumber);
                    if($lastPosition){
                        $position = 'T';
                    } 
                }                
                
                if($position == 'T')
                    $this->util->updateVendaRestConsumer($CDFILIAL, $NRVENDAREST, $CDCLIENTE, $CDCONSUMIDOR);

                foreach ($payments as $payment) {
                    $paymentCode    = $payment['paymentCode'];
                    $paymentValue   = $payment['value'];

                    /*
                     * Gera NRSEQMOVMOB utiilizando um novo código
                     */

                    $NRSEQMOVMOB = $this->orderService->generateMovcaixamobSeq($CDFILIAL);
 
                    $NRLUGARMESA    = $position;
                    $CDTIPORECE     = $paymentCode;
                    $VRMOV          = $paymentValue;
                    $IDTIPMOV       = 'E';
                    $DSBANDEIRA     = null;
                    $IDTPTEF        = null;
                    $NRSEQVENDA     = null;
                    $IDUTILTXSERV   = $saveServiceTax ? 'S' : 'N';
                    
                    /*
                     * Insere o registro na tabela MOVCAIXAMOB
                     */
                    $responseInsertion = $this->orderService->insertMovcaixamob($NRSEQMOVMOB, $CDFILIAL, $CDCAIXA, $CDVENDEDOR, $NRVENDAREST, $NRCOMANDA, $NRSEQVENDA, $NRMESA, $NRLUGARMESA, $CDTIPORECE, $IDTIPMOV, $VRMOV, $DSBANDEIRA, $IDTPTEF, $IDUTILTXSERV, $CDCLIENTE, $CDCONSUMIDOR);
                    if($responseInsertion['error']){
                        throw new \Exception($responseInsertion['message']);
                    }
                }
                /*
                 * Alterar o status da mesa para 'Conta Solicitada'
                 * $IDSTMESAAUX: 'S'
                */
                if($position ==  'T') {
                    $IDSTMESAAUX = self::PAID;
                    $responseStatusMesa = $this->orderService->changeTableStatus($IDSTMESAAUX, $CDFILIAL, $CDLOJA, $NRMESA);
                    if($responseStatusMesa['error']) {
                        throw new \Exception($responseStatusMesa['message']);
                    }
                }else{
                    $IDSTLUGARMESA = self::PAID;
                    $responseStatusPositions = $this->orderService->changePositionStatus($IDSTLUGARMESA, $CDFILIAL, $NRVENDAREST, $NRCOMANDA, $positionNumber);
                    if($responseStatusPositions['error']) {
                        throw new \Exception($responseStatusPositions['message']);
                    }
                }

                $responseCpfPosition = $this->orderService->changePositionCpf($CDFILIAL, $NRVENDAREST, $NRCOMANDA, $positionNumber, $positionCPF);
                if($responseCpfPosition['error']) {
                    throw new \Exception($responseCpfPosition['message']);
                }
                $serviceTax = 0;
                if($saveServiceTax){
                    $serviceTax = $this->setServiceTax($storeServiceTax,$paymentsValue);
                    /*$usesServiceTax   = $storeServiceTax['IDCOMISVENDA'] == 'S';
                    $servTaxAsProduct = $storeServiceTax['IDTRATTAXASERV'] == 'P';
                    $servTaxProduct   = $storeServiceTax['CDPRODTAXASERV'];
                    $servTaxValue     = $storeServiceTax['VRCOMISVENDA'];
                    if(($usesServiceTax) && (!empty($servTaxValue)) && (($servTaxAsProduct && !empty($servTaxProduct)) || (!$servTaxAsProduct))){
                        $serviceTax   = $paymentsValue/$servTaxValue;
                    }*/
                }
                if($position == 'T'){
                    $this->util->updateBillServiceTax($CDFILIAL, $NRVENDAREST, $NRCOMANDA, $IDUTILTXSERV, $this->util->float($serviceTax));
                }
            }
            $this->util->commit();
        } catch (\Exception $e) {
            $this->util->rollBack();
            $message = 'Erro ao pagar posições. Erro: '.$e->getMessage();
            $response->addMessage(new Message($message, self::ERROR_MESSAGE));
        }
    }

    public function lockTable(Request $request, Response $response){
        $inputJSON = file_get_contents('php://input');
        $this->util->saveLog($inputJSON,'Order','lockTable');
        $input = json_decode($inputJSON, true);
        $this->util->beginTransaction();
        try {
            if (!isset($input)) {
                throw new \Exception('Não foram recebidos parâmetros');
            }
            
            $data   = array(); // dados a serem retornados

            $params    = $input;
            $checkinId = $params['checkinId'];
            $lockDate  = date('d/m/Y H:i:s');

            $this->changeTableStatus($checkinId, self::REQUEST_ACCOUNT, $lockDate);
            $this->util->commit();
        }catch(\Exception $e){
            $this->util->rollBack();
            $message = 'Erro ao alterar situação da mesa. Erro: '.$e->getMessage();
            $response->addMessage(new Message($message, self::ERROR_MESSAGE));
        }
    }

    public function lockPositions(Request $request, Response $response){
        $inputJSON = file_get_contents('php://input');
        $this->util->saveLog($inputJSON,'Order','lockPositions');
        $input = json_decode($inputJSON, true);
        $this->util->beginTransaction();
        try {
            if (!isset($input)) {
                throw new \Exception('Não foram recebidos parâmetros');
            }
            
            $data   = array(); // dados a serem retornados

            $params    = $input;
            $checkinId = $params['checkinId'];
            $positions = $params['positions'];
            $status    = self::REQUEST_ACCOUNT;
            $lockDate  = date('d/m/Y H:i:s');
            $this->changePositionStatus($checkinId, $positions, $status, $lockDate);
            $this->util->commit();
        }catch(\Exception $e){
            $this->util->rollBack();
            $message = 'Erro ao alterar situação das posicoes: '.$e->getMessage();
            $response->addMessage(new Message($message, self::ERROR_MESSAGE));
        }
    }

    public function unlockTable(Request $request, Response $response){
        $inputJSON = file_get_contents('php://input');
        $this->util->saveLog($inputJSON,'Order','unlockTable');
        $input = json_decode($inputJSON, true);
        $this->util->beginTransaction();
        try {
            if (!isset($input)) {
                throw new \Exception('Não foram recebidos parâmetros');
            }
            
            $data   = array(); // dados a serem retornados

            $params    = $input;
            $checkinId = $params['checkinId'];
            
            $this->changeTableStatus($checkinId, self::BUSY);
            $this->util->commit();
        }catch(\Exception $e){
            $this->util->rollBack();
            $message = 'Erro ao alterar situação da mesa. Erro: '.$e->getMessage();
            $response->addMessage(new Message($message, self::ERROR_MESSAGE));
        }
    }

    public function unlockPositions(Request $request, Response $response){
        $inputJSON = file_get_contents('php://input');
        $this->util->saveLog($inputJSON,'Order','unlockPositions');
        $input = json_decode($inputJSON, true);
        $this->util->beginTransaction();
        try {
            if (!isset($input)) {
                throw new \Exception('Não foram recebidos parâmetros');
            }
            
            $data   = array(); // dados a serem retornados

            $params    = $input;
            $checkinId = $params['checkinId'];
            $positions = $params['positions'];
            $status    = self::BUSY;
            
            $this->changePositionStatus($checkinId, $positions, $status);
            $this->util->commit();
        }catch(\Exception $e){
            $this->util->rollBack();
            $message = 'Erro ao alterar situação da mesa. Erro: '.$e->getMessage();
            $response->addMessage(new Message($message, self::ERROR_MESSAGE));
        }
    }
    

    public function changeTableStatus($checkinId, $status, $date = null){
        /*
         * Verifica se o checkinId é válido
        */
        $validCheckinId = $this->orderService->isValidCheckinId($checkinId);
        if($validCheckinId['error']){
            throw new \Exception($validCheckinId['message']);
        }
        /*
         * Obtém CDFILIAL E NRVENDAREST partindo de um IDCHECKIN
         */
        $resultCheckinMesPos = $this->getTableInfoByCheckin($checkinId);
        /*$resultCheckinMesPos = $this->orderService->getResultCheckinMesPos($checkinId);
        if($resultCheckinMesPos['error']){
            throw new \Exception($resultCheckinMesPos['message']);
        }

        $resultCheckinMesPos = $resultCheckinMesPos['data'];*/

        $tableNumber    = $resultCheckinMesPos['NRMESA'];
        $vendarestSeq   = $resultCheckinMesPos['NRVENDAREST'];
        $comandavenSeq  = $resultCheckinMesPos['NRCOMANDA'];
        $subsidiaryCode = $resultCheckinMesPos['CDFILIAL'];
        $storeCode      = $resultCheckinMesPos['CDLOJA'];

        $tableStatus    = $this->util->getTableStatus($subsidiaryCode,$storeCode, $tableNumber);
        if(empty($tableStatus)){
            throw new \Exception('Mesa não encontrada');
        }else if($tableStatus !== $status){
            $responseStatusMesa = $this->orderService->changeTableStatus($status, $subsidiaryCode, $storeCode, $tableNumber, $date);
            if($responseStatusMesa['error']) {
                throw new \Exception($responseStatusMesa['message']);
            }
        }
    }

    public function changePositionStatus($checkinId, $positions, $status, $date = null){
        /*
         * Verifica se o checkinId é válido
        */
        $validCheckinId = $this->orderService->isValidCheckinId($checkinId);
        if($validCheckinId['error']){
            throw new \Exception($validCheckinId['message']);
        }

        /*
         * Obtém CDFILIAL E NRVENDAREST partindo de um IDCHECKIN
        */
        $resultCheckinMesPos = $this->getTableInfoByCheckin($checkinId);
        /*$resultCheckinMesPos = $this->orderService->getResultCheckinMesPos($checkinId);
        if($resultCheckinMesPos['error']){
            throw new \Exception($resultCheckinMesPos['message']);
        }

        $resultCheckinMesPos = $resultCheckinMesPos['data'];*/

        $tableNumber    = $resultCheckinMesPos['NRMESA'];
        $vendarestSeq   = $resultCheckinMesPos['NRVENDAREST'];
        $comandavenSeq  = $resultCheckinMesPos['NRCOMANDA'];
        $subsidiaryCode = $resultCheckinMesPos['CDFILIAL'];
        $storeCode      = $resultCheckinMesPos['CDLOJA'];

        $tableStatus    = $this->util->getTableStatus($subsidiaryCode,$storeCode, $tableNumber);

        if(empty($tableStatus)){
            throw new \Exception('Mesa não encontrada');
        }else if($tableStatus == self::REQUEST_ACCOUNT){
            throw new \Exception('Conta solicitada para a mesa');
        }else{
            foreach($positions as $position){
                $positionNumber = $position['positionNumber'];
                $positionStatus = $this->util->getPositionStatus($subsidiaryCode,$vendarestSeq, $comandavenSeq, $positionNumber);
                if(empty($positionStatus)){
                    throw new \Exception('Posicao não encontrada');
                }else if($positionStatus !== $status){
                    $responseStatusMesa = $this->orderService->changePositionStatus($status, $subsidiaryCode, $vendarestSeq, $comandavenSeq, $positionNumber, $date);
                    if($responseStatusMesa['error']) {
                        throw new \Exception($responseStatusMesa['message']);
                    }
                }
            }
        }
    }

    public function validateAddress(Request $request, Response $response){
        $inputJSON = file_get_contents('php://input');
        $input = json_decode($inputJSON, true);
        $this->util->beginTransaction();
        try {
            if (!isset($input)) {
                throw new \Exception('Não foram recebidos parâmetros');
            }
            
            $data   = array(); // dados a serem retornados
            //@todo alterar getCityCode para usar o país também 
            $params        = $input;
            $country       = '0055'; 
            $stateCode     = $params['state'];
            $cityName      = $params['cityName'];
            $neighborhood  = $params['neighborhood'];
            $streetAddress = $params['streetAddress'];
            $number        = $params['number'];
            $locationLat   = $params['lat'];
            $locationLng   = $params['lng'];
            $responseObject = array(
                'isAttended'   => false,
                'subsidiaries' => array()
            );
            if(!empty($locationLat) && !empty($locationLng)){
                $locationCoords = array($locationLat,$locationLng);
            }else{
                $address = $streetAddress.', '.$number;
                if(!empty($neighborhood)){
                    $address   = $address.', '.$neighborhood;
                }
                if(!empty($cityName)){
                    $address   = $address.', '.$cityName;
                }
                if(!empty($stateCode)){
                    $address   = $address.', '.$stateCode;
                }
                $addressUrl = urlencode($address);
                $url        = $this->geocodeURL.$addressUrl;
                $respJSON   = file_get_contents($url);
                $resp       = json_decode($respJSON,true);
                if($resp['status'] == 'OK'){
                    $location       = $resp['results'][0]['geometry']['location'];
                    $locationLat    = $location['lat'];
                    $locationLng    = $location['lng'];
                    $locationCoords = array($locationLat,$locationLng);
                }else{
                    throw new \Exception("Não foi possível encontrar as coordenadas do endereço", self::ERROR_MESSAGE);
                }
            }    
            $deliveryAreasResp = $this->orderService->getDeliveryAreas();
            if(!$deliveryAreasResp['error']){
                $deliveryAreas = $deliveryAreasResp['data'];
                foreach($deliveryAreas as $area){
                    $subsidiaryCode      = $area['CDFILIAL'];
                    $deliveryFee         = $area['VRTXENTREGA'];
                    $priority            = $area['NRORDATEND'];
                    $areaCode            = $area['CDAREA'];
                    $stringCoordinates   = $area['COORDAREA'];
                    $stringCoordinates   = str_replace("['","",$stringCoordinates);
                    $stringCoordinates   = str_replace("']","",$stringCoordinates);
                    $stringCoordinates   = str_replace("','",'$',$stringCoordinates);
                    $polyCoordinates     = array();
                    $explodedCoordinates = explode('$',$stringCoordinates);  
                    foreach($explodedCoordinates as $latLongStr){
                        $latLong = explode(',',$latLongStr);
                        array_push($polyCoordinates, $latLong);
                    } 
                    $inside = $this->inside($locationCoords,$polyCoordinates);
                    if($inside){
                        $responseObject['isAttended'] = true;
                        $subsidiaryObject = array(
                            'subsidiaryCode' => $subsidiaryCode,
                            'deliveryFee'    => $deliveryFee,
                            'priority'       => $priority,
                            'areaCode'       => $areaCode
                        );
                        array_push($responseObject['subsidiaries'], $subsidiaryObject);
                    }                    
                }
            }else{
                throw new \Exception("Não foi possível buscar as áreas de atendimento", self::ERROR_MESSAGE);
            }  
            array_push($data,$responseObject); 
            $response->addDataSet(new DataSet('', $data));
            $this->util->commit();
        }catch(\Exception $e){
            $this->util->rollBack();
            $message = 'Erro ao validar endereço. Erro: '.$e->getMessage();
            $response->addMessage(new Message($message, self::ERROR_MESSAGE));
        }
    }

    public function inside($point, $vs) {
        $x = $point[0];
        $y = $point[1];

        $inside = false;
        for ($i = 0, $j = count($vs) - 1; $i < count($vs); $j = $i++) {
            $xi = (float)$vs[$i][0];
            $yi = (float)$vs[$i][1];
            $xj = (float)$vs[$j][0];
            $yj = (float)$vs[$j][1];

            $intersect = (($yi > $y) != ($yj > $y)) && ($x < ($xj - $xi) * ($y - $yi) / ($yj - $yi) + $xi);
            if ($intersect) 
                $inside = !$inside;
        }
        return $inside;
    }

    public function getDeliveryDetails(Request $request, Response $response){
        $inputJSON = file_get_contents('php://input');
        $input = json_decode($inputJSON, true);
        $this->util->beginTransaction();
        try {
            if (!isset($input)) {
                throw new \Exception('Não foram recebidos parâmetros');
            }
            
            $data   = array(); // dados a serem retornados

            $params         = $input;
            $areaCode       = $params['areaCode'];
            $subsidiaryCode = $params['subsidiaryCode'];
            $storeCode      = $params['storeCode'];

            $storeDeliveryTime = $this->util->getStoreDeliveryTime($storeCode, $subsidiaryCode);
            $deliveryFee       = $this->util->getDeliveryFee($areaCode,$subsidiaryCode);
            $data = array(
                array(
                    'deliveryFee'  => $this->util->removeDecimal($deliveryFee),
                    'deliveryTime' => $storeDeliveryTime
                )
            );
            $response->addDataSet(new DataSet('', $data));
            $this->util->commit();
        }catch(\Exception $e){
            $this->util->rollBack();
            $message = 'Erro ao buscar detalhes de entrega da loja. Erro: '.$e->getMessage();
            $response->addMessage(new Message($message, self::ERROR_MESSAGE));
        }
    }
    
    public function recalculateItensPrice(Request $request, Response $response){
        $inputJSON = file_get_contents('php://input');
        $input = json_decode($inputJSON, true);
        $this->util->beginTransaction();
        try {
            if (!isset($input)) {
                throw new \Exception('Não foram recebidos parâmetros');
            }
            // dados a serem retornados
            
            $params = $input;
            
            $subsidiaryCode = $params['subsidiaryCode'];
            $clientCode     = $params['clientCode'];
            $consumerCode   = $params['consumerCode'];
            $checkinId      = $params['checkinId'];
            
            $validCheckinId = $this->orderService->isValidCheckinId($checkinId);
            if($validCheckinId['error']){
                throw new \Exception($validCheckinId['message']);
            }
            $resultCheckinMesPos = $this->getTableInfoByCheckin($checkinId);
            /*$resultCheckinMesPos = $this->orderService->getResultCheckinMesPos($checkinId);
            if($resultCheckinMesPos['error']){
                throw new \Exception($resultCheckinMesPos['message']);
            }
            $checkinMesPos   = $resultCheckinMesPos['data'];*/
            $storeCode   = $resultCheckinMesPos['CDLOJA'];
            $tableNumber = $resultCheckinMesPos['NRMESA'];
            $nrvendarest = $resultCheckinMesPos['NRVENDAREST'];
            $positionNumber = $this->util->getTablePositions($subsidiaryCode, $nrvendarest);
            
            $data = array();
            
            if (!$this->util->consumerExists($clientCode, $consumerCode)) {
                throw new \Exception('Consumidor inexistente.');
            }
            foreach($positionNumber as $position){
                $positionData = array(
                    'subsidiaryCode' => $subsidiaryCode,
                    'clientCode'     => $clientCode,
                    'consumerCode'   => $consumerCode,
                    'positionNumber' => $position,
                    'storeCode'      => $storeCode,
                    'tableNumber'    => $tableNumber,
                    'items'          => array()
                );
                if(isset($params['items']) && !empty($params['items']))
                    $return = $this->recalculateItemsPrice($params['items'],$subsidiaryCode,$clientCode,$consumerCode,$storeCode);
                
                $return = $this->getProductsToRecalculate($subsidiaryCode, $tableNumber, $position, $storeCode);
                if(!$return['error']){
                    $productsToRecalculate = $return['data'];
                    $productsData = $this->recalculateProductsPrice($productsToRecalculate,$subsidiaryCode, $tableNumber, $position, $storeCode, $clientCode, $consumerCode, true);
                    $positionData['items'] = $productsData;
                }
                
                $return = $this->getCombosToRecalculate($subsidiaryCode, $tableNumber, $position, $storeCode);
                if(!$return['error']){
                    $combosToRecalulate = $return['data'];
                    $combosData = $this->recalculateCombos($combosToRecalulate, $subsidiaryCode, $storeCode, $clientCode, $consumerCode, true);
                    $positionData['items'] = array_merge($positionData['items'],$combosData);
                }
                
                array_push($data,$positionData);
            }
            
            $response->addDataSet(new DataSet('', $data));
            $this->util->commit();
        }catch(\Exception $e){
            $this->util->rollBack();
            $message = 'Erro ao realizar recálculo dos itens do pedido. Erro: '.$e->getMessage();
            $response->addMessage(new Message($message, self::ERROR_MESSAGE));
        }
    }
    
    private function recalculateItemsPrice($items,$subsidiaryCode,$clientCode,$consumerCode,$storeCode){
        foreach($items as &$item){
            $this->recalculateItemPrice($item,$subsidiaryCode,$clientCode,$consumerCode,$storeCode);
        }
    }
    
    private function recalculateItemPrice($item,$subsidiaryCode,$clientCode,$consumerCode,$storeCode){
        if(!empty($item['childItems'])){
            $item['unitValue'] = 0;
            $item['discount']  = 0;
            foreach ($item['childItems'] as $childItem) {
                $priceInfo = $this->recalculateItemPrice($childItem,$subsidiaryCode,$clientCode,$consumerCode,$storeCode);
                $item['unitValue'] += $priceInfo['unitValue'];
                $item['discount']  += $priceInfo['discount'];
            }
        }else{
            $priceDetails = $this->getConsumerItemPrice($subsidiaryCode,$clientCode,$item['productCode'],$storeCode,$consumerCode);
            $discount     = $this->getNewDiscount($priceDetails,$item);
            $newUnitPrice = !empty($priceDetails['newPrice']) ? $priceDetails['newPrice'] : $item['VRPRECCOMVEN'];

            $this->updateItemPrice($item['CDFILIAL'],$item['NRVENDAREST'],$item['NRCOMANDA'],$item['NRPRODCOMVEN'], $this->util->removeDecimal($newUnitPrice), $discount);

            $item['unitValue'] = $newUnitPrice;
            $item['discount']  = $discount;
            return array(
                'unitValue' => $newUnitPrice,
                'discount'  => $discount
            );
        }
    }
    
    private function getProductsToRecalculate($subsidiaryCode, $tableNumber, $positionNumber, $storeCode){
        return $this->orderService->getProductsToRecalculate($subsidiaryCode, $tableNumber, $positionNumber, $storeCode);
    }
    
    private function recalculateProductsPrice($productsToRecalculate, $subsidiaryCode, $tableNumber, $positionNumber, $storeCode, $clientCode, $consumerCode, $saveRecalculatedItem = true){
        $itemsData = array();
            
        foreach($productsToRecalculate as $item){
            
            $productCode  = $item['CDPRODUTO'];
            $priceDetails = $this->getConsumerItemPrice($subsidiaryCode,$clientCode,$productCode,$storeCode,$consumerCode);
            
            $discount = $this->getNewDiscount($priceDetails,$item);
            $newUnitPrice = !empty($priceDetails['newPrice']) ? $priceDetails['newPrice'] : $item['VRPRECCOMVEN'];
            
            $itemData = array(
                'productCode'  => $productCode,
                'productName'  => $item['NMPRODUTO'],
                'quantity'     => $this->util->float($item['QTPRODCOMVEN']),
                'unitValue'    => $this->util->removeDecimal($newUnitPrice),
                'discount'     => $this->util->removeDecimal($discount),
                'observations' => array()
            );
            if($saveRecalculatedItem){
                $this->updateItemPrice($item['CDFILIAL'],$item['NRVENDAREST'],$item['NRCOMANDA'],$item['NRPRODCOMVEN'], $this->util->removeDecimal($newUnitPrice), $this->util->removeDecimal($discount));
            }

            $return = $this->getObservations($item['CDFILIAL'],$item['NRVENDAREST'],$item['NRCOMANDA'],$item['NRPRODCOMVEN']);
            if(!$return['error'])
                $itemData['observations'] = $comboProductData['observations'] = $this->formatObservations($return['data']);

            array_push($itemsData, $itemData);
        }
        return $itemsData;
    }
    
    private function getConsumerItemPrice($subsidiaryCode,$clientCode,$productCode,$storeCode,$consumerCode){
        
        $return = array(
            'newPrice'      => 0,
            'discountType'  => '',
            'discountValue' => 0
        );
        $priceDetails = $this->orderService->recalculateItensPrice($subsidiaryCode,$clientCode,$productCode,$storeCode,$consumerCode);
        if(!$priceDetails['error']){
            $return['newPrice']      = ($priceDetails['PRECO'] + $priceDetails['ACRE']) - $priceDetails['DESC'];
            // $return['discountType']  = $priceDetails['TIPO'];
            // $return['discountValue'] = $priceDetails['PRECODIA'];
        }
        return $return;
    }
    
    private function getObservations($CDFILIAL, $NRVENDAREST, $NRCOMANDA, $NRPRODCOMVEN){
        return $this->orderService->getObservations($CDFILIAL, $NRVENDAREST, $NRCOMANDA, $NRPRODCOMVEN);
    }

    private function updateItemPrice($CDFILIAL, $NRVENDAREST, $NRCOMANDA, $NRPRODCOMVEN, $VRPRECCOMVEN, $VRDESCCOMVEN){
        return $this->orderService->updateItemPrice($CDFILIAL, $NRVENDAREST, $NRCOMANDA, $NRPRODCOMVEN, $VRPRECCOMVEN, $VRDESCCOMVEN);
    }
    
    private function getCombosToRecalculate($subsidiaryCode, $tableNumber, $positionNumber, $storeCode){
        return $this->orderService->getCombosToRecalculate($subsidiaryCode, $tableNumber, $positionNumber, $storeCode);
    }
    
    private function recalculateCombos($combosToRecalulate, $subsidiaryCode, $storeCode, $clientCode, $consumerCode, $saveRecalculatedItem = true){
        $recalculatedCombos = array();
        foreach ($combosToRecalulate as $combo) {
            
            $comboData = array(
                'productCode'   => $combo['CDPRODPROMOCAO'],
                'productName'   => $combo['NMPRODUTO'],
                'quantity'      => 1, 
                'unitValue'     => 0,
                'discount'      => 0,
                'observations'  => array(),
                'childItems'    => array()
            );
            $return = $this->getComboProductsToRecalculate($combo['CDPRODPROMOCAO'],$combo['CDFILIAL'],$combo['NRVENDAREST'],$combo['NRCOMANDA'], $combo['NRSEQPRODCOM']);
            if(!$return['error']){
                $comboProductsToRecalculate = $return['data'];
                $comboProducts = $this->recalculateComboProducts($comboProductsToRecalculate, $subsidiaryCode, $storeCode, $clientCode, $consumerCode, $combo['NRCOMANDA'], $combo['NRVENDAREST'], $saveRecalculatedItem);
                $comboData['childItems'] = $comboProducts;
            }
            foreach($comboData['childItems'] as $childItem){
                $comboData['unitValue'] += $childItem['unitValue'];
                $comboData['discount']  += $childItem['discount'];
            }
            array_push($recalculatedCombos,$comboData);
        }
        
        return $recalculatedCombos;
    }
    
    private function getComboProductsToRecalculate($CDPRODPROMOCAO,$CDFILIAL,$NRVENDAREST,$NRCOMANDA, $NRSEQPRODCOM){
        return $this->orderService->getComboProductsToRecalculate($CDPRODPROMOCAO,$CDFILIAL,$NRVENDAREST,$NRCOMANDA, $NRSEQPRODCOM);
    }

    private function numberFormatPrecision($number, $precision = 2, $separator = '.'){
        $numberParts = explode($separator, $number);
        $response = $numberParts[0];
        if(count($numberParts)>1){
            $response .= $separator;
            $response .= substr($numberParts[1], 0, $precision);
        }
        return $response;
    }

    private function getNewDiscount($priceDetails,$item){

        $discountType  = $priceDetails['discountType'];
        $discountValue = $priceDetails['discountValue'];
        
        $itemDiscount = isset($item['VRDESCCOMVEN']) ? $item['VRDESCCOMVEN'] : $item['discount'];

        if($discountType == 'V')
            $discount = $itemDiscount + $itemDiscount - $discountValue;
        else
            $discount = $itemDiscount - ($itemDiscount * ($discountValue/100));

        return floatval($this->numberFormatPrecision($discount));
    }
    
    private function recalculateComboProducts($comboProducts, $subsidiaryCode, $storeCode, $clientCode, $consumerCode, $nrcomanda, $nrvendarest, $saveRecalculatedItem){
        $comboData = array();
        
        foreach ($comboProducts as $comboProduct) {
            
            $productCode     = $comboProduct['CDPRODUTO'];
            $priceDetails    = $this->getConsumerItemPrice($subsidiaryCode, $clientCode, $productCode, $storeCode, $consumerCode);
            $discount        = $this->getNewDiscount($priceDetails,$comboProduct);
            $newUnitPrice    = $priceDetails['newPrice'];
            $itcomandavenSeq = $comboProduct['NRPRODCOMVEN'];
            $newUnitPrice    = !empty($newUnitPrice) ? $newUnitPrice : $comboProduct['VRPRECCOMVEN'];
            
            $comboProductData = array(
                'productCode'   => $comboProduct['CDPRODUTO'],
                'productName'   => $comboProduct['NMPRODUTO'],
                'quantity'      => $this->util->float($comboProduct['QTPRODCOMVEN']),
                'unitValue'     => $this->util->removeDecimal($newUnitPrice),
                'discount'      => $this->util->removeDecimal($discount),
                'observations'  => array()
            );
            if($saveRecalculatedItem){
                $this->updateItemPrice($comboProduct['CDFILIAL'],$comboProduct['NRVENDAREST'],$comboProduct['NRCOMANDA'],$comboProduct['NRPRODCOMVEN'], $this->util->removeDecimal($newUnitPrice), $this->util->removeDecimal($discount));
            }

            $return = $this->getObservations($subsidiaryCode,$nrvendarest,$nrcomanda,$itcomandavenSeq);
                if(!$return['error']){
                $comboProductData['observations'] = $this->formatObservations($return['data']);

            }
            
            array_push($comboData, $comboProductData);
        }
        return $comboData;
    }

    private function formatObservations($observations){

        $observationsData = array();
        foreach($observations as $observation){
            
            $observationData = array(
                'obsGroupCode' => $observation['CDGRPOCOR'],
                'obsCode'      => $observation['CDOCORR'],
                'obsName'      => $observation['DSOCORR'],
                'quantity'     => 1,    
                'price'        => 0
            );
            
            array_push($observationsData, $observationData);
        }        
        return $observationsData;                      
    }
    
    public function printOrder($subsidiaryCode, $comandavenSeq, $storeCode, $items, $sellerCode, $standardTable, $pagerId, $toGoString, $taaOrder){
        $this->orderService->printOrder($subsidiaryCode, $storeCode, $items, $sellerCode, null, $standardTable, $pagerId, 'B', $toGoString, false, true, $comandavenSeq, $taaOrder);
    }
    
    private function formatOrderToPrint($items){
        $dataToPrint = array();
        foreach ($items as $item) {
            $obsData = array();
            $itemData = $this->printItemPattern();
            if($item['childItems']){
                $itemData['NMPRODPROMOCAO'] = $item['productName'];
                $itemData['IDTIPOCOMPPROD'] = '3';
                foreach($item['childItems'] as $childItem){
                    if(!empty($childItem['observations'])){
                        $obsData = array_merge($obsData, $this->returnTxProdComVen($childItem['observations']));
                    }
                }
            }
            
            if(!empty($item['observations'])){
                $obsData = array_merge($obsData, $this->returnTxProdComVen($item['observations']));  
            }
            if(!empty($obsData)){
                $itemData['TXPRODCOMVEN'] ='-> '.implode(PHP_EOL.'   -> ',$obsData);
            }
            $itemData['CDPRODUTO']    = $item['productCode'];
            $itemData['NMPRODUTO']    = $item['productName'];
            $itemData['QTPRODCOMVEN'] = $item['quantity'];
            
            array_push($dataToPrint,$itemData);
        }
        return $dataToPrint;
    }
    
    private function printItemPattern(){
        return array(
            "CDPRODUTO"      => "",
            "NMPRODUTO"      => "",
            "IDTIPOCOMPPROD" => "",
            "QTPRODCOMVEN"   => 0,
            "TXPRODCOMVEN"   => "",
            "IDIMPPROMOCAO"  => "N",
            "ORDEMIMP"       => null,
            "ATRASOPROD"     => null,
            "ORDEM"          => null,
            "ORDEMIMP"       => null,
            "NMPRODPROMOCAO" => null
        );
    }
    
    private function returnTxProdComVen($observations){
        $obsData = array();
        foreach ($observations as $observation) {
            array_push($obsData,$observation['obsName']);
        }
        return $obsData;  
    }

    private function getTableInfoByCheckin($checkinId){
        $resultCheckinMesPos = $this->orderService->getResultCheckinMesPos($checkinId);
        if($resultCheckinMesPos['error']){
            throw new \Exception($resultCheckinMesPos['message']);
        }
        $resultCheckinMesPos = $resultCheckinMesPos['data'];
        return array(
            'CDFILIAL'      => $resultCheckinMesPos['CDFILIAL'],
            'NRVENDAREST'   => $resultCheckinMesPos['NRVENDAREST'],
            'NRPOSICAO'     => $resultCheckinMesPos['NRPOSICAO'],
            'NRCOMANDA'     => $resultCheckinMesPos['NRCOMANDA'],
            'NRMESA'        => $resultCheckinMesPos['NRMESA'],
            'CDLOJA'        => $resultCheckinMesPos['CDLOJA'],
            'CDCLIENTE'     => $resultCheckinMesPos['CDCLIENTE'],
            'CDCONSUMIDOR'  => $resultCheckinMesPos['CDCONSUMIDOR']
        );
    }

    private function verifyDailyCreditLimit($paymentValue, $clientCode,$consumerCode){
        $cdfamilisald = '001';
        $maxDailyBuyValue = $this->orderService->getConsumerMaxDaily($clientCode, $consumerCode);
        $balance          = $this->util->getConsumerBalance($clientCode, $consumerCode, $cdfamilisald);
        $todaysMovement   = $this->util->getTodaysExpenditure($clientCode, $consumerCode);
        $todasyPayment    = $this->util->getTodayPayments($clientCode, $consumerCode);

        $maxDailyBuyValue = !empty($maxDailyBuyValue['VRMAXCREDCONS']) ? $maxDailyBuyValue['VRMAXCREDCONS'] : 0;
        $maxDailyBuyValue = $this->util->removeDecimal($maxDailyBuyValue);
        
        if($paymentValue > $balance)
            return $this->returnPattern(true, 'O valor do pagamento é maior que o saldo disponível.');
        else if(($maxDailyBuyValue != 0) && ($todaysMovement + $paymentValue > $maxDailyBuyValue))
            return $this->returnPattern(true, 'O valor do pagamento é maior que o limite diário estabelecido.');
        else if($balance - $todasyPayment < $paymentValue)
            return $this->returnPattern(true, 'O valor do pagamento é maior que o saldo atual.');
        else
            return $this->returnPattern(false, '');
    }

    private function returnPattern($error, $message){
        return array(
            'error'   => $error,
            'message' => $message
        );
    }

    public function getOrderStatus(Request $request, Response $response){
        $inputJSON = file_get_contents('php://input');
        $input = json_decode($inputJSON, true);
        try {
            if (empty($input))
                throw new \Exception('Não foram recebidos parâmetros');

            $params = $input;
            
            $subsidiaryCode = $params['subsidiaryCode'];
            $saleCode       = $params['saleCode'];
            $saleId         = $params['saleId'];

            $orderInfo = $this->util->getOrderStatus($subsidiaryCode, $saleCode, $saleId);
            if(empty($orderInfo)) throw new \Exception("Pedido não encontrado.");
            $orderStatus = $this->getStatus($orderInfo);

            $data = array(
                array(
                    'orderStatus' => $orderStatus
                )
                
            );
            $response->addDataSet(new DataSet('', $data));

        } catch (\Exception $e) {
            $message = 'Erro ao recuperar status do pedido. Erro: '.$e->getMessage();
            $response->addMessage(new Message($message, self::ERROR_MESSAGE));
        }
    }

    private function getStatus($orderInfo){
        $orderStatus = '2';
        if($orderInfo['IDSTCOMANDA'] == '4')     $orderStatus = 'C';
        elseif($orderInfo['IDSINCMSDE'] == 'N')  $orderStatus = '0';
        elseif(!empty($orderInfo['DTCHEGACMD'])) $orderStatus = '4';
        elseif(!empty($orderInfo['DTSAIDACMD'])) $orderStatus = '3';

        return $orderStatus;
    }
    
    public function setOrderArrival(Request $request, Response $response){
        $inputJSON = file_get_contents('php://input');
        $input = json_decode($inputJSON, true);
        try{
            $this->util->beginTransaction();
            
            if (empty($input))
                throw new \Exception('Não foram recebidos parâmetros');
            $params = $input;
            
            $subsidiaryCode = $params['subsidiaryCode'];
            $saleCode       = $params['saleCode'];
            $saleId         = $params['saleId'];
            $arrivalTime    = !empty($params['arrivalTime']) ? $params['arrivalTime'] : date('d/m/Y H:i:s', time());;
            
            $retorno = $this->util->setOrderArrival($subsidiaryCode, $saleCode, $saleId, $arrivalTime);
            if(empty($retorno)) throw new \Exception("Não foi possível encontrar o pedido.");
            
            $this->util->commit();
            
            $response->addMessage(new Message("Horário atualizado com sucesso.", self::GENERAL_MESSAGE));
            
        }catch(\Exception $e){
            $this->util->rollback();
            $message = 'Erro ao atualizar horário de chegada. Erro: '.$e->getMessage();
            $response->addMessage(new Message($message, self::ERROR_MESSAGE));
        }
    }
    
    public function getMinTime(Request $request, Response $response){
        $inputJSON = file_get_contents('php://input');
        $input = json_decode($inputJSON, true);
        try{
            if (empty($input))
                throw new \Exception('Não foram recebidos parâmetros');
            $params = $input;
            
            $subsidiaryCode = $params['subsidiaryCode'];
            $storeCode      = $params['storeCode'];
            
            $time = $this->util->getMinTime($subsidiaryCode, $storeCode);
            
            $data = array(
                array(
                    'minimumTime' => $time
                )
                
            );
            $response->addDataSet(new DataSet('', $data));
            
        }catch(\Exception $e){
            throw new \Exception("Houve um erro ao recuperar o tempo de produção desta loja.");
        }
    }
    
    private function validateParams($params){
        $order = $params['order'];
        $data = array();
        if(!isset($order['subsidiaryCode'])) $data['subsidiaryCode'] = true;
        if(!isset($order['items'])) $data['items'] = true;
        else{
            $items = $order['items'];
            foreach ($items as $item)
                $this->validateItems($item); 
        }
        if(!empty($data))
            throw new \Exception("Alguns parâmetros obrigatórios estão vazios, verifique os dados.");
    }
    
    public function savePayment(Request $request, Response $response){
        
        $inputJSON = file_get_contents('php://input');
        $input = json_decode($inputJSON, true);
        $timeout = $this->requestTimeout;
        $now = strtotime("now");
                
        $this->util->beginTransaction();
        try{
            $nrcomanda = $input['saleCode'];
            $cdfilial  = $input['subsidiaryCode'];
            $cashBack  = $input['cashBack'];
            $payments  = $input['payments'];
            $giftCards = isset($input['giftCards']) ? $input['giftCards'] : array();
            $order = $this->util->getNrvendarest($cdfilial, $nrcomanda);
            $deliveryFee = $this->util->removeDecimal($order['VRACRCOMANDA']);
            $nrvendarest = $order['NRVENDAREST'];
            $orderPaymentValue = $this->util->getPaymentValue($cdfilial, $nrvendarest);
            if(!empty($orderPaymentValue)) throw new \Exception("Esta comanda já foi paga!");
            $cdcliente = $order['CDCLIENTE'];
            $cdconsumidor = $order['CDCONSUMIDOR'];
            if(!$nrvendarest) throw new \Exception("Não foi posssível encontrar o pedido.");
            if(empty($payments) && empty($giftCards) && empty($cashBack)) throw new \Exception("Nenhuma forma de pagamento informada.");
            if(!empty($payments)){
                $savePaymentsreturn = $this->savePayments($payments,$cdfilial,$nrvendarest, $cdcliente, $cdconsumidor, 0);
                if($savePaymentsreturn['error']){
                    $message = $savePaymentsreturn['message'];
                    throw new \Exception($message);
                }
            }
            if(!empty($giftCards)){
                $totalValue = $this->util->removeDecimal($this->orderService->getItemsValue($cdfilial, $nrvendarest, $nrcomanda));
                $totalValue = $this->util->subtract($this->util->add($totalValue, $deliveryFee), $cashBack);
                $caixa = $this->util->getCDCAIXA($cdfilial, $nrcomanda, $nrvendarest);
                $cdcaixa = $caixa['CDCAIXACOLETOR'];
                $DTABERCAIX = $this->util->getPosDate($cdfilial, $cdcaixa);
                $giftCardsResponse = $this->saveGiftCards($giftCards, $cdfilial, $nrvendarest, $cdcaixa, $DTABERCAIX, $totalValue);
                if($giftCardsResponse['error']){
                    $message = $giftCardsResponse['message'];
                    throw new \Exception($message);
                }
            }
            $productsValue = 0;
            $items = $this->orderService->getOrderItems($nrvendarest, $nrcomanda, $cdfilial);
            foreach($items as $item){
                $productsValue = $this->util->add($productsValue, $item['TOTAL']);
            }
            if(!empty($cashBack)){
                
                $this->validateCashBack($cdcliente, $cdconsumidor, $cashBack);
                
                $realItemsQuant = count($items);
                $realItemsQuant = $this->util->div($realItemsQuant, 100);
                foreach($items as &$item){
                    $item['quantity'] = $item['QTPRODCOMVEN'];
                    $item['unitValue'] = $item['VRPRECCOMVEN'];
                    $item['discount'] = $item['VRDESCCOMVEN'];
                }
                unset($item);
                
                $remaingValue = $productsValue;
                
                if($cashBack < $productsValue){
                    $orderDiscount = $cashBack;
                }else{
                    $cashToApply = $this->util->subtract($productsValue, $realItemsQuant);
                    $orderDiscount = $cashToApply;
                    $userCashBack = $cashBack;
                    $cashBack = $this->util->add($cashToApply, $realItemsQuant);
                    if($userCashBack < $this->util->add($cashBack, $deliveryFee)){
                        $cashBack = $userCashBack;
                    }else{
                        $cashBack = $this->util->add($cashBack, $deliveryFee);
                    }
                    $cdtiporece = $this->util->getMoney();
                    $orderPayments = $this->orderService->getOrderPayments($cdfilial, $nrvendarest);
                    if(empty($orderPayments)){
                        $movcaixadlvSeq = str_pad(1, 10, '0', STR_PAD_LEFT);
                        $paymentValue = $this->util->add($realItemsQuant, $deliveryFee);
                        $this->insertMovCaixaDlv($cdfilial,$nrvendarest,$movcaixadlvSeq,'E',$cdcliente,$cdconsumidor,$cdtiporece,$paymentValue, null, 1);
                    }
                }
                
                $totalDiscount = $remaingDiscount = $orderDiscount;
                while($orderDiscount > 0){
                    $sumValue = 0;
                    foreach ($items as &$item) {
                        $this->shareDiscount($item, $orderDiscount, $productsValue, $remaingValue, $remaingDiscount, false);
                    }
                    unset($item);
                    $remaingValue = $productsValue;
                    foreach ($items as $item) {
                        $sumValue += $this->getDiscountValue($item);
                    }
                    $orderDiscount = $this->util->subtract($totalDiscount, $sumValue);
                    $remaingDiscount = $orderDiscount;
                    unset($item);
                }
                foreach($items as $item){
                    $cdproduto = $item['CDPRODUTO'];
                    $nrprodcomven = $item['NRPRODCOMVEN'];
                    $discount = $item['discount'];
                    $this->orderService->updateItemDiscount($nrcomanda, $nrvendarest, $cdfilial, $cdproduto, $nrprodcomven, $discount);
                }
                $this->orderService->saveCashBack($nrcomanda, $nrvendarest, $cdfilial, $cashBack);
            }

            $this->validatePayments($cdfilial, $nrvendarest, $nrcomanda, $order['VRACRCOMANDA']);
            
            if(empty($cdconsumidor) && !empty($giftCards)){
                $this->orderService->updateConsumerVendarest($giftCards[0]['clientCode'], $giftCards[0]['consumerCode'], $nrvendarest, $cdfilial);
            }
            
            $response->addMessage(new Message("Pagamento efetuado com sucesso ($cdfilial-$nrcomanda).", self::GENERAL_MESSAGE));
            $this->util->commit();
        }catch(\Exception $e){
            $this->util->rollback();
            $message = 'Erro ao salvar o pagamento. Erro: '.$e->getMessage();
            
            $response->addMessage(new Message($message, self::ERROR_MESSAGE));
        }
        
    }

    public function cancelOrder(Request $request, Response $response){
        $this->util->beginTransaction();
        try{
            $inputJSON = file_get_contents('php://input');
            $input = json_decode($inputJSON, true);

            $nrcomanda = $input['saleCode'];
            $nrseqvenda = $input['saleSeq'];
            $cdcaixa = $input['posCode'];
            $cdfilial  = $input['subsidiaryCode'];

            $controlPath = "{$this->logPath}\\..\\CANCEL-CTRL-FILE.dat";
            if(!file_exists($controlPath)) throw new \Exception("Serviço de cancelamento de pedidos inativo.");
            $controlFile = json_decode(base64_decode(file_get_contents($controlPath)), true);
            if((time() - $controlFile['expiresIn']) > self::MAX_CONTROL_FILE_UPDATE_TIME) throw new \Exception("Serviço de cancelamento de pedidos inativo.");
            
            $this->createCancelOrder($cdfilial, $cdcaixa, $nrcomanda, $nrseqvenda);
            
            $this->util->commit();
            $response->addMessage(new Message("O pedido ($cdfilial-$nrcomanda), venda (".((!empty($nrseqvenda)) ? $nrseqvenda : "não informada").") foi incluído para cancelamento.", self::GENERAL_MESSAGE));
            // $response->addMessage(new Message("Pedido cancelado com sucesso.", self::GENERAL_MESSAGE));
        }catch(\Exception $e){
            $this->util->rollback();
            $message = 'Erro ao cancelar o pedido. Erro: '.$e->getMessage();
            $response->addMessage(new Message($message, self::ERROR_MESSAGE));
        }
    }

    private function createCancelOrder($cdfilial, $cdcaixa, $nrcomanda, $nrseqvenda){
        
        $nrvendarest = $this->util->getNrvendarest($cdfilial, $nrcomanda);
        $nrvendarest = $nrvendarest['NRVENDAREST'];
        
        if(!$nrvendarest) throw new \Exception("Não foi posssível encontrar o pedido ($cdfilial-$nrcomanda) .");

        $receivedFile = null;
        $receivedPath = "{$this->logPath}\\Cancel\\Received\\".date('Y')."\\".date('m')."\\".date('d')."\\";
        if(file_exists($receivedPath)){
            chdir($receivedPath);
            $receivedFile = glob("*-".$nrcomanda.".json");
        }
        if(!empty($receivedFile)) throw new \Exception("Não é possível cancelar o pedido ($cdfilial-$nrcomanda), venda (".((!empty($nrseqvenda)) ? $nrseqvenda : "não informada")."), pois o cancelamento já foi solicitado anteriormente.");

        $saleStatus = $this->util->getSaleStatus($cdfilial, $cdcaixa, $nrseqvenda);
        
        if(empty($saleStatus) || !isset($saleStatus)|| $saleStatus == false) $saleStatus = $this->util->getSaleByOrderCode($cdfilial, $cdcaixa, $nrcomanda);

        $canceledItems = $this->util->cancelProducts($cdfilial,$nrvendarest,$nrcomanda);
        foreach ($canceledItems as $canceledItem) {
            $kdsSeq = $this->orderService->generateKDSSeq('KDSOPERACAOTEMP');
            $dsOperacao = array(
                'operation' => 'cancelItem',
                'params' => array(
                    'CDFILIAL'     => $cdfilial,
                    'NRVENDAREST'  => $nrvendarest,
                    'NRCOMANDA'    => $nrcomanda,
                    'NRPRODCOMVEN' => $canceledItem['NRPRODCOMVEN']
                )
            );
            $this->util->insertKDSTEMP($kdsSeq, $dsOperacao);
        }
        if(!empty($saleStatus) && isset($saleStatus)){
            if($saleStatus['IDSITUVENDA'] == 'O'){
                $cdoperador = $saleStatus['CDOPERADOR'];
                $idstatusnfce = $saleStatus['IDSTATUSNFCE'];
                $cdsupervisor = $saleStatus['CDSUPERVISOR'];
                $nrseqvenda = $saleStatus['NRSEQVENDA'];
                // $result = $this->orderService->cancelOrder($cdfilial, $cdcaixa, $nrseqvenda, $cdoperador, $cdsupervisor);
                $order = array(
                    'CDFILIAL' => $cdfilial,
                    'CDCAIXA' => $cdcaixa,
                    'NRSEQVENDA' => $nrseqvenda, 
                    'CDOPERADOR' => $cdoperador,
                    'CDSUPERVISOR' => $cdsupervisor,
                    'NRCOMANDA'   => $nrcomanda
                );
                $dtvenda = new \DateTime($saleStatus['DTVENDA']);
                $diff = $dtvenda->diff(new \DateTime());
                if($diff->i >= 30) throw new \Exception("Essa venda não pode ser cancelada. (Execedeu os 30 minutos)");
                
                $result = $this->registerReceivedOrder('Cancel', date('dmy'), $order);
                $result = $this->registerPendingOrder('Cancel', date('dmy'), $order);

                if($result['error']){
                    throw new \Exception($result['message']);
                }
                
            } else if($saleStatus['IDSITUVENDA'] == 'C'){
                throw new \Exception("A venda $nrseqvenda já foi cancelada.");
            } else if($saleStatus['IDSITUVENDA'] == 'P'){
                throw new \Exception("A venda $nrseqvenda se encontra pendente.");
            }
        } else {
            $order = array(
                'CDFILIAL' => $cdfilial,
                'CDCAIXA' => $cdcaixa,
                'NRSEQVENDA' => $nrseqvenda, 
                'CDOPERADOR' => null,
                'CDSUPERVISOR' => null,
                'NRCOMANDA'   => $nrcomanda
            );
            $result = $this->registerReceivedOrder('Cancel', date('dmy'), $order);
            $this->deleteOrder($nrvendarest, $cdfilial, $nrcomanda);
            $this->finish('Cancel', date('dmy'), $receivedPath, $result['fileName'], false);
        }
        
    }

    private function deleteOrder($nrvendarest, $cdfilial, $nrcomanda){
        $this->util->deleteItcomandaven($nrvendarest, $cdfilial, $nrcomanda);
        $this->util->deleteComandaven($nrvendarest, $cdfilial, $nrcomanda);
        $this->util->deleteMovCaixaDlv($nrvendarest, $cdfilial);
        $this->util->deleteVendarest($nrvendarest, $cdfilial);
    }
    
    private function validatePayment($totalValue, $payments, &$giftCards, $cashBack){
        
        $money = $this->util->getMoney();
        $moneyValue = 0;
        $paymentValue = 0;
        $giftCardValue = 0;

        foreach($payments as $payment){
            if($payment['paymentCode'] === $money){
                $moneyValue = $this->util->add($payment['value'], $moneyValue);
            } else {
                $paymentValue = $this->util->add($payment['value'], $paymentValue);
            }
        }
        
        
        foreach($giftCards as $card){
            $giftCardValue = $this->util->add($card['value'], $giftCardValue);
        }
        if($moneyValue >= $totalValue){
            $giftCards = array();
        }

        if($giftCardValue >= $totalValue && !empty($payments)){
            throw new \Exception("Não é possível utilizar outro meio de pagamento visto que o Cartão Vale Presente é suficiente para pagar o pedido.");
        }else if($this->util->add($moneyValue, $this->util->add($paymentValue, $this->util->add($giftCardValue, $cashBack))) < $totalValue){
            throw new \Exception("O valor pago é menor que valor do pedido.");
        } else if(($this->util->add($paymentValue, $cashBack)) > $totalValue){
            throw new \Exception("O valor pago é maior que o valor do pedido.");
        }
    }

    public function validateCashBack($clientCode, $consumerCode, $cashBack){
        if(empty($clientCode) || empty($consumerCode))
            throw new \Exception("Não é possível utilizar cashback sem informar o consumidor!");
        $consumerBalance          = $this->util->getConsumerFidelity($clientCode, $consumerCode);
        $orderFidelityBalance     = $this->util->getValueFidelityOrders($clientCode, $consumerCode);
        $consumerBalance['value'] = $this->util->subtract($consumerBalance['value'], $orderFidelityBalance);
        if($consumerBalance['value'] < $cashBack) 
            throw new \Exception("Não foi possível utilizar $cashBack de saldo fidelidade, consumidor possuí apenas $consumerBalance[value].");
    }

    public function getTables(Request $request, Response $response){
        $body = $request->getParameters();
        if(empty($body['subsidiaryCode'])) throw new \Exception("Parâmetro subsidiayCode não encontrado.");
        $final = array('tables' => array());
        $subsidiaryCode = $body['subsidiaryCode'];
        $tables = $this->orderService->getTables($subsidiaryCode);
        foreach($tables as $table)
            array_push($final['tables'], $table['NRMESA']);
        $response->addDataSet(new DataSet('', array($final)));
    }

    public function imprimeParcial(Request $request, Response $response){
        try{
            $body = $request->getParameters();
            if(empty($body['saleCode'])) throw new \Exception("Parâmetro saleCode não encontrado.");
            if(empty($body['tableNumber'])) throw new \Exception("Parâmetro tableNumber não encontrado.");
            $nrcomanda = $body['saleCode'];
            $nrmesa = $body['tableNumber'];
            $comandaInfo = $this->util->getComandaInfo($nrcomanda, $nrmesa);
            $cdfilial = $comandaInfo['CDFILIAL'];
            $nrvendarest = $comandaInfo['NRVENDAREST'];
            $cdloja = $comandaInfo['CDLOJA'];
            $dscomanda = $comandaInfo['DSCOMANDA'];
            $nrpessoas = $comandaInfo['NRPESMESAVEN'];
            $cdprodcouver = $comandaInfo['CDPRODCOUVER'];
            $cdprodconsum = $comandaInfo['CDPRODCONSUM'];
            $idcomisvenda = $comandaInfo['IDCOMISVENDA'];
            $vrcomisvenda = $comandaInfo['VRCOMISVENDA'];
            $idlugarmesa = $comandaInfo['IDLUGARMESA'];
            $idstmesaaux = $comandaInfo['IDSTMESAAUX'];
            $nmfanven = $comandaInfo['NMFANVEN'];

            $stComandaVens = "_".$nrcomanda."_";
            $stVendaRests  = "_".$nrvendarest."_";
            $stPosicoes = $impPosicao = "T";
            $produtosParcial = $this->util->getProdutosParcial($cdfilial, $stComandaVens, $stPosicoes, $impPosicao);

            $totalProdutosTaxa = 0;
			$totalProdutosTaxaDesc = 0;
			$totalSemDesconto = 0;
			$totalDesconto = 0;
            $totalSubsidy = 0;
			self::calculaValoresParcial($produtosParcial, $cdprodcouver, $cdprodconsum, $totalProdutosTaxa, $totalProdutosTaxaDesc, $totalSemDesconto, $totalDesconto, $totalSubsidy);
			$totalSemDesconto = round($totalSemDesconto, 2);
			$totalProdutosTaxa = round(round($totalProdutosTaxa, 2) - $totalProdutosTaxaDesc, 2);
			$total = round($totalSemDesconto - $totalDesconto, 2);

			$fidelityValue = 0;
            $fidelityDiscount = 0;
            
            $taxaDeServico = $this->taxaDeServico($idcomisvenda, $vrcomisvenda, $cdprodcouver, $cdprodconsum,
                                                $totalProdutosTaxa, $cdfilial, $nrvendarest, $nrcomanda,
                                                $idstmesaaux, $produtosParcial);
            $total += $taxaDeServico;
            
            $impressoraInfo = $this->util->getDadosImpressora();
            if(empty($impressoraInfo)) throw new \Exception("Não foi encontrada impressora parametrizada!");
            $nrseqimprloja = $impressoraInfo['NRSEQIMPRLOJA'];

            $tempoPermanencia = self::calculaTempoPermanencia($cdfilial, $nrvendarest, $nrcomanda);
			$horas = $tempoPermanencia["horas"];
			$minutos = $tempoPermanencia["minutos"];
			$segundos = $tempoPermanencia["segundos"];
            $modo = "M";
            $dadosImpressao = '';
            $totalPorPessoa = floatval($total) / floatval($nrpessoas);
            $retorno = $this->orderService->imprimeParcial($produtosParcial, $cdfilial, $stVendaRests,
                     $stComandaVens, $cdprodcouver, $modo, $nrvendarest, $nrcomanda, $cdloja, $nrmesa, $impPosicao, $nrpessoas,
                     round($totalSemDesconto - $totalDesconto, 2), $taxaDeServico, $total, $totalPorPessoa, $horas, $minutos,
                     $segundos, $dscomanda, $dadosImpressao, $fidelityValue, $idcomisvenda, $nmfanven, $idlugarmesa,
                     $vrcomisvenda, $nrseqimprloja);
            if(isset($retorno['error']) && $retorno['error'] === false){
                $response->addMessage(new Message("Conta impressa com sucesso.", self::GENERAL_MESSAGE));
            }else if (isset($retorno['error']) && $retorno['error'] === true){
                throw new \Exception($retorno['message']);
            }else{
                throw new \Exception("Houve um problema ao imprimir seu pedido.");
            }
        }catch(\Exception $e){
            $message = 'Erro ao imprimir parcial. Erro: '.$e->getMessage();
            $response->addMessage(new Message($message, self::ERROR_MESSAGE));
        }
        
    }

    public function printText(Request $request, Response $response){
        try{
            $body = $request->getParameters();
        
            $text = $body['text'];
    
            $dadosImpressora = $this->util->getDadosImpressora();
    
            $printerParams = $this->impressaoUtil->buscaParametrosImpressora($dadosImpressora['IDMODEIMPRES']);
            $options['bold'] = true;
            $options['letterType'] = '2';
            $string = $this->impressaoUtil->centraliza($printerParams, "") . $printerParams['comandoEnter'];
            $string .= $this->impressaoUtil->centraliza($printerParams, "") . $printerParams['comandoEnter'];
            $string .= $this->impressaoUtil->centraliza($printerParams, $text) . $printerParams['comandoEnter'];
            $string .= $this->impressaoUtil->centraliza($printerParams, "") . $printerParams['comandoEnter'];
            $string .= $this->impressaoUtil->centraliza($printerParams, "") . $printerParams['comandoEnter'];

            $this->impressaoUtil->imprimeNaoFiscal($string, $dadosImpressora, false, $options);
        }catch(\Exception $e){
            $message = 'Erro ao imprimir texto. Erro: '.$e->getMessage();
            $response->addMessage(new Message($message, self::ERROR_MESSAGE));
        }
        
    }

    private function calculaTempoPermanencia($cdfilial, $nrvendarest, $nrcomanda){
		$tempoPermanencia = $this->util->getTempoPermanencia($cdfilial, $nrvendarest, $nrcomanda);
        $now = new \DateTime();
		$entrada = \DateTime::createFromFormat("Y-m-d H:i:s.u", $tempoPermanencia["DATA"]);
		$intervalo = $entrada->diff($now);
		$intervaloHoras = str_pad((int)$intervalo->format('%R%h'), 2, '0', STR_PAD_LEFT);
		$intervaloMinutos = str_pad((int)$intervalo->format('%R%i'), 2, '0', STR_PAD_LEFT);
		$intervaloSegundos = str_pad((int)$intervalo->format('%R%s'), 2, '0', STR_PAD_LEFT);

		return array (
			"horas"    => $intervaloHoras,
			"minutos"  => $intervaloMinutos,
			"segundos" => $intervaloSegundos
		);
    }
    
    private function calculaValoresParcial($produtosParcial, $cdprodcouver, $cdprodconsum, &$totalProdutosTaxa, &$totalProdutosTaxaDesc, &$totalSemDesconto, &$totalDesconto, &$totalSubsidy){
		foreach ($produtosParcial as $item){
			if ($item['CDPRODUTO'] != $cdprodcouver && $item['CDPRODUTO'] != $cdprodconsum) {
				$vrProd = floatval(bcdiv($item['QTPRODCOMVEN'] * $item['VRPRECCOMVEN'], 1, 2)) + $item['VRACRCOMVEN'];
				if ($item['IDCOBTXSERV'] == 'S') {
					$totalProdutosTaxa += $vrProd;
					$totalProdutosTaxaDesc += $item['VRDESCCOMVEN'];
				}
				$totalSemDesconto += $vrProd;
				$totalDesconto += $item['VRDESCCOMVEN'];
                $totalSubsidy += floatval(bcmul($item['QTPRODCOMVEN'], $item['VRPRECCLCOMVEN'], 2));
			}
		}
    }
    
    private function taxaDeServico($idcomisvenda, $vrcomisvenda, $cdprodcouver, $cdprodconsum, $totalProdutos, $cdfilial, $nrvendarest, $nrcomanda, $IDSTMESAAUX, $produtosParcial){
		$taxaDeServico = 0;
		$params = array(
			'CDFILIAL' => $cdfilial,
			'NRVENDAREST' => $nrvendarest,
			'NRCOMANDA' => $nrcomanda
		);
    	$dadosComissao = $this->util->getComissaoVenda($cdfilial, $nrvendarest, $nrcomanda);

        if ($idcomisvenda === "S") {
           if (is_null($dadosComissao['VRCOMISPOR'])) {
            	$taxaCalculada = $this->util->truncate($totalProdutos * floatval($vrcomisvenda)/100, 2);
        	} else {
        		$taxaCalculada = $this->util->truncate($totalProdutos * floatval($dadosComissao["VRCOMISPOR"])/100, 2);
        	}
        }
        else if ($idcomisvenda === "G"){
            $taxaCalculada = 0;
            foreach ($produtosParcial as $item){
                if ($item['CDPRODUTO'] != $cdprodcouver && $item['CDPRODUTO'] != $cdprodconsum){
                    $vrProd = $item['QTPRODCOMVEN'] * ($item['VRPRECCOMVEN'] + $item['VRPRECCLCOMVEN']) - $item['VRDESCCOMVEN'];
                    if ($item['IDCOBTXSERV'] == 'S' && $item['IDVRINCIDECONTA'] == "S"){
                        $taxaCalculada += $this->util->truncate($vrProd * floatval($item['VRPERCCOMISVEND'])/100, 2);
                    }
                }
            }
        }

        if ($idcomisvenda !== "N"){
            // caso existir, retorna o valor calculado se for diferente de zero; se não, salva e retorna
            if ($IDSTMESAAUX !== 'O') {
				if (floatval($dadosComissao['VRCOMISVENDE']) != 0 || $dadosComissao['VRCOMISPOR'] > 0) {
					$taxaDeServico = $taxaCalculada;
				}
			} else {
				if ($taxaCalculada > 0) {
					$params = array(
						$taxaCalculada,
						$cdfilial,
						$nrvendarest,
						$nrcomanda
					);
					$this->entityManager->getConnection()->executeQuery("UPDATE_COMISSAO_VENDA", $params);
				}
				$taxaDeServico = $taxaCalculada;
			}
        }

		return $taxaDeServico;
    }
    
    public function recalculateComboDiscount($subsidiaryCode, $items){
        $recalculate = false;
        foreach($items as &$item){
            if(!empty($item['childItems'])){
                $sonFullValues = array();
                $totalDiscount = 0;
                $sonsTotalPrice = 0;
                $sonsToApplyDiscount = 0;
                $quantSons = 0;
                $fatherProduct = $item['productCode'];
                $returnGroups = $this->menuService->getComboGroupsBySubsidiaryDiscount($subsidiaryCode, $fatherProduct);
                if (!$returnGroups['error']) {
                    $comboGroups = $returnGroups['data'];
                    if (empty($comboGroups)) {
                        $useSubsidiaryGroups = false;
                        $returnGroups = $this->menuService->getComboGroupsByProductDiscount($fatherProduct);
                        if (!$returnGroups['error']) {
                            $comboGroups = $returnGroups['data'];
                        }
                    }
                }
                foreach($comboGroups as $group){
                    if($group['IDAPLICADESC'] === 'S')
                        $recalculate = true;
                }
                if($recalculate === true){
                    foreach($item['childItems'] as &$son){
                        $quantSons++;
                        foreach($comboGroups as $group){
                            $sonDiscount = $this->util->getSonDiscount($son['productCode'], $item['productCode'], $group['CDGRUPROMOC']);
                            if(!empty($sonDiscount)){
                                if($sonDiscount['IDPERVALORDES'] === 'P'){
                                    $percentApllied = bcsub(100, $sonDiscount['VRDESPRODPROMOC'], 2);
                                    $fullValue = bcdiv($son['unitValue'] * 100, $percentApllied, 2);
                                    $totalDiscount = bcadd($totalDiscount, bcsub($fullValue, $son['unitValue']), 2);
                                    $sonFullValues[$son['productCode']] = $fullValue;
                                }else{
                                    $fullValue = bcadd($son['unitValue'], $sonDiscount['VRDESPRODPROMOC'], 2);
                                    $totalDiscount = bcadd($totalDiscount, $sonDiscount['VRDESPRODPROMOC'], 2);
                                    $sonFullValues[$son['productCode']] = $fullValue;
                                }

                                if($group['IDAPLICADESC'] === 'S'){
                                    $sonsToApplyDiscount++;
                                    $son['APPLYDISCOUNT'] = true;
                                    $sonsTotalPrice = bcadd($sonsTotalPrice, $sonFullValues[$son['productCode']], 2);
                                }
                            }
                        }
                    }
                    $remaingDiscount = $totalDiscount;
                    unset($son);
                    if($quantSons > $sonsToApplyDiscount){
                        if($totalDiscount < $sonsTotalPrice){
                            if($sonsToApplyDiscount === 1){
                                foreach($item['childItems'] as &$son){
                                    if(isset($sonFullValues[$son['productCode']])){
                                        $son['unitValue'] = $sonFullValues[$son['productCode']];
                                        if(!empty($son['APPLYDISCOUNT'])){
                                            $son['unitValue'] = bcsub($sonFullValues[$son['productCode']], $totalDiscount, 2);
                                        }
                                    }
                                }
                                unset($son); 
                            }else{
                                foreach($item['childItems'] as &$son){
                                    if(isset($sonFullValues[$son['productCode']])){
                                        $son['unitValue'] = $sonFullValues[$son['productCode']];
                                        if(!empty($son['APPLYDISCOUNT'])){
                                            $productDiscount = bcdiv(
                                                (
                                                    (
                                                        (
                                                            $son['unitValue']
                                                        ) * $son['quantity']
                                                    ) * $totalDiscount
                                                ) / $sonsTotalPrice, 1, 2
                                            );
                                            $son['unitValue'] = bcdiv(bcsub($son['unitValue'], $productDiscount, 2), 1, 2);
                                            $remaingDiscount = bcdiv(bcsub($remaingDiscount, $productDiscount, 2), 1, 2);
                                        }
                                    }
                                    
                                }
                                unset($son);
                                while($remaingDiscount > 0){
                                    foreach($item['childItems'] as &$son){
                                        if($remaingDiscount > 0){
                                            if($son['unitValue'] > 0.01 && !empty($son['APPLYDISCOUNT'])){
                                                $son['unitValue'] = bcdiv(bcsub($son['unitValue'], 0.01, 2), 1, 2);
                                                $remaingDiscount = bcdiv(bcsub($remaingDiscount, 0.01, 2), 1, 2);
                                            }
                                        }
                                    }
                                    unset($son);
                                }
                            }
                        }
                    }
                }
            }
        }
        unset($item);
        return $items;
    }

    public function getWaiters(Request $request, Response $response){
        try {
            $waiters = $this->orderService->getWaiters();
            $response->addDataSet(
                new DataSet(
                    '',
                    $waiters
                )
            );
        } catch (\Exception $e) {
            $response->addMessage(new Message($e->getMessage(), self::ERROR_MESSAGE));
        }
    }

    public function getOrder(Request $request, Response $response){
        try {
            if (isset($_GET['saleCode'])) $nrcomanda = $_GET['saleCode'];
            else $nrcomanda = 'T';

            if (isset($_GET['tableNumber'])) $nrmesa = $_GET['tableNumber'];
            else throw new \Exception('Mesa não informada.');
            $fileName = $nrcomanda;
            $this->util->saveLog($nrcomanda . '-' . $nrmesa, 'Comanda' . date("dmY"), $fileName . '-');
            $order = $this->orderService->getOrderProducts($nrcomanda, $nrmesa);
            $response->addDataSet(
                new DataSet(
                    '',
                    $order
                )
            );
        } catch (\Exception $e) {
            $message = 'Erro ao recuperar comanda. ' . $e->getMessage();
            $response->addMessage(new Message($message, self::ERROR_MESSAGE));
        }
    }

    public function closeOrder(Request $request, Response $response){
        try {
            $this->util->beginTransaction();
            $inputJSON = file_get_contents('php://input');
            $input = json_decode($inputJSON, true);
            if (isset($input['subsidiaryCode'])) $cdfilial = $input['subsidiaryCode'];
            else throw new \Exception('Filial não informada.');
            
            if (isset($input['saleCode'])) $nrcomanda = $input['saleCode'];
            else throw new \Exception('Comanda não informada.');
            
            if (isset($input['posCode'])) $cdcaixa = $input['posCode'];
            else throw new \Exception('Caixa não informado.');
            
            $order = $this->orderService->getOrder($cdfilial, $nrcomanda);
            
            if(empty($order)) throw new \Exception("Comanda não encontrada! Verifique os dados de entrada.");

            $nrvendarest = $order['NRVENDAREST'];
            $xml = '';
            $status = 'N';
            $message = '';
            $dtentrvenda = null;

            if($order['IDSTCOMANDA'] === 'X'){
                $cdcaixa = $order['CDCAIXA'];
                $saleDetails = $this->util->getSaleDetails($cdfilial, $nrcomanda, $nrvendarest, $this->logPath, $cdcaixa);
                $nrseqvenda  = $saleDetails['NRSEQVENDA'];
                $cdcaixa     = $saleDetails['CDCAIXA'];
                $status      = $saleDetails['IDSTATUSNFCE'];
                $dtentrvenda = $saleDetails['DTENTRVENDA'];
                $xml         = $saleDetails['DSARQXMLNFCE'];
                $xml         = str_replace('"',"'", $xml);
                $message = "O pedido ($cdfilial-$nrcomanda) já foi fechado, a consulta encontra-se disponível no endpoint '/getSale'.";

                $response->addMessage(new Message($message, self::ERROR_MESSAGE));

            } else {
                $payments = $this->orderService->getOrderPayments($cdfilial, $nrvendarest);
                
                $paymentsValue = 0;
                if(!empty($payments)) {
                    foreach ($payments as $payment) {
                        $paymentsValue = bcadd($payment['VRMOVIVEND'], $paymentsValue, 2);
                    }
                }
                
                $cashBackValue = $this->orderService->getOrderCashback($cdfilial, $nrvendarest, $nrcomanda);
                $items = $this->orderService->getOrderItems($nrvendarest, $nrcomanda, $cdfilial);
                $productsValue = 0;
                foreach ($items as $item) {
                    $productsValue = $this->util->add($productsValue, $item['TOTAL']);
                }
                $totalPayment = $this->util->add($cashBackValue['VRDESCFID'], $paymentsValue);
                $totalOrderValue = $this->util->add($productsValue, $order['VRACRCOMANDA']);
                if($totalPayment < $totalOrderValue){
                    $remaingValue = $this->util->subtract($totalOrderValue, $totalPayment);
                    throw new \Exception("Ainda devem ser pago(s) R$$remaingValue para que a comanda seja fechada.");
                } 
                $this->registerReceivedOrder('Close', date('dmy'), $order);
                $this->registerPendingOrder('Close', date('dmy'), $order);

                $response->addMessage(new Message("O pedido ($cdfilial-$nrcomanda) foi incluído para fechamento.", self::GENERAL_MESSAGE));
            }

        } catch (\Exception $e) {
            $this->util->rollBack();
            $message = 'Erro ao solicitar o fechamento da comanda. ' . $e->getMessage();
            $response->addMessage(new Message($message, self::ERROR_MESSAGE));
        }
    }

    public function registerReceivedOrder($function, $today, $order){
        $receivedPath = "{$this->logPath}/$function/Received/".date('Y')."/".date('m')."/".date('d')."/";
        if(!is_dir($receivedPath)) mkdir($receivedPath, 0777, true);
        
        $fileName = date('Ymd-H-i-s')."-".$order['NRCOMANDA'].".json";
        $fileData = array(
            "PATH" => $receivedPath,
            "FILENAME" => $fileName,
            "order" => $order
        );
        file_put_contents($receivedPath.$fileName, json_encode($fileData));
        return array('error' => false, 'fileName' => $fileName, 'message' => "");
    }

    public function registerPendingOrder($function, $today, $order){
        $pendingPath = "{$this->logPath}/$function/Pending/";
        if(!is_dir($pendingPath)) mkdir($pendingPath, 0777, true);
        
        $fileName = date('Ymd-H-i-s')."-".$order['NRCOMANDA'].".json";
        $fileData = array(
            "PATH" => $pendingPath,
            "FILENAME" => $fileName,
            "order" => $order
        );
        file_put_contents($pendingPath.$fileName, json_encode($fileData));
        return array('error' => false, 'message' => "");
    }

    private function validatePayments($cdfilial, $nrvendarest, $nrcomanda, $deliveryFee){
        $productsValue = $this->orderService->getItemsValue($cdfilial, $nrvendarest, $nrcomanda);
        $orderPayments = $this->orderService->getOrderPayments($cdfilial, $nrvendarest);
        // $orderCashBack = $this->orderService->getOrderCashBack($cdfilial, $nrvendarest, $nrcomanda);
        $paymentsValue = 0;
        $money = $this->util->getMoney();
        $hasMoney = false;
        foreach ($orderPayments as $orderPayment) {
            $hasMoney = $orderPayment['CDTIPORECE'] === $money;
            $paymentsValue = $this->util->add($paymentsValue, $orderPayment['VRMOVIVEND']);
        }
        // $totalPaymentsValue = $this->util->add($paymentsValue, $orderCashBack['VRDESCFID']);
        // $totalPaymentsValue = $paymentsValue;
        $totalOrderValue = $this->util->add($productsValue, $deliveryFee);
        if($paymentsValue != $totalOrderValue){
            if($paymentsValue > $totalOrderValue){
                if($hasMoney === false){
                    throw new \Exception("Valor do pagamento inválido. Valor total da compra: R$$totalOrderValue.");
                }
            } else if($paymentsValue < $totalOrderValue){
                throw new \Exception("Valor do pagamento inválido. Valor total da compra: R$$totalOrderValue.");
            }
            
        } 
    }

    public function fechaPedidos(){
        $controlPath = "{$this->logPath}\\..\\CLOSE-CTRL-FILE.dat";
        if(!file_exists($controlPath)) $this->updateControlFile(false);
        $controlFile = json_decode(base64_decode(file_get_contents($controlPath)), true);
        
        if($controlFile['lock']) die;
        $executionErrors = array();
        $this->util->saveLog('Início serviço fechamento de pedidos','Execution/'.date("Y/m"),'CLOSE-');
        
        $this->updateControlFile(true);
        $processingErrors = array();
        try{
            $pedidos = $this->recuperaPedidos('Close');
            
            foreach($pedidos as $pedido){
                
                try {
                    $fechamento = $this->orderService->closeOrder($pedido['order'], $pedido['order']['CDFILIAL'], $pedido['order']['NRCOMANDA']);
                    
                    if(!empty($fechamento['mensagemNfce'])) $fechamento['message'] .= $fechamento['mensagemNfce'];
                    
                    if($fechamento['error'] || (!empty($fechamento['IDSTATUSNFCE']) && $fechamento['IDSTATUSNFCE'] == "R")){
                        
                        if(!empty($fechamento['IDSTATUSNFCE']) && $fechamento['IDSTATUSNFCE'] == "R") $this->createCancelOrder($pedido['order']['CDFILIAL'], $pedido['order']['CDCAIXA'], $pedido['order']['NRCOMANDA'], $fechamento['NRSEQVENDA']); 
                        
                        $message = $pedido['order']['DSCOMANDA'].' - '.$pedido['order']['NRCOMANDA'].' - '.$fechamento['message'];
                        $this->util->saveLog($message,'Error','CLOSE-');
                        $this->util->beginTransaction();
                        try{
                            $this->util->insertLogOperFos($pedido['order']['CDFILIAL'], $pedido['order']['CDCAIXA'], 'FEC_PED', "Erro Fechamento ".$pedido['order']['CDFILIAL']." - ".$pedido['order']['NRCOMANDA']." - ". $fechamento['message']);
                            $this->util->commit();
                        } catch(\Exception $e){
                            $this->util->rollback();
                        }
                        $this->error('Close', date('dmy'), $pedido['PATH'], $pedido['FILENAME'], $fechamento['message']);
                        
                        
                    }else{
                        $this->finish('Close', date('dmy'), $pedido['PATH'], $pedido['FILENAME']);
                    }
                } catch(\Exception $e){
                    array_push($processingErrors, $pedido['order']['NRCOMANDA']." - ".$e->getMessage());
                    $this->error('Close', date('dmy'), $pedido['PATH'], $pedido['FILENAME'], $e->getMessage());
                }
            }
            if(!empty($processingErrors)){
                $message = "";
                foreach($processingErrors as $error) $message .= $error;
                throw new \Exception($message);
            }
        }catch(\Exception $e){
            $this->util->saveLog($e->getMessage(),'Error','CLOSE-');
        }
        
        $this->util->saveLog('Fim serviço fechamento de pedidos.','Execution/'.date("Y/m"),'CLOSE-');
        $this->updateControlFile(false);
    }

    public function cancelaPedidos(){
        $controlPath = "{$this->logPath}\\..\\CANCEL-CTRL-FILE.dat";
        if(!file_exists($controlPath)) $this->updateCancelControlFile(false);
        $controlFile = json_decode(base64_decode(file_get_contents($controlPath)), true);
        
        if($controlFile['lock']) die;
        $executionErrors = array();
        $this->util->saveLog('Início serviço cancelamento de pedidos','Execution/'.date("Y/m"),'CANCEL-');
        
        $this->updateCancelControlFile(true);
        $processingErrors = array();
        try{
            $pedidos = $this->recuperaPedidos('Cancel');
            
            foreach($pedidos as $pedido){
                
                try{
                    $cancelamento = $this->orderService->cancelOrder($pedido['order']['CDFILIAL'], $pedido['order']['CDCAIXA'], $pedido['order']['NRSEQVENDA'], $pedido['order']['CDOPERADOR'], $pedido['order']['CDSUPERVISOR']);

                    if(!empty($cancelamento['mensagemNfce'])) $cancelamento['message'] .= $cancelamento['mensagemNfce'];

                    if($cancelamento['error']){
                        
                        $message = $pedido['order']['NRCOMANDA'].' - '.$cancelamento['message'];
                        $this->util->saveLog($message,'Error','CANCEL-');
                        $this->util->beginTransaction();
                        try{
                            $this->util->insertLogOperFos($pedido['order']['CDFILIAL'], $pedido['order']['CDCAIXA'], 'CAN_PED', "Erro Cancelamento ".$pedido['order']['CDFILIAL']." - ".$pedido['order']['NRCOMANDA']." - ".$pedido['order']['NRSEQVENDA'].": ".$cancelamento['message']);
                            $this->util->commit();
                        } catch(\Exception $e){
                            $this->util->rollback();
                        }
                        $this->error('Cancel', date('dmy'), $pedido['PATH'], $pedido['FILENAME'], $cancelamento['message']);
                    } else{
                        $this->finish('Cancel', date('dmy'), $pedido['PATH'], $pedido['FILENAME']);
                        
                    }
                } catch(\Exception $e){
                    array_push($processingErrors, $pedido['order']['NRCOMANDA']." - ".$e->getMessage());
                    $this->error('Cancel', date('dmy'), $pedido['PATH'], $pedido['FILENAME'], $e->getMessage());
                }
            }
            if(!empty($processingErrors)){
                $message = "";
                foreach($processingErrors as $error) $message .= $error;
                throw new \Exception($message);
            }
        }catch(\Exception $e){
            $this->util->saveLog($e->getMessage(),'Error','CANCEL-');
        }
        
        $this->util->saveLog('Fim serviço cancelamento de pedidos.','Execution/'.date("Y/m"),'CANCEL-');
        $this->updateCancelControlFile(false);
    }

    public function getSale(Request $request, Response $response){
        try {
            $inputJSON = file_get_contents('php://input');
            $input = json_decode($inputJSON, true);
            if (isset($input['subsidiaryCode'])) $cdfilial = $input['subsidiaryCode'];
            else throw new \Exception('Filial não informada.');
            
            if (isset($input['saleCode'])) $nrcomanda = $input['saleCode'];
            else throw new \Exception('Comanda não informada.');

            if (isset($input['posCode'])) $cdcaixa = $input['posCode'];
            else throw new \Exception('Caixa não informado.');
            
            $controlPath = "{$this->logPath}\\..\\CLOSE-CTRL-FILE.dat";
            if(!file_exists($controlPath)) throw new \Exception("Serviço de fechamento de pedidos inativo.");
            $controlFile = json_decode(base64_decode(file_get_contents($controlPath)), true);
            if((time() - $controlFile['expiresIn']) > self::MAX_CONTROL_FILE_UPDATE_TIME) throw new \Exception("Serviço de fechamento de pedidos inativo.");

            $order = $this->orderService->getOrder($cdfilial, $nrcomanda);
            
            $nrvendarest = $order['NRVENDAREST'];
            $errorFile = null;
            $errorPath = "{$this->logPath}\\Close\\Error\\".date('Y')."\\".date('m')."\\".date('d')."\\";
            if(file_exists($errorPath)){
                chdir($errorPath);
                $errorFile = glob("*-".$nrcomanda.".json");
            }

            if($order['IDSTCOMANDA'] === 'X'){
                $cdcaixa = $order['CDCAIXA'];
                $saleDetails = $this->util->getSaleDetails($cdfilial, $nrcomanda, $nrvendarest, $this->logPath, $cdcaixa);
                $nrseqvenda  = $saleDetails['NRSEQVENDA'];
                $cdcaixa     = $saleDetails['CDCAIXA'];
                $status      = $saleDetails['IDSTATUSNFCE'];
                $dtentrvenda = $saleDetails['DTENTRVENDA'];
                $xml         = $saleDetails['DSARQXMLNFCE'];
                $xml         = str_replace('"',"'", $xml);
                $message = "";
                if(!empty($errorFile)){
                    $file = json_decode(file_get_contents($errorPath.$errorFile[0]), true);
                    $message = $file['error'];
                }
            } else if(!empty($errorFile)){
                $file = json_decode(file_get_contents($errorPath.$errorFile[0]), true);
                $response->addMessage(new Message("Erro no fechamento do pedido: ".$file['error'], self::ERROR_MESSAGE));
            } else $response->addMessage(new Message("O pedido ($cdfilial-$nrcomanda) está em processo de fechamento, tente novamente mais tarde.", self::GENERAL_MESSAGE));

            $storeName = $this->util->getStoreName($cdfilial, $cdcaixa, $order['CDLOJA']);
            
            $fidelityPoints = 0;
            $personalCredit = 0;
            
            if(!empty($order['CDCONSUMIDOR'])){
                $personalCredit = $this->util->getConsumerBalance($order['CDCLIENTE'], $order['CDCONSUMIDOR'], '001');
                $fidelityPoints = $this->util->getConsumerFidelity($order['CDCLIENTE'], $order['CDCONSUMIDOR']);
                if(!empty($fidelityPoints)) $fidelityPoints = $fidelityPoints['value'];
            }
            
            $response->addDataSet(
                new DataSet(
                    '',
                    array(
                        'message'        => $message,
                        'consumerName'   => $order['NMCONSUMIDOR'],
                        'saleDate'       => $dtentrvenda,
                        'fidelityPoints' => $fidelityPoints,
                        'personalCredit' => $personalCredit,
                        'saleSeq'        => $nrseqvenda,
                        'posCode'        => $cdcaixa,
                        'posName'        => $storeName['NMCAIXA'],
                        'subsidiaryCode' => $cdfilial,
                        'subsidiaryName' => $storeName['NMFILIAL'],
                        'storeCode'      => $order['CDLOJA'],
                        'storeName'      => $storeName['NMLOJA'],
                        'status'         => $status,
                        'pagerId'        => $order['CDSENHAPED'],
                        'xml'            => $xml
                    )
                )
            );
            
        } catch (\Exception $e) {
            // $this->util->rollBack();
            $message = 'Erro ao buscar o fechamento da comanda. ' . $e->getMessage();
            $response->addMessage(new Message($message, self::ERROR_MESSAGE));
        }
    }

    public function recuperaPedidos($function){
		$notas = array();
		$directory = "{$this->logPath}/$function/pending/";
		if(is_dir($directory)){
			$scanned_directory = array_diff(scandir($directory), array('..', '.'));
			foreach($scanned_directory as $file){
				$fileData = json_decode(file_get_contents("$directory/$file"), true);
				array_push($notas, $fileData);
			}
		}
		
        return $notas;
	}

    public function finish($function, $dataProcessamento, $path, $fileName, $removeFile = true){
        $processedPath = "{$this->logPath}/$function/Processed/".date("Y/m/d")."/";
        if(!is_dir($processedPath)) mkdir($processedPath, 0777, true);
        copy($path.$fileName, $processedPath.$fileName);
        if($removeFile) unlink(realpath($path.$fileName));
        
	}

    public function error($function, $dataProcessamento, $path, $fileName, $message){
        $errorPath = "{$this->logPath}/$function/Error/".date('Y')."/".date('m')."/".date('d')."/";
        if(!is_dir($errorPath)) mkdir($errorPath, 0777, true);
        $file = file_get_contents($path.$fileName);
        $file = json_decode($file, true);
        $file['error'] = $message;
        file_put_contents($errorPath.$fileName, json_encode($file));
        unlink(realpath($path.$fileName));
	}

    public function updateControlFile($alreadyRunning){
        $value = array('expiresIn' => time() + 60, 'lock' => $alreadyRunning);
        file_put_contents("{$this->logPath}\\..\\CLOSE-CTRL-FILE.dat", base64_encode(json_encode($value)));
    }

    public function updateCancelControlFile($alreadyRunning){
        $value = array('expiresIn' => time() + 60, 'lock' => $alreadyRunning);
        file_put_contents("{$this->logPath}\\..\\CANCEL-CTRL-FILE.dat", base64_encode(json_encode($value)));
    }
}
