<?php

namespace Service;

class Menu {
    
    protected $connection;
    protected $util;
    
    public function __construct($connection, \Odhen\API\Util\Util $util) {
        $this->connection = $connection;
        $this->util = $util;
    }
    
    public function getGroups($CDFILIAL, $CDCAIXA) {
		$params = array(
			'CDFILIAL' => $CDFILIAL,
			'CDCAIXA' => $CDCAIXA
		);
		try {
			if($this->checkForConfTela()['error'] == false){
				$confTelaCaixa = $this->getConfTelaCaixa($CDFILIAL, $CDCAIXA);
				if(!empty($confTelaCaixa['data'])){
					$params['DTINIVIGENCIA'] = $confTelaCaixa['data'][0]['DTINIVIGENCIA'];
					$groups = $this->connection->fetchAll(\Util\Query::GET_GROUPS_CONFTELA, $params);
				} else {
					$groups = $this->connection->fetchAll(\Util\Query::GET_GROUPS, $params);
				}
			} else {
				$groups = $this->connection->fetchAll(\Util\Query::GET_GROUPS, $params);
			}
			if(empty($groups)) throw new \Exception("Não foram encontrados grupos de produtos para esta filial/caixa");
			$return = array(
				'error' => false,
				'data'  => $groups
			);
		} catch (\Exception $e) {
			$return = array(
				'error' => true,
				'message' => 'Erro ao buscar grupos de produtos. Erro: '.$e->getMessage()
			);
		}
		return $return;
	}
    
    public function getGroupProducts($CDFILIAL, $CDCAIXA, $group, $CDLOJA) {
		$params = array(
			'CDFILIAL' => $CDFILIAL,
			'CDCAIXA' => $CDCAIXA,
			'NRPGCONFTELA' => $group['NRPGCONFTELA'],
			'NRBUTTON' => $group['NRBUTTON'],
			'CDLOJA' => $CDLOJA
		);
		
		try {    	
			if($this->checkForConfTela()['error'] == false){
				$confTelaCaixa = $this->getConfTelaCaixa($CDFILIAL, $CDCAIXA);
				if(!empty($confTelaCaixa['data'])){
					$params['DTINIVIGENCIA'] = $confTelaCaixa['data'][0]['DTINIVIGENCIA'];
					if ($this->util->databaseIsOracle()) $products = $this->connection->fetchAll(\Util\Query::ORACLE_GET_GROUP_PRODUCTS_CONFTELACX, $params);
					else $products = $this->connection->fetchAll(\Util\Query::SQL_GET_GROUP_PRODUCTS_CONFTELACX, $params);
				} else {
					if ($this->util->databaseIsOracle()) $products = $this->connection->fetchAll(\Util\Query::ORACLE_GET_GROUP_PRODUCTS, $params);
					else $products = $this->connection->fetchAll(\Util\Query::SQL_GET_GROUP_PRODUCTS, $params);
				}
			} else {
				if ($this->util->databaseIsOracle()) $products = $this->connection->fetchAll(\Util\Query::ORACLE_GET_GROUP_PRODUCTS, $params);
				else $products = $this->connection->fetchAll(\Util\Query::SQL_GET_GROUP_PRODUCTS, $params);
			}
			$return = array(
				'error' => false,
				'data' => $products
			);
		} catch (\Exception $e) {
			$return = array(
				'error' => true,
				'message' => 'Erro ao buscar produtos do grupo. Erro: '.$e->getMessage()
			);
		}
		return $return;
	}
    
    public function getSubGroups($CDFILIAL, $CDCAIXA, $group) {
    	$params = array(
			'CDFILIAL' => $CDFILIAL,
			'CDCAIXA' => $CDCAIXA,
			'NRPGCONFTELA' => $group['NRPGCONFTELA'],
			'NRBUTTON' => $group['NRBUTTON']
		);
		
		try {
			if($this->checkForConfTela()['error'] == false){
				$confTelaCaixa = $this->getConfTelaCaixa($CDFILIAL, $CDCAIXA);
				if(!empty($confTelaCaixa['data'])){
					$params['DTINIVIGENCIA'] = $confTelaCaixa['data'][0]['DTINIVIGENCIA'];
					$subGroup = $this->connection->fetchAll(\Util\Query::GET_SUBGROUPS_CONFTELACX, $params);
				} else {
					$subGroup = $this->connection->fetchAll(\Util\Query::GET_SUBGROUPS, $params);
				}
			} else {
				$subGroup = $this->connection->fetchAll(\Util\Query::GET_SUBGROUPS, $params);
			}
			
			$return = array(
				'error' => false,
				'data' => $subGroup
			);
		} catch (\Exception $e) {
			$return = array(
				'error' => true,
				'message' => 'Erro ao buscar subgrupos do grupo. Erro: '.$e->getMessage()
			);
		}
		
		return $return;
    }
    
    public function getSubGroupProducts($CDFILIAL, $CDCAIXA, $group, $subGroup, $CDLOJA) {
		$params = array(
			'CDFILIAL' => $CDFILIAL,
			'CDCAIXA' => $CDCAIXA,
			'NRPGCONFTELA' => $group['NRPGCONFTELA'],
			'NRBUTTON' => $group['NRBUTTON'],
			'NRPGCONFTAUX' => $subGroup['NRPGCONFTELA'],
			'NRBUTTONAUX' => $subGroup['NRBUTTON'],
			'CDLOJA' => $CDLOJA
		);
		
		try {
			if($this->checkForConfTela()['error'] == false){
				$confTelaCaixa = $this->getConfTelaCaixa($CDFILIAL, $CDCAIXA);
				if(!empty($confTelaCaixa['data'])){
					$params['DTINIVIGENCIA'] = $confTelaCaixa['data'][0]['DTINIVIGENCIA'];
					if ($this->util->databaseIsOracle()) $products = $this->connection->fetchAll(\Util\Query::ORACLE_GET_SUBGROUP_PRODUCTS_CONFTELACX, $params);
					else $products = $this->connection->fetchAll(\Util\Query::SQL_GET_SUBGROUP_PRODUCTS_CONFTELACX, $params);
				} else {
					if ($this->util->databaseIsOracle()) $products = $this->connection->fetchAll(\Util\Query::ORACLE_GET_SUBGROUP_PRODUCTS, $params);
					else $products = $this->connection->fetchAll(\Util\Query::SQL_GET_SUBGROUP_PRODUCTS, $params);
				}
			} else {
				// verifica qual banco de dados está sendo utilizado
				if ($this->util->databaseIsOracle()) $products = $this->connection->fetchAll(\Util\Query::ORACLE_GET_SUBGROUP_PRODUCTS, $params);
				else $products = $this->connection->fetchAll(\Util\Query::SQL_GET_SUBGROUP_PRODUCTS, $params);
			}
			
			$return = array(
				'error' => false,
				'data' => $products
			);
		} catch (\Exception $e) {
			$return = array(
				'error' => true,
				'message' => 'Erro ao buscar produtos do subgrupo. Erro: '.$e->getMessage()
			);
		}
		
		return $return;
	}

    public function checkProductObservations($productCode, $subsidiaryCode, $storeCode){
        $params = array(
            'CDFILIAL' => $subsidiaryCode,
            'CDLOJA' => $storeCode,
            'CDPRODUTO' => $productCode
        );
       
        try {
        	if ($this->util->databaseIsOracle()) {
       		 	$result = $this->connection->fetchAll(\Util\Query::ORACLE_GET_CHECK_PRODUCT_OBSERVATIONS, $params);
       	    } else {
       		 	$result = $this->connection->fetchAll(\Util\Query::SQL_GET_CHECK_PRODUCT_OBSERVATIONS, $params);
       	    }
	        return count($result) > 0;
		} catch (\Exception $e) {
			return false;
		}
		
    }
    
    public function getActivePrincingTable($CDFILIAL, $CDLOJA) {
        $params = array(
            'CDFILIAL' => $CDFILIAL,
            'CDLOJA' => $CDLOJA
        );
        
        try {
	        $tabelaLoja = $this->connection->fetchAssoc(\Util\Query::STORE_PRICING_TABLE, $params);
	        if ($tabelaLoja && $tabelaLoja["CDTABEPREC"]) {
	            $DTINIVGPREC = self::checkPrincingTable($CDFILIAL, $tabelaLoja["CDTABEPREC"]);
	            if (!$DTINIVGPREC) {
	                $return = array(
	                    'error' => true,
	                    'message' => 'Tabela de preços associada à loja está inativa.'
	                );
	            } else {
	                $return = array(
	                    'error' => false,
	                    'CDFILIAL' => $CDFILIAL,
	                    'CDTABEPREC' => $tabelaLoja["CDTABEPREC"],
	                    'DTINIVGPREC' => $DTINIVGPREC
	                );
	            }
	        } else {
	        	$return = array(
	                'error' => true,
	                'message' => 'Não há tabela de preços cadastrada para a loja. Verifique o campo CDTABEPREC da tabela LOJA.'
	            );
	        }
        } catch (\Exception $e) {
        	$return = array(
                'error' => true,
                'message' => 'Erro ao buscar tabela de preços da loja '.$CDLOJA.'. Erro: '.$e->getMessage()
            );
        }
        
        return $return;
	}

	public function checkPrincingTable($CDFILIAL, $CDTABEPREC) {
        // valida vigência
        $params = array(
            ':CDFILIAL'   => $CDFILIAL,
            ':CDTABEPREC' => $CDTABEPREC
        );


        // verifica qual banco de dados está sendo utilizado
        if ($this->util->databaseIsOracle()) {
        	$return = $this->connection->fetchAssoc(\Util\Query::ORACLE_TABELA_VENDA, $params);
        } else {
        	$return = $this->connection->fetchAssoc(\Util\Query::SQL_TABELA_VENDA, $params);
        }

        if ($return) {
            return $return["DTINIVGPREC"];
        } else {
            return false;
        }
    }
    
    public function getPrices($CDFILIAL, $CDTABEPREC, $DTINIVGPREC, $CDTIPOCONSPD, $timeOfDay, $isCombo) {
		$currentDate = new \DateTime();
		$params = array(
            'CDFILIAL' => $CDFILIAL,
            'DTFERIFILI' => $currentDate->format('d/m/Y')
        );
        if ($this->util->databaseIsOracle()) {
            $return = $this->connection->fetchAssoc(\Util\Query::ORACLE_HOLIDAY, $params);
        } else {
            $return = $this->connection->fetchAssoc(\Util\Query::SQL_HOLIDAY, $params);
        }
        if ($return) {
 			$day = 'F';
        } else {
            $day = $currentDate->format('N') % 7 + 1;
        }
		$params = array(
	        'CDFILIAL' => $CDFILIAL,
	        'CDTABEPREC' => $CDTABEPREC,
	        'DTINIVGPREC' => $DTINIVGPREC,
	        'CDTIPOCONSPD' => $CDTIPOCONSPD,
	        'DAYOFWEEK' => strval($day),
	        'TIMEOFDAY' => $timeOfDay
	    );
	    try {
		    if ($this->util->databaseIsOracle()) {
				if($isCombo === true)
					$prices = $this->connection->fetchAll(\Util\Query::ORACLE_GET_PRICES_COMBOS, $params);
				else
					$prices = $this->connection->fetchAll(\Util\Query::ORACLE_GET_PRICES, $params);
	        } else {
				if($isCombo === true)
					$prices = $this->connection->fetchAll(\Util\Query::SQL_GET_PRICES_COMBOS, $params);
				else
					$prices = $this->connection->fetchAll(\Util\Query::SQL_GET_PRICES, $params);
	        }
	        if (!empty($prices)) {
	    		$indexedPrices = array();
	            foreach ($prices as $product) {
	            	$indexedProduct = !empty($indexedPrices[$product['CDPRODUTO']]) ? $indexedPrices[$product['CDPRODUTO']] : null;
	            	if(!empty($indexedProduct)){
	            		if($indexedProduct['NRDIASEMANPR'] == 'T'){
	                		$indexedPrices[$product['CDPRODUTO']] = $product;
	            		}
	            	}
	            	else
            			$indexedPrices[$product['CDPRODUTO']] = $product;
				}
	            $return = array(
	                'error' => false,
	                'data' => $indexedPrices
	            );
	        } else {
	            $return = array(
	                'error' => true,
	                'message' => 'Não há produtos para a tabela de preços vigente.'
	            );
	        }
	    } catch (\Exception $e) {
        	$return = array(
                'error' => true,
                'message' => 'Erro ao buscar os preços dos produtos da tabela de preços da loja '.$CDFILIAL.'. Erro: '.$e->getMessage()
            );
        }
        return $return;
	}

    public function getObservationsByProduct($CDGRPOCOR, $CDPRODUTO, $CDFILIAL, $CDLOJA){
        $params = array(
            'CDGRPOCOR' => $CDGRPOCOR,
            'CDPRODUTO' => $CDPRODUTO,
            'CDFILIAL' => $CDFILIAL,
            'CDLOJA' => $CDLOJA	
        );
        try {
        	if ($this->util->databaseIsOracle()) {
	            $observations = $this->connection->fetchAll(\Util\Query::ORACLE_GET_PRODUCT_OBSERVATIONS, $params);
	        } else {
	            $observations = $this->connection->fetchAll(\Util\Query::SQL_GET_PRODUCT_OBSERVATIONS, $params);
	        }
 			$return = array(
	            'error' => false,
	            'data' => $observations
	        );
 		}catch (\Exception $e) {
        	$return = array(
                'error' => true,
                'message' => 'Erro ao buscar observacoes do produto. Erro: '.$e->getMessage()
            );
        } 
        return $return;
    }

    public function getObservationGroups($CDFILIAL, $CDLOJA){
        $params = array(
            'CDFILIAL' => $CDFILIAL,
            'CDLOJA' => $CDLOJA
        );
        try {
		    if ($this->util->databaseIsOracle()) {
	            $groups = $this->connection->fetchAll(\Util\Query::ORACLE_GET_STORE_OBSERVATIONS_GROUPS, $params);
	        } else {
	            $groups = $this->connection->fetchAll(\Util\Query::SQL_GET_STORE_OBSERVATIONS_GROUPS, $params);
	        }
 			$return = array(
	            'error' => false,
	            'data' => $groups
	        );
 		}catch (\Exception $e) {
        	$return = array(
                'error' => true,
                'message' => 'Erro ao buscar grupos de observacoes. Erro: '.$e->getMessage()
            );
        } 

        return $return;
    }

    public function getComboGroupsBySubsidiary($CDFILIAL, $CDIDENTBUTON){
    	$params = array(
            'CDFILIAL' => $CDFILIAL,
            'CDIDENTBUTON' => $CDIDENTBUTON
        );
        try {
      		$groups = $this->connection->fetchAll(\Util\Query::SQL_GET_COMBO_GROUPS_SUBSIDIARY, $params);
 			$return = array(
	            'error' => false,
	            'data' => $groups
	        );
 		}catch (\Exception $e) {
        	$return = array(
                'error' => true,
                'message' => 'Erro ao buscar grupos de combo do produto. Erro: '.$e->getMessage()
            );
        } 

        return $return;
    }

    public function getComboGroupsByProduct($CDIDENTBUTON){
    	$params = array(
            'CDIDENTBUTON' => $CDIDENTBUTON
        );
        try {
      		$groups = $this->connection->fetchAll(\Util\Query::SQL_GET_COMBO_GROUPS_PRODUCT, $params);
 			$return = array(
	            'error' => false,
	            'data' => $groups
	        );
 		}catch (\Exception $e) {
        	$return = array(
                'error' => true,
                'message' => 'Erro ao buscar grupos de combo do produto. Erro: '.$e->getMessage()
            );
        } 

        return $return;
    }

    public function getComboProductsBySubsidiary($CDFILIAL, $CDLOJA, $CDGRUPROMOC, $CDIDENTBUTON){
    	$params = array(
            'CDGRUPROMOC' => $CDGRUPROMOC,
            'CDFILIAL' => $CDFILIAL,
            'CDLOJA' => $CDLOJA,
            'CDIDENTBUTON' => $CDIDENTBUTON
        );
        try {
      		if ($this->util->databaseIsOracle()) {
	            $products = $this->connection->fetchAll(\Util\Query::ORACLE_GET_COMBO_PRODUCTS_SUBSIDIARY, $params);
	        } else {
	            $products = $this->connection->fetchAll(\Util\Query::SQL_GET_COMBO_PRODUCTS_SUBSIDIARY, $params);
	        }
 			$return = array(
	            'error' => false,
	            'data' => $products
	        );
 		}catch (\Exception $e) {
        	$return = array(
                'error' => true,
                'message' => 'Erro ao buscar produtos do combo. Erro: '.$e->getMessage()
            );
        } 
        return $return;
    }

    public function getComboProductsByProduct($CDFILIAL, $CDLOJA, $CDGRUPROMOC, $CDIDENTBUTON, $pricingTableCode){
		$params = array(
            'CDGRUPROMOC' => $CDGRUPROMOC,
            'CDFILIAL' => $CDFILIAL,
            'CDLOJA' => $CDLOJA,
            'CDIDENTBUTON' => $CDIDENTBUTON,
            'CDTABEPREC' => $pricingTableCode
        );
        try {
		    if ($this->util->databaseIsOracle()) {
	            $products = $this->connection->fetchAll(\Util\Query::ORACLE_GET_COMBO_PRODUCTS_PRODUCT, $params);
	        } else {
	            $products = $this->connection->fetchAll(\Util\Query::SQL_GET_COMBO_PRODUCTS_PRODUCT, $params);
	        }
 			$return = array(
	            'error' => false,
	            'data' => $products
	        );
 		}catch (\Exception $e) {
        	$return = array(
                'error' => true,
                'message' => 'Erro ao buscar produtos do combo. Erro: '.$e->getMessage()
            );
        } 
        return $return;
	}

	public function getComboGroupsBySubsidiaryDiscount($CDFILIAL, $CDIDENTBUTON){
    	$params = array(
            'CDFILIAL' => $CDFILIAL,
            'CDIDENTBUTON' => $CDIDENTBUTON
        );
        try {
      		$groups = $this->connection->fetchAll(\Util\Query::SQL_GET_COMBO_GROUPS_SUBSIDIARY_DISCOUNT, $params);
 			$return = array(
	            'error' => false,
	            'data' => $groups
	        );
 		}catch (\Exception $e) {
        	$return = array(
                'error' => true,
                'message' => 'Erro ao buscar grupos de combo do produto. Erro: '.$e->getMessage()
            );
        } 

        return $return;
	}
	
	public function getComboGroupsByProductDiscount($CDIDENTBUTON){
    	$params = array(
            'CDIDENTBUTON' => $CDIDENTBUTON
        );
        try {
      		$groups = $this->connection->fetchAll(\Util\Query::SQL_GET_COMBO_GROUPS_PRODUCT_DISCOUNT, $params);
 			$return = array(
	            'error' => false,
	            'data' => $groups
	        );
 		}catch (\Exception $e) {
        	$return = array(
                'error' => true,
                'message' => 'Erro ao buscar grupos de combo do produto. Erro: '.$e->getMessage()
            );
        } 

        return $return;
    }
    
    public function getWaitingTime($CDFILIAL, $CDLOJA){
		$params = array(
			'CDFILIAL' => $CDFILIAL,
			'CDLOJA'   => $CDLOJA
		);
		try {
		   	$time = $this->connection->fetchAssoc(\Util\Query::GET_WAITING_TIME, $params);
		   	$return = array(
				'error' => false,
			    'data' => $time
		  	);
	    } catch (\Exception $e) {
		    $return = array(
				'error' => true,
			    'message' => 'Erro ao buscar tempo de espera. Erro: '.$e->getMessage()
   		    );
		}
		return $return;
	}

	public function getStoreWaitingTime($CDFILIAL, $CDLOJA){
		$params = array(
			'CDFILIAL' => $CDFILIAL,
			'CDLOJA'   => $CDLOJA
		);
		try {
		   	$time = $this->connection->fetchAssoc(\Util\Query::GET_STORE_WAITING_TIME, $params);
		   	$return = array(
				'error' => false,
			    'data' => $time
		  	);
	    } catch (\Exception $e) {
		    $return = array(
				'error' => true,
			    'message' => 'Erro ao buscar tempo de espera da loja. Erro: '.$e->getMessage()
   		    );
		}
		return $return;
	}

	public function checkForConfTela(){
		try {
			$checkForConfTela = $this->connection->fetchAssoc(\Util\Query::CHECK_FOR_CONFTELA);
			$return = array(
				'error' => false,
				'data' => $checkForConfTela
			);
		} catch (\Exception $e){
			$return = array(
				'error' => true,
			    'message' => 'Erro ao buscar a tabela CONFTELACAIXA. Erro: '.$e->getMessage()
   		    );
		}
		return $return;
	}

	public function getConfTelaCaixa($CDFILIAL, $CDCAIXA){
		$params = array(
			'CDFILIAL' => $CDFILIAL,
			'CDCAIXA'   => $CDCAIXA
		);
		try {
			$confTelaCaixa = $this->connection->fetchAll(\Util\Query::GET_CONFTELA_CAIXA, $params);
			$return = array(
				'error' => false,
				'data' => $confTelaCaixa
			);
		} catch (\Exception $e){
			$return = array(
				'error' => true,
			    'message' => 'Erro ao buscar a configuração da tela do caixa. Erro: '.$e->getMessage()
   		    );
		}
		return $return;
	}
}