<?php

namespace Service;

class Order {
    
    protected $connection;
    protected $util;
    protected $precoAPI;
    protected $impressaoPedidoApi;
    protected $caixaAPI;
    protected $localUtil;
	protected $impressaoUtilAPI;
    
    const DELIVERY_ORDER = 'DLV';
    const TOGO_ORDER = 'TGO';
    const RESTAURANT_ORDER = 'MES';
    
	public function __construct($connection, \Odhen\API\Util\Util $util, \Odhen\API\Service\Preco $precoAPI,
								\Odhen\API\Service\ImpressaoPedido $impressaoPedidoApi,
								\Odhen\API\Service\Caixa $caixaAPI, \Util\Util $localUtil,
								\Odhen\API\Lib\ImpressaoUtil $impressaoUtilAPI,
								\Odhen\API\Service\Venda $vendaAPI) {
        $this->connection = $connection;
        $this->util = $util;
        $this->precoAPI = $precoAPI;
        $this->impressaoPedidoApi = $impressaoPedidoApi;
        $this->caixaAPI = $caixaAPI;
        $this->localUtil = $localUtil;
		$this->impressaoUtilAPI = $impressaoUtilAPI;
		$this->vendaAPI = $vendaAPI;
    }
    
    public function getStandardSeller($CDFILIAL) {
		$params = array(
			'CDFILIAL' => $CDFILIAL
		);
		try {
			if ($this->util->databaseIsOracle()) $standardSeller = $this->connection->fetchAssoc(\Util\Query::ORACLE_STANDARD_SELLER, $params);
			else $standardSeller = $this->connection->fetchAssoc(\Util\Query::SQL_STANDARD_SELLER, $params);
			if(!$standardSeller){
				$return = array(
					'error' => true,
					'message' => 'Parametrizacao de vendedor padrao incompleta ou inexistente'
				);
			}else{
				$return = array(
					'error' => false,
					'data' => $standardSeller
				);
			}	
		} catch (\Exception $e) {
			$return = array(
				'error' => true,
				'message' => 'Erro ao buscar parametros de vendedor padrao. Erro: '.$e->getMessage()
			);
		}
		return $return;
	}

    public function insertVendaRest($params){
		try {
            // verifica qual banco de dados está sendo utilizado
            if ($this->util->databaseIsOracle()) $success = $this->connection->executeQuery(\Util\Query::ORACLE_INSERT_VENDAREST, $params);
            else $success = $this->connection->executeQuery(\Util\Query::SQL_INSERT_VENDAREST, $params);
            $return = array('error' => false);
		} catch (\Exception $e) {
			$return = array(
				'error'   => true,
				'message' => 'Erro ao inserir vendarest. Erro: '.$e->getMessage()
			);
		}
		return $return;
	}

    public function insertComandaVen($params){
		try {
            if ($this->util->databaseIsOracle()) $success = $this->connection->executeQuery(\Util\Query::ORACLE_INSERT_COMANDAVEN, $params);
            else $success = $this->connection->executeQuery(\Util\Query::SQL_INSERT_COMANDAVEN, $params);
            $return = array('error' => false);
		} catch (\Exception $e) {
			$return = array(
				'error'   => true,
				'message' => 'Erro ao inserir comandaven. Erro: '.$e->getMessage()
			);
		}
		return $return;
	}

    public function insertMovCaixaDlv($params){
		try {
            if ($this->util->databaseIsOracle()) $success = $this->connection->executeQuery(\Util\Query::ORACLE_INSERT_MOVCAIXADLV, $params);
            else $success = $this->connection->executeQuery(\Util\Query::SQL_INSERT_MOVCAIXADLV, $params);
            $return = array('error' => false);
		} catch (\Exception $e) {
			$return = array(
				'error'   => true,
				'message' => 'Erro ao inserir pagamento. Erro: '.$e->getMessage()
			);
		}
		return $return;
	}

    public function insertItComandaVen($params){
		try {
            if ($this->util->databaseIsOracle()) $success = $this->connection->executeQuery(\Util\Query::ORACLE_INSERT_ITCOMANDAVEN, $params);
            else $success = $this->connection->executeQuery(\Util\Query::SQL_INSERT_ITCOMANDAVEN, $params);
            $return = array('error' => false);
		} catch (\Exception $e) {
			$return = array(
				'error' => true,
				'message' => 'Erro ao inserir produto. Erro: '.$e->getMessage()
			);
		}
		return $return;
	}

    public function insertItHrComanda($params){
		try {
            if (!$this->util->databaseIsOracle()) $success = $this->connection->executeQuery(\Util\Query::INSERT_ITHRCOMANDA, $params);
            $return = array('error' => false);
		} catch (\Exception $e) {
			$return = array(
				'error' => true,
				'message' => 'Erro ao inserir produto(hr). Erro: '.$e->getMessage()
			);
		}
		return $return;
	}

    public function insertObsItComandaVen($params){
		try {
            $success = $this->connection->executeQuery(\Util\Query::INSERT_OBSITCOMANDAVEN, $params);
            $return = array('error' => false);
		} catch (\Exception $e) {
			$return = array(
				'error' => true,
				'message' => 'Erro ao inserir observacao. Erro: '.$e->getMessage()
			);
		}
		return $return;
	}
    
    public function isValidCheckinId($checkinId) {
		$params = array(
			'IDCHECKIN' => $checkinId
		);

		$return = array('error' => false);		
		
		try {
			$response = $this->connection->fetchAssoc(\Util\Query::CHECK_VALID_CHECKINID, $params);
			if(!$response) {
				$return['error'] = true;
				$return['message']	= 'CheckinId inválido';
			}	
		} catch (\Exception $e) {
			$return['error'] = true;
			$return['message']	= 'Erro: ' . $e->getMessage();
		}
		return $return;
	}
    
    public function getResultCheckinMesPos($checkinId) {
		$params = array(
            'IDCHECKIN' => $checkinId
        );

		$return = array('error' => false);		

		try {
			$response = $this->connection->fetchAssoc(\Util\Query::GET_RESULT_CHECKINMESPOS, $params);
			if($response) {
				$return['data'] = $response;
			} else {
				$return['error'] = true;
				$return['message']	= 'CDFILIAL e NRVENDAREST não encontrados';
			}	
		} catch (\Exception $e) {
			$return['error'] = true;
			$return['message']	= 'Erro: ' . $e->getMessage();
		}
		return $return;
	}
    
    public function getResultParavendAndVendedor($CDFILIAL) {
		$params = array(
        	'CDFILIAL' => $CDFILIAL
        );

		$return = array('error' => false);		

		try {
			$response = $this->connection->fetchAssoc(\Util\Query::SQL_STANDARD_SELLER, $params);
			if($response) {
				$return['data'] = $response;
			} else {
				$return['error'] = true;
				$return['message']	= 'CDVENDEDOR, CDCAIXA e CDLOJA não encontrados';
			}	
		} catch (\Exception $e) {
			$return['error'] = true;
			$return['message']	= 'Erro: ' . $e->getMessage();
		}
		return $return;
	}

    public function getNrcomanda($NRVENDAREST, $CDFILIAL) {

		$params = array(
        	'NRVENDAREST' => $NRVENDAREST,
            'CDFILIAL' => $CDFILIAL
        );

		$return = array('error' => false);		
		
		try {
			$response = $this->connection->fetchAssoc(\Util\Query::GET_NRCOMANDA, $params);
			if($response) {
				$return['data'] = $response;
			} else {
				$return['error'] = true;
				$return['message']	= 'NRCOMANDA não encontrado';
			}	
		} catch (\Exception $e) {
			$return['error'] = true;
			$return['message']	= 'Erro: ' . $e->getMessage();
		}
		return $return;
	}

    public function getNrmesa($CDFILIAL, $NRVENDAREST) {
		$params = array(
        	'CDFILIAL' => $CDFILIAL,
            'NRVENDAREST'	=> $NRVENDAREST
        );

		$return = array('error' => false);		
		
		try {
			$response = $this->connection->fetchAssoc(\Util\Query::GET_NRMESA, $params);
			if($response) {
				$return['data'] = $response;
			} else {
				$return['error'] = true;
				$return['data'] = 'NRMESA não encontrado';
			}	
		} catch (\Exception $e) {
			$return['error'] = true;
			$return['message']	= 'Erro: ' . $e->getMessage();
		}
		return $return;
    }
    
    public function insertMovcaixamob($NRSEQMOVMOB, $CDFILIAL, $CDCAIXA, $CDVENDEDOR, $NRVENDAREST, $NRCOMANDA, $NRSEQVENDA, $NRMESA, $NRLUGARMESA, $CDTIPORECE, $IDTIPMOV, $VRMOV, $DSBANDEIRA, $IDTPTEF, $IDUTILTXSERV, $CDCLIENTE, $CDCONSUMIDOR) {
		$params = array(
	        'NRSEQMOVMOB' => $NRSEQMOVMOB, 
	        'CDFILIAL' => $CDFILIAL, 
	        'CDCAIXA' => $CDCAIXA, 
	        'CDVENDEDOR' => $CDVENDEDOR, 
	        'NRVENDAREST' => $NRVENDAREST, 
	        'NRCOMANDA' => $NRCOMANDA, 
	        'NRSEQVENDA' => $NRSEQVENDA, 
	        'NRMESA' => $NRMESA,
	        'NRLUGARMESA' => $NRLUGARMESA, 
	        'CDTIPORECE' => $CDTIPORECE, 
	        'IDTIPMOV' => $IDTIPMOV, 
	        'VRMOV' => $VRMOV, 
	        'DSBANDEIRA' => $DSBANDEIRA, 
	        'IDTPTEF' => $IDTPTEF,
	        'IDUTILTXSERV' => $IDUTILTXSERV,
	        'CDCLIENTE' => $CDCLIENTE,
	        'CDCONSUMIDOR' => $CDCONSUMIDOR
        );

		$return = array('error' => false);		

		try {
        	$this->connection->executeQuery(\Util\Query::INSERT_MOVCAIXAMOB, $params);
		} catch (\Exception $e) {
			$return['error'] = true;
			$return['message']	= 'Erro: ' . $e->getMessage();
		}
		return $return;
    }
    
    public function changeTableStatus($IDSTMESAAUX, $CDFILIAL, $CDLOJA, $NRMESA, $DTBLOQMESA = null) {
		$params = array(
			'IDSTMESAAUX' => $IDSTMESAAUX, 
			'CDFILIAL' => $CDFILIAL, 
			'CDLOJA' => $CDLOJA, 
			'NRMESA' => $NRMESA,
			'DTBLOQMESA' => $DTBLOQMESA
		);
		$return = array('error' => false);		
		try {
        	$this->connection->executeQuery(\Util\Query::SQL_ALTERA_STATUS_MESA, $params);
		} catch(\Exception $e) {
			$return['error'] = true;
			$return['message']	= $e->getMessage();
		}
		return $return;
	}

    public function changePositionStatus($IDSTLUGARMESA, $CDFILIAL, $NRVENDAREST, $NRCOMANDA, $NRLUGARMESA, $DTBLOQPOS = null){
		$params = array(
			'IDSTLUGARMESA' =>$IDSTLUGARMESA,
			'CDFILIAL' =>$CDFILIAL,
			'NRVENDAREST' =>$NRVENDAREST,
			'NRCOMANDA' =>$NRCOMANDA,
			'NRLUGARMESA' =>$NRLUGARMESA,
			'DTBLOQPOS' =>$DTBLOQPOS
    	);
		$return = array('error' => false);		
		try {
        	$this->connection->executeQuery(\Util\Query::SQL_ALTERA_STATUS_POSICAO, $params);
		} catch(\Exception $e) {
			$return['error'] = true;
			$return['message']	= $e->getMessage();
		}
		return $return;
	}

    public function changePositionCpf($CDFILIAL, $NRVENDAREST, $NRCOMANDA, $NRLUGARMESA, $NRINSCRCONS){
		$params = array(
			'CDFILIAL' =>$CDFILIAL,
			'NRVENDAREST' =>$NRVENDAREST,
			'NRCOMANDA' =>$NRCOMANDA,
			'NRLUGARMESA' =>$NRLUGARMESA,
			'NRINSCRCONS' =>$NRINSCRCONS
    	);
		$return = array('error' => false);		
		try {
        	$this->connection->executeQuery(\Util\Query::SQL_ALTERA_CPF_POSICAO, $params);
		} catch(\Exception $e) {
			$return['error'] = true;
			$return['message']	= $e->getMessage();
		}
		return $return;
	}

    public function getDeliveryAreas(){
    	try {
    		$params = array(
    			'SINGLE' => "'",
    			'DOUBLE' => '"'
    		);
    		if ($this->util->databaseIsOracle())$deliveryAreas = $this->connection->fetchAll(\Util\Query::ORACLE_GET_DELIVERY_AREAS, $params);
            else $deliveryAreas = $this->connection->fetchAll(\Util\Query::SQL_GET_DELIVERY_AREAS, $params);
            $return = array(
                'error' => false,
                'data'  => $deliveryAreas
            );
        } catch (\Exception $e) {
            $return = array(
                'error'   => true,
                'message' => 'Erro ao buscar areas de atendimento.Erro: '.$e->getMessage()
            );
        }
        return $return;
    }
    
    public function getProductsToRecalculate($CDFILIAL, $NRMESA, $NRLUGARMESA, $CDLOJA){
    	$params = array(		        
            'CDFILIAL' => $CDFILIAL,
            'NRMESA' => $NRMESA,
            'NRLUGARMESA' => $NRLUGARMESA,
            'CDLOJA' => $CDLOJA
        );
    	
        if ($this->util->databaseIsOracle()) $itens = $this->connection->fetchAll(\Util\Query::ORACLE_GET_PRODUCTS_RECALCULATE, $params);
        else $itens = $this->connection->fetchAll(\Util\Query::SQL_GET_PRODUCTS_RECALCULATE, $params);
        
        if(count($itens) == 0){
            $return = array(
                'error' => true,
                'message' => 'Não foram encontrado itens para esta venda'
            );
        }else{
            $return = array(
                'error' => false,
                'data' => $itens
            );
        }
        return $return;
    }
    
    public function getObservations($CDFILIAL, $NRVENDAREST, $NRCOMANDA, $NRPRODCOMVEN){
        $params = array(
            'CDFILIAL' => $CDFILIAL, 
            'NRVENDAREST' => $NRVENDAREST, 
            'NRCOMANDA' => $NRCOMANDA,
            'NRPRODCOMVEN' => $NRPRODCOMVEN    
        );
        
        try {
            if ($this->util->databaseIsOracle())$observations = $this->connection->fetchAll(\Util\Query::GET_LASTSALE_PRODUCT_OBSERVATIONS, $params);
            else $observations = $this->connection->fetchAll(\Util\Query::GET_POSITION_PRODUCT_OBSERVATIONS, $params);
            $return = array(
                'error' => false,
                'data' => $observations
            );
        } catch (\Exception $e) {
            $return = array(
                'error' => true,
                'message' => 'Erro ao buscar observacoes. Erro: '.$e->getMessage()
            );
        }
        return $return;
    }
    
    public function recalculateItensPrice($CDFILIAL, $CDCLIENTE, $CDPRODUTO, $CDLOJA, $CDCONSUMIDOR){
        return $this->precoAPI->buscaPreco($CDFILIAL, $CDCLIENTE, $CDPRODUTO, $CDLOJA, $CDCONSUMIDOR);
    }

    public function getCombosToRecalculate($CDFILIAL, $NRMESA, $NRLUGARMESA, $CDLOJA){
    	$params = array(
    		'CDFILIAL' => $CDFILIAL,
    		'NRMESA' => $NRMESA,
    		'NRLUGARMESA' => $NRLUGARMESA,
    		'CDLOJA' => $CDLOJA
    		
        );
        try {
        	if ($this->util->databaseIsOracle()) $combos = $this->connection->fetchAll(\Util\Query::ORACLE_GET_COMBOS_RECALCULATE, $params);
        	else $combos = $this->connection->fetchAll(\Util\Query::SQL_GET_COMBOS_RECALCULATE, $params);
            $return = array(
                'error' => false,
                'data' => $combos
            );
        } catch (\Exception $e) {
            $return = array(
                'error' => true,
                'message' => 'Erro ao buscar combos relacionados à venda. Erro: '.$e->getMessage()
            );
        }
        return $return;
    }
    
    public function getComboProductsToRecalculate($CDPRODPROMOCAO,$CDFILIAL,$NRVENDAREST,$NRCOMANDA, $NRSEQPRODCOM){
    	$params = array(
    		'CDPRODPROMOCAO' => $CDPRODPROMOCAO,
    		'CDFILIAL' => $CDFILIAL,
    		'NRVENDAREST' => $NRVENDAREST,
    		'NRCOMANDA' => $NRCOMANDA,
    		'NRSEQPRODCOM' => $NRSEQPRODCOM
        );
        try {
            if ($this->util->databaseIsOracle()) $comboProducts = $this->connection->fetchAll(\Util\Query::ORACLE_GET_COMBO_PRODUCTS_RECALCULATE, $params);
            else $comboProducts = $this->connection->fetchAll(\Util\Query::SQL_GET_COMBO_PRODUCTS_RECALCULATE, $params);
            $return = array(
                'error' => false,
                'data' => $comboProducts
            );
        } catch (\Exception $e) {
            $return = array(
                'error' => true,
                'message' => 'Erro ao buscar produtos dos combos relacionados à venda'.$e->getMessage()
            );
        }
        return $return;
    }
    
    public function printOrder($CDFILIAL, $CDLOJA, $produtos, $CDVENDEDOR, $DSCOMANDA, $NRMESA, $CDSENHAPED, $tipoVenda, $toGoString, $multiplasComandas, $ultimaComanda, $NRCOMANDA, $pedidoTaa){
		if($this->util->databaseIsOracle()){
			return true;
		}else{
			return $this->impressaoPedidoApi->imprimePedido($CDFILIAL, $CDLOJA, $produtos, $CDVENDEDOR, $DSCOMANDA, $NRMESA, $CDSENHAPED, $tipoVenda, $toGoString, $multiplasComandas, $ultimaComanda, $NRCOMANDA, $pedidoTaa);
		}
    }

    public function updateItemPrice($CDFILIAL,$NRVENDAREST,$NRCOMANDA,$NRPRODCOMVEN, $VRPRECCOMVEN, $VRDESCCOMVEN){
		$params = array(
			'CDFILIAL' => $CDFILIAL,
			'NRVENDAREST' => $NRVENDAREST,
			'NRCOMANDA' => $NRCOMANDA,
			'NRPRODCOMVEN' => $NRPRODCOMVEN,
			'VRPRECCOMVEN' => $VRPRECCOMVEN,
			'VRDESCCOMVEN' => $VRDESCCOMVEN
		);
		try {
            $success = $this->connection->executeQuery(\Util\Query::UPDATE_ITEM_PRICE, $params);
		} catch (\Exception $e) {
			$return = array(
				'error' => true,
				'message' => 'Erro ao atualizar valor do produto. Erro: '.$e->getMessage()
			);
		}
	}

    public function getTodaysExpenditure($CDCLIENTE, $CDCONSUMIDOR){
    	$params = array(
    		'CDCLIENTE' => $CDCLIENTE,
    		'CDCONSUMIDOR' => $CDCONSUMIDOR
    	);
    	if ($this->util->databaseIsOracle()) $todaysExpenditure = $this->connection->fetchAssoc(\Util\Query::ORACLE_GET_TODAYS_EXPENDITURE,$params);
        else $todaysExpenditure = $this->connection->fetchAll(\Util\Query::SQL_GET_TODAYS_EXPENDITURE, $params);
    	return !empty($todaysExpenditure['VRMOVEXTCONS']) ? $todaysExpenditure['VRMOVEXTCONS'] : 0;
	}
	
	public function getConsumerMaxDaily($clientCode, $consumerCode){
		$params = array(
			'CDCLIENTE' => $clientCode,
			'CDCONSUMIDOR' => $consumerCode
		);
		return $this->connection->fetchAssoc(\Util\Query::GET_CONSUMER_MAX_DAILY,$params);
	}
    /* public function makeSale(
        $NRORG, $CDFILIAL, $CDLOJA, $CDCAIXA,
        $CDVENDEDOR, $CDOPERADOR, $DTABERCAIX, $DTVENDA,
        $TOTALVENDA, $ITEMVENDA, $TIPORECE, $NMCONSVEND,
        $NRINSCRCONS, $NRSEQVENDA, $VRTROCOVEND, $EMAIL,
        $VRDESCVENDA, $CDCLIENTE, $CDCONSUMIDOR, $simulaImpressao,
        $simulateSaleValidation, $IDORIGEMVENDA, $CDSENHAPED){
        return $this->saleAPI->venda(
        $NRORG, $CDFILIAL, $CDLOJA, $CDCAIXA,
        $CDVENDEDOR, $CDOPERADOR, $DTABERCAIX, $DTVENDA,
        $TOTALVENDA, $ITEMVENDA, $TIPORECE, $NMCONSVEND,
        $NRINSCRCONS, $NRSEQVENDA, $VRTROCOVEND, $EMAIL,
        $VRDESCVENDA, $CDCLIENTE, $CDCONSUMIDOR, $simulaImpressao,
        $simulateSaleValidation, $IDORIGEMVENDA, $CDSENHAPED);
	} */
	
    public function getNRSEQVENDA($CDFILIAL, $CDCAIXA){
		$CDCONTADOR = 'PEDIDOVENDA' . $CDFILIAL . $CDCAIXA;
		if($this->util->databaseIsOracle())
			return $this->localUtil->newCode($CDCONTADOR, 10);
		else
			return $this->util->geraCodigo($this->connection, $CDCONTADOR, 1, 1, 10);
    }

    public function generateMovcaixamobSeq($CDFILIAL) {
		$movCounter = 'MOVCAIXAMOB' . $CDFILIAL;
		if($this->util->databaseIsOracle())
			return $this->localUtil->newCode($movCounter, 10);
		else
			return $this->util->geraCodigo($this->connection, $movCounter, 1, 1, 10);
    }
    
    public function generateVendarestSeq($CDFILIAL){
		$vendarestCounter = 'VENDAREST'.$CDFILIAL;
		if($this->util->databaseIsOracle())
        	return $this->localUtil->newCode($vendarestCounter, 10);
		else
			return $this->util->geraCodigo($this->connection, $vendarestCounter, 1, 1, 10);
	}

    public function generateComandavenSeq($CDFILIAL){
		$comandavenCounter = 'COMANDAVEN'.$CDFILIAL;
		if($this->util->databaseIsOracle())
        	return $this->localUtil->newCode($comandavenCounter, 10);
		else
			return $this->util->geraCodigo($this->connection, $comandavenCounter, 1, 1, 10);
		// return $this->util->geraCodigo($this->connection, $comandavenCounter, 1, 1, 10);
	}

    public function generateMovcaixadlvSeq($CDFILIAL, $NRVENDAREST){
		$movcaixadlvCounter = 'MOVCAIXADLV'.$CDFILIAL.$NRVENDAREST;
		if($this->util->databaseIsOracle())
        	return $this->localUtil->newCode($movcaixadlvCounter, 10);
		else
			return $this->util->geraCodigo($this->connection, $movcaixadlvCounter, 1, 1, 10);
	}

    public function generateItemsSeq($CDFILIAL){
		$itemsCounter = 'ITCOMANDAVEN'.$CDFILIAL;
		if($this->util->databaseIsOracle())
        	return $this->localUtil->newCode($itemsCounter, 10);
		else
			return $this->util->geraCodigo($this->connection, $itemsCounter, 1, 1, 10);
	}

    public function generateItcomandavenSeq($CDFILIAL, $NRCOMANDA){
		$itcomandavenCounter = 'ITCOMANDAVEN'.$CDFILIAL.$NRCOMANDA;
		if($this->util->databaseIsOracle())
        	return $this->localUtil->newCode($itcomandavenCounter, 6);
		else
			return $this->util->geraCodigo($this->connection, $itcomandavenCounter, 1, 1, 6);
	}

    public function generateKDSSeq($tableName){
		$kdsCounter = $tableName;
		if($this->util->databaseIsOracle())
        	return $this->localUtil->newCode($kdsCounter, 10);
		else
			return $this->util->geraCodigo($this->connection, $kdsCounter, 1, 1, 10);
	}

	public function validateTable($nrMesa){
		try{
		  $parametros = array(
			'NRMESA' => $nrMesa
		  );
		  $mesaValida = $this->connection->fetchAssoc(\Util\Query::VALIDA_MESA, $parametros);
		  if(empty($mesaValida)) throw new \Exception("Mesa não encontrada!");
		  if($mesaValida['IDSTMESAAUX'] == 'D') $this->connection->executeQuery(\Util\Query::ABRE_MESA, $parametros);
		  if($mesaValida['IDSTMESAAUX'] == 'S' || $mesaValida['IDSTMESAAUX'] == 'R') throw new \Exception("No momento a mesa se encontra bloqueada para realizar pedidos.");
		}catch(\Exception $e){
		  throw new \Exception($e->getMessage());
		}
	
	}

	public function getTableData($nrmesa, $cdfilial){
		$parametros = array(
			'NRMESA'   => $nrmesa,
			'CDFILIAL' => $cdfilial
		);
		  
		return $this->connection->fetchAssoc(\Util\Query::GET_DADOS_MESA, $parametros);
	}

	public function updateTableStatus($subsidiaryCode, $storeCode, $nrmesa){
		$parametros = array(
		  'NRMESA'   => $nrmesa,
		  'CDFILIAL' => $subsidiaryCode,
		  'CDLOJA'   => $storeCode
		);
		
		return $this->connection->executeQuery(\Util\Query::ATUALIZA_MESA, $parametros);
	}

	public function getTables($cdfilial){
		$params = array(
			'CDFILIAL' => $cdfilial
		);
		return $this->connection->fetchAll(\Util\Query::GET_TABLES, $params);
	}

	public function imprimeParcial ($produtosParcial , $cdfilial, $stVendaRests, $stComandaVens, $cdprodcouver, $modo,
		$nrvendarest, $nrcomanda, $cdloja, $nrmesa, $impPosicao, $nrpessoas, $totalProdutos, $taxaDeServico, $total, 
		$totalPorPessoa, $horas, $minutos, $segundos, $dscomanda, &$dadosImpressao, $fidelityValue, $idcomisvenda,
		$nmfanven, $idlugarmesa, $vrcomisvenda, $nrseqimprloja){
        // PRODUCT AGRUPATOR
    	$dadosComissao = $this->localUtil->getComissaoVenda($cdfilial, $nrvendarest, $nrcomanda);

		$produtosAgrupados = array();
        foreach ($produtosParcial as $produto){
            $quant = floatval($produto['QTPRODCOMVEN']);
            $key = $produto['NRLUGARMESA'].$produto['CDPRODUTO'].$produto['VRPRECCOMVEN'].$produto['VRDESCCOMVEN'].$produto['VRACRCOMVEN'];
            if ($produto['IDPESAPROD'] != 'S' && array_key_exists($key, $produtosAgrupados)){
                $produtosAgrupados[$key]['QTPRODCOMVEN'] = floatval($produtosAgrupados[$key]['QTPRODCOMVEN']) + $quant;
            }
            else {
            	if ($produto['IDPESAPROD'] == 'S') $key .= $produto['NRPRODCOMVEN'];
            	$produtosAgrupados[$key] = $produto;
            }
        }
        $produtosParcial = $produtosAgrupados;

        
		$arrPosicoes = array_filter(array_unique(array_column($produtosParcial,'NRLUGARMESA')));
		$arrNmPosicao = $this->buscaNomePorPosicao($cdfilial, $nrvendarest, $arrPosicoes);


		$valImpreLoja = $this->localUtil->getImprLoja($cdfilial, $cdloja, $nrseqimprloja);
		// Define parametros da impressora.
		$printerParams = $this->impressaoUtilAPI->buscaParametrosImpressora($valImpreLoja["IDMODEIMPRES"]);
		$strImprimir ='';
		// Monta string de impressao.
		$strImprimir .= $this->impressaoUtilAPI->centraliza($printerParams, "ACOMPANHAMENTO DO PEDIDO", $printerParams['largura']) . $printerParams['comandoEnter'];
		$strImprimir .= $this->impressaoUtilAPI->preencheLinha($printerParams, '', ' ') . $printerParams['comandoEnter'];
		if ($modo === "C") $strImprimir .= "Comanda: $dscomanda". $printerParams['comandoEnter'];

		$strImprimir .= "Mesa   : $nrmesa". $printerParams['comandoEnter'];

		$dataAtual = new \DateTime();
		$strImprimir .= "Garcom : ".$this->localUtil->removeAcentos($nmfanven). $printerParams['comandoEnter'];//nome do Garcom
		$strImprimir .= 'Emissao: '.$dataAtual->format("d/m/Y H:i:s"). $printerParams['comandoEnter'];

		$firstBlank   = true;
		$posicaoAtual = null;
		foreach ($produtosParcial as $produtoParcial){
			$posicaoMesa = $produtoParcial["NRLUGARMESA"];
			if (($idlugarmesa === "S") && ($modo === "M")){
				if ($posicaoAtual != $posicaoMesa){
					$strImprimir .= $this->impressaoUtilAPI->imprimeLinha($printerParams, $strImprimir);
					if ($firstBlank) $firstBlank = false;
					else $strImprimir .=  $printerParams['comandoEnter'];
					$strImprimir .= "Posicao ".$posicaoMesa;
					if (array_key_exists($posicaoMesa, $arrNmPosicao)){
						$nomeConsumidor = $arrNmPosicao[$posicaoMesa]['DSCONSUMIDOR'];
						$strImprimir .= ' - ' . $nomeConsumidor;
					}
					$strImprimir .= $printerParams['comandoEnter'];
					$posicaoAtual = $posicaoMesa;
				}
			}
			if ($firstBlank){
				$strImprimir .= "Posicao ".$posicaoMesa;
				if (array_key_exists($posicaoMesa, $arrNmPosicao)){
					$nomeConsumidor = $arrNmPosicao[$posicaoMesa]['DSCONSUMIDOR'];
					$strImprimir .=  ' - ' . $nomeConsumidor;
				}
				$strImprimir .= $printerParams['comandoEnter'];
				$firstBlank = false;
			}

			$strImprimir .= $produtoParcial["NMPRODUTO"]. $printerParams['comandoEnter'];
			$vrProd = round($produtoParcial["VRPRECCOMVEN"] - $produtoParcial["VRDESCCOMVEN"], 2);
			$totProd = $this->localUtil->formataPreco($produtoParcial["QTPRODCOMVEN"] * $vrProd);
			$vrProd = $this->localUtil->formataPreco($vrProd);
			$texto = $this->localUtil->toRight(str_replace(".",",",number_format($produtoParcial["QTPRODCOMVEN"], 3)) . " X " . $vrProd, $totProd, $printerParams['largura']);
			$largura = $printerParams['largura'] - strlen($texto);
			$strImprimir .= $texto. $printerParams['comandoEnter'];
		}
		$vrcomisvenda1 = floatval($vrcomisvenda);
		if ($impPosicao == '' && $nrpessoas > 1 && $idlugarmesa == "S" && $modo == "M") {
			$porcentServico = isset($dadosComissao['VRCOMISPOR']) ? $dadosComissao['VRCOMISPOR']: $vrcomisvenda1;
			$strImprimir .= $this->impressaoUtilAPI->imprimeLinha($printerParams, $strImprimir);
			$posicaoParcial = $this->localUtil->getPosicaoParcial($cdfilial, $nrvendarest, $nrcomanda);
			foreach ($posicaoParcial as $pos){
				// TOTAL PRODUTOS POSIÇÃO
				$vrTotPos = floatval($pos["PRECOTX"]) + floatval($pos["PRECONTX"]);
				$texto = "Total P" . $pos["NRLUGARMESA"] . ": " . $this->localUtil->formataPreco($vrTotPos) . " ";
				// TAXA POSIÇÃO
				if($taxaDeServico > 0) {
					$vrTxPos = (floatval($pos["PRECOTX"]) / 100) * $porcentServico;
					$texto .= "TX ". $this->localUtil->formataPreco($vrTxPos) . " ";	
				} else {
					$vrTxPos = 0;
					$texto .= "TX ". $this->localUtil->formataPreco($vrTxPos) . " ";
				}
				// TOTAL GERAL POR POSIÇÃO
				$texto .= $this->localUtil->formataPreco($vrTotPos + $vrTxPos);
				$strImprimir .= $texto . $printerParams['comandoEnter'];
			}
		}

		// lista produtos
		$strImprimir .= $this->impressaoUtilAPI->imprimeLinha($printerParams, $strImprimir);
		$texto        = "Total Produtos";
		$strImprimir .= $this->localUtil->toRight($texto, $this->localUtil->formataPreco($totalProdutos), $printerParams['largura']) . $printerParams['comandoEnter'];

		if ($fidelityValue > 0){
			$texto        = "Desconto Fidelidade";
			$strImprimir .= $this->localUtil->toRight($texto, $this->localUtil->formataPreco($fidelityValue), $printerParams['largura']) . $printerParams['comandoEnter'];
		}
		$totalPagar = round($totalProdutos - $fidelityValue, 2);
		$texto = "Subtotal";
		$strImprimir .= $this->localUtil->toRight($texto, $this->localUtil->formataPreco($totalPagar), $printerParams['largura']) . $printerParams['comandoEnter'];

		if ($taxaDeServico > 0) {
			$texto        = "Gorjeta Sugerida";
			$strImprimir .= $this->localUtil->toRight($texto, $this->localUtil->formataPreco($taxaDeServico), $printerParams['largura']) . $printerParams['comandoEnter'];
		}

		// coloca total de couvert
		$params = array($cdfilial, $nrvendarest, $nrcomanda, $cdprodcouver);
		$produtoCouvertParcial = $this->localUtil->getProdutosParcialCouver($cdfilial, $nrvendarest, $nrcomanda, $cdprodcouver);
		if ($produtoCouvertParcial["PRECO"] > 0) {
			$texto = "Couvert ";
			$strImprimir .= $texto . $this->localUtil->formataPreco($produtoCouvertParcial["PRECO"]). $printerParams['comandoEnter'];
		}
		$temp = "Total Geral";
		$strImprimir .= $this->localUtil->toRight($temp, $this->localUtil->formataPreco($total), $printerParams['largura']) . $printerParams['comandoEnter'];

		if ($idcomisvenda === 'S') {
	        $vrcomisvenda2 = null;
	        $vrcomisvenda3 = null;
	        
			if ($vrcomisvenda1 === 0.0 && $vrcomisvenda2 === 0.0 && $vrcomisvenda3 === 0.0){
				$vrcomisvenda1 = $vrcomisvenda2 = $vrcomisvenda3 = null;
			} else if ($vrcomisvenda1 === 0.0 && $vrcomisvenda2 === 0.0){
	        	$vrcomisvenda2 = null;
	        } else if ($vrcomisvenda1 === 0.0 && $vrcomisvenda3 === 0.0){
	        	$vrcomisvenda3 = null;
	        } else if ($vrcomisvenda2 === 0.0 && $vrcomisvenda3 === 0.0){
	        	$vrcomisvenda3 = null;
	        }

			if (($vrcomisvenda1 !== null && $vrcomisvenda2 !== null)  || ($vrcomisvenda1 !== null && $vrcomisvenda3 !== null) || ($vrcomisvenda2 !== null && $vrcomisvenda3 !== null)){
	            $strImprimir .= $this->impressaoUtilAPI->imprimeLinha($printerParams, $strImprimir);
	            $texto = "Gorjeta Sugerida";
	            
	            if ($vrcomisvenda1 !== null) {
		            $vrTaxa = ($totalPagar / 100) * $vrcomisvenda1;
		            $strImprimir .= $this->localUtil->toRight($texto . ' ' . $this->localUtil->formataPreco($vrcomisvenda1) . '%', $this->localUtil->formataPreco($vrTaxa + $totalPagar), $printerParams['largura']) . $printerParams['comandoEnter'];
		        }
	            if ($vrcomisvenda2 !== null) {
	                $vrTaxa = ($totalPagar / 100) * $vrcomisvenda2;
	                $strImprimir .= $this->localUtil->toRight($texto . ' ' . $this->localUtil->formataPreco($vrcomisvenda2) . '%', $this->localUtil->formataPreco($vrTaxa + $totalPagar), $printerParams['largura']) . $printerParams['comandoEnter'];
	            }
	            if ($vrcomisvenda3 !== null) {
	                $vrTaxa = ($totalPagar / 100) * $vrcomisvenda3;
	                $strImprimir .= $this->localUtil->toRight($texto . ' ' . $this->localUtil->formataPreco($vrcomisvenda3) . '%', $this->localUtil->formataPreco($vrTaxa + $totalPagar), $printerParams['largura']) . $printerParams['comandoEnter'];
	            }
	        } 
	    }

		if ($nrpessoas > 1) {
			$strImprimir .= $this->impressaoUtilAPI->imprimeLinha($printerParams, $strImprimir);
			if ($impPosicao == ''){
				$strImprimir .= "$nrpessoas PESSOA(S) - " . $this->localUtil->formataPreco($totalPorPessoa) . " POR PESSOA".$printerParams['comandoEnter'];
			}
		} else {
			$strImprimir .= $this->impressaoUtilAPI->imprimeLinha($printerParams, $strImprimir);
		}

		$strImprimir .= "Tempo de permanencia - $horas:$minutos:$segundos". $printerParams['comandoEnter'];
		$strImprimir .= $this->impressaoUtilAPI->imprimeLinha($printerParams, $strImprimir);
        $strImprimir .= $this->impressaoUtilAPI->centraliza($printerParams, 'Teknisa Software - www.teknisa.com') . $printerParams['comandoEnter'];
		$largura      = $printerParams['largura'];
		$strImprimir .= $printerParams['comandoEnter'];

		if (!$printerParams['impressaoFront']){
			$options['bold'] = true;
            $options['letterType'] = '2';
			$yesterday = date("d/m/Y", time() - 60 * 60 * 24);
			$this->localUtil->newCodeWt('VENDADIA' . $cdfilial . $yesterday, 20);
			$pedido = $this->localUtil->getNewCode('VENDADIA' . $cdfilial . $yesterday, 5);
			return $this->impressaoUtilAPI->imprimeNaoFiscal($strImprimir, $valImpreLoja, $pedido . 'D', $options);
		} else {
			$dadosImpressao = $strImprimir;			
		}
	}

	private function buscaNomePorPosicao($cdfilial, $nrvendarest, $posicoes){
        return array();
	}

	public function getWaiters(){
		$waiters = $this->connection->fetchAll(\Util\Query::GET_WAITERS);
		if(empty($waiters)) throw new \Exception("Não foram encontrados garçons.");
		return $waiters;
	}

	public function getOrderProducts($nrcomanda, $nrmesa){
		$params = array(
			'NRCOMANDA' => $nrcomanda,
			'NRMESA' => $nrmesa
		);
		$orderProducts = $this->connection->fetchAll(\Util\Query::GET_ORDER, $params);
		if (empty($orderProducts)) throw new \Exception("Comanda não encontrada.");
		foreach ($orderProducts as &$product) {
			$product['preco'] = $this->localUtil->removeDecimal($product['preco'], 3);
			$product['quantidade'] = $this->localUtil->removeDecimal($product['quantidade'], 3);
			$product['desconto'] = $this->localUtil->removeDecimal($product['desconto']);
			$product['acrescimo'] = $this->localUtil->removeDecimal($product['acrescimo']);
			$product['total'] = $this->localUtil->removeDecimal($product['total'], 3);
		}
		unset($product);
		return $orderProducts;
	}

	public function getVendorInfo($sellerCode){
		$params = array(
		  'CDVENDEDOR' => $sellerCode  
		);
		$vendor = $this->connection->fetchAssoc(\Util\Query::GET_VENDOR_INFO, $params);
		if(empty($vendor)) throw new \Exception("Vendedor não vinculado a um operador.");
		$return = array(
			'error' => false,
			'data'  => $vendor
		);
		return $return;
	}

	public function getComandaData($comandavenSeq, $subsidiaryCode){
		$params = array(
			'NRCOMANDA' => $comandavenSeq,
			'CDFILIAL' => $subsidiaryCode
		);
		return $this->connection->fetchAssoc(\Util\Query::GET_COMANDAVEN_DATA, $params);
	}

	public function getOrder($cdfilial, $nrcomanda){
		$params = array(
			'CDFILIAL' => $cdfilial,
			'NRCOMANDA' => $nrcomanda
		);
		if($this->util->databaseIsOracle()) return $this->connection->fetchAssoc(\Util\Query::GET_ORDER_DATA_ORACLE, $params);
		else return $this->connection->fetchAssoc(\Util\Query::GET_ORDER_DATA_SQL, $params);
	}

	public function getOrderPayments($cdfilial, $nrvendarest){
		$params = array(
			'CDFILIAL' => $cdfilial,
			'NRVENDAREST' => $nrvendarest
		);
		return $this->connection->fetchAll(\Util\Query::GET_ORDER_PAYMENTS, $params);
	}

	public function closeOrder($order, $cdfilial, $nrcomanda){
		        
        if(empty($this->getOrder($cdfilial, $nrcomanda))) throw new \Exception("O pedido ($cdfilial-$nrcomanda) não foi encontrado para fechamento.");

		$cdfilial = $order['CDFILIAL'];
		$nrvendarest = $order['NRVENDAREST'];
		$cdoperador = $order['CDOPERADOR'];
		$nrinscrcons = $order['NRINSCRCONS'];
		$nrmesa = $order['NRMESA'];
		$nrpesmesaven = $order['NRPESMESAVEN'];
		$dsComanda = $order['DSCOMANDA'];
		$cdloja = $order['CDLOJA'];
		$cdvendedor = $order['CDVENDEDOR'];
		$nmconsvend = $order['NMCONSUMIDOR'];
		$cdcliente = $order['CDCLIENTE'];
		$cdconsumidor = $order['CDCONSUMIDOR'];
		$idorigemvenda = $order['IDORGCMDVENDA'];
		$cdsenhaped = $order['CDSENHAPED'];
		$vracrcomanda = $this->localUtil->removeDecimal($order['VRACRCOMANDA']);
		$vrdescfid = $this->localUtil->removeDecimal($order['VRDESCFID']);
        $email = null;
		$income = array();
		$change = array();
		
		$payments = $this->getOrderPayments($cdfilial, $nrvendarest);

		$itensValue = $this->getItemsValue($cdfilial, $nrvendarest, $nrcomanda);

		foreach($payments as $payment){
			if($payment['IDTIPOMOVIVEDLV'] == 'E') array_push($income, $payment);
			else if($payment['IDTIPOMOVIVEDLV'] == 'S') $change = $payment;
		}
		
		$caixa = $this->localUtil->getCDCAIXA($cdfilial, $nrcomanda, $nrvendarest);
		if($caixa['IDCOLETOR'] === 'C') $cdcaixa = $this->localUtil->recuperaCaixaRecebedor();
		else {
			$cdcaixa = $caixa['CDCAIXACOLETOR'];
			$checaEstadoFiscal = $this->caixaAPI->checaEstadoFiscal($cdfilial, $cdcaixa, '1');
		}
		
        if(empty($cdcaixa)) throw new \Exception("Não foi encontrado caixa recebedor aberto.");
        
		$dtabercaix    = $this->getAberturaCaixa($cdfilial, $cdcaixa, $cdoperador);
        
		$nrorg         = '1'; 
        $dtvenda       = new \DateTime();
        $dtabercaix    = new \DateTime($dtabercaix);
        
        $dadosMesa     = array(
            array(
                'NRVENDAREST'    => $nrvendarest,
                'NRCOMANDA'      => $nrcomanda,
                'NRMESA'         => $nrmesa,
                'NRPESMESAVEN'   => $nrpesmesaven,
                'DSCOMANDA'      => $dsComanda,
                'DTHRMESAFECH'   => $dtvenda
            )
        );
		
        $lugarmesa = $this->getLugarMesa($cdfilial, $nrcomanda, $nrvendarest);
        $arrayPosicoes = array(
            $lugarmesa
        );
		
        $imprimeCupom  = false;
        $simuSaleValid = false;
        $retornoVenda = $this->vendaAPI->vendaMesa(
            $nrorg,
            $cdfilial,
            $cdloja,
            $cdcaixa,
            $cdvendedor,
            $cdoperador,
            $dtabercaix,
			$dtvenda,
			$itensValue,
            $income,
            $nmconsvend,
            $nrinscrcons,
            $cdsenhaped,
            $change,
            $email,
            0,
            $cdcliente,
            $cdconsumidor,
            $imprimeCupom,
            $simuSaleValid,
            $dadosMesa,
            $arrayPosicoes,
            $idorigemvenda,
            0,
            $vrdescfid,
            0,
            null,
            null,
            null,
            true,
            $vracrcomanda
        );
        $retornoVenda['CDCAIXA'] = $cdcaixa;
        return $retornoVenda;

	}
	
	private function getAberturaCaixa($cdfilial, $cdcaixa, $cdoperador){
        // Pega as informações do caixa informado
        $abertura = $this->caixaAPI->getEstadoCaixa($cdfilial, $cdcaixa, '1');
		
        // Se caixa fechado, abre e retorna a data, senão, retorna a data de abertura.
        if(!empty($abertura) && $abertura['estado'] == "aberto") {
            $nrconftela = $this->getConfTela($cdcaixa, $cdfilial);
            
            if($this->util->databaseIsOracle()) $dataAbertura = \DateTime::createFromFormat('Y-m-d H:i:s', $abertura['DTABERCAIX']);
            else $dataAbertura = \DateTime::createFromFormat('Y-m-d H:i:s.u', $abertura['DTABERCAIX']);
            $today = new \DateTime();
            $NRORG = "1";
            if($dataAbertura->format('d-m-Y') !== $today->format('d-m-Y')){
				$tiporece = $this->getTiporeceFechamento($cdcaixa, $cdfilial, $dataAbertura->format('d/m/Y H:i:s'));
				
				$caixaInfo = $this->getCaixaInfo($cdfilial, $cdcaixa);
				
				$IDHABCAIXAVENDA = $caixaInfo['IDHABCAIXAVENDA'];
				$IDSINCCAIXADLV = $caixaInfo['IDSINCCAIXADLV'];
                $respostaFechamento = $this->caixaAPI->fechaCaixa(
                    'S',
                    $dataAbertura,
                    $today,
                    $cdfilial,
                    $cdcaixa,
                    $NRORG,
                    $cdoperador,
                    $nrconftela,
                    $tiporece,
					false,
					$IDHABCAIXAVENDA,
					$IDSINCCAIXADLV
                );
				
                if(!$respostaFechamento['error']){
                    $respostaAbertura = $this->abreCaixa($cdfilial, $cdcaixa, $cdoperador);
                    if(!$respostaAbertura['error']) return $this->caixaAPI->getEstadoCaixa($cdfilial,$cdcaixa,'1')['DTABERCAIX'];
                    else throw new \Exception($respostaAbertura['message']);
                }else throw new \Exception($respostaFechamento['message']);
            }else{
                return $dataAbertura->format('d-m-Y H:i:s');
            }
        }else{
            $respostaAbertura = $this->abreCaixa($cdfilial, $cdcaixa, $cdoperador);
            if(!$respostaAbertura['error']) return $this->caixaAPI->getEstadoCaixa($cdfilial,$cdcaixa, '1')['DTABERCAIX'];
            else throw new \Exception($respostaAbertura['message']);
        }
	}
	
	private function getConfTela($cdcaixa, $cdfilial){
        $parametros = array(
            'CDCAIXA' => $cdcaixa,
            'CDFILIAL' => $cdfilial
        );
        
        $resultado = $this->connection->fetchAssoc(\Util\Query::GET_CONFTELA, $parametros);
        return $resultado['NRCONFTELA'];
	}
	
	private function getTiporeceFechamento($cdcaixa, $cdfilial, $dataAbertura){
        $parametros = array(
            'CDFILIAL' => $cdfilial,
            'CDCAIXA' => $cdcaixa,
            'DTABERCAIX' => $dataAbertura
        );
        if ($this->util->databaseIsOracle()) return $this->connection->fetchAll(\Util\Query::ORACLE_GET_RECEBIMENTOS_FECHAMENTO, $parametros);
        else return $this->connection->fetchAll(\Util\Query::SQL_GET_RECEBIMENTOS_FECHAMENTO, $parametros);
	}
	
	private function abreCaixa($cdfilial, $cdcaixa, $cdoperador){
        $NRORG            = "1";
        $IDMONGO          = "";
        $VRMOVIVEND       = 0;
        $IDATUTURCAIXA    = "I";
        $DTABERCAIX       = new \DateTime();
		$DTMOVTURCAIX     = new \DateTime();
		
		$caixaInfo = $this->getCaixaInfo($cdfilial, $cdcaixa);

		$IDHABCAIXAVENDA = $caixaInfo['IDHABCAIXAVENDA'];
		$IDSINCCAIXADLV = $caixaInfo['IDSINCCAIXADLV'];

        // Realiza a abertura de caixa.
        return $this->caixaAPI->abreCaixa(
            $cdfilial,
            $DTABERCAIX,
            $cdcaixa,
            $NRORG,
            $cdoperador,
            $DTMOVTURCAIX, 
            $VRMOVIVEND, 
            $IDMONGO, 
            $IDATUTURCAIXA, 
			false,
			$IDHABCAIXAVENDA,
			$IDSINCCAIXADLV
		);
	}
	
	private function getLugarMesa($cdfilial, $nrcomanda, $nrvendarest){
        $parametros = array(
            'CDFILIAL'    => $cdfilial,
            'NRCOMANDA'   => $nrcomanda,
            'NRVENDAREST' => $nrvendarest
        );
        
        $retorno = $this->connection->fetchAssoc(\Util\Query::GET_LUGARMESA, $parametros);
        return $retorno['NRLUGARMESA'];
	}
	
	private function getCaixaInfo($cdfilial, $cdcaixa){
		$parametros = array(
            'CDFILIAL' => $cdfilial,
            'CDCAIXA'  => $cdcaixa,
        );
        
        return $this->connection->fetchAssoc(\Util\Query::GET_CAIXA_INFO, $parametros);
	}

	public function getItemsValue($cdfilial, $nrvendarest, $nrcomanda){
		$params = array(
			'CDFILIAL' => $cdfilial,
			'NRVENDAREST' => $nrvendarest,
			'NRCOMANDA' => $nrcomanda
		);

		$total = $this->connection->fetchAssoc(\Util\Query::GET_ITEMS_VALUE, $params);
		return $total['TOTAL'];
	}
	
	public function cancelOrder($cdfilial, $cdcaixa, $nrseqvenda, $cdoperador, $cdsupervisor){
		return $this->vendaAPI->cancelaCupom($cdfilial, $cdcaixa, '1', $nrseqvenda, $cdoperador, $cdsupervisor);
	}
	
	public function getOrderItems($nrvendarest, $nrcomanda, $cdfilial){
		$params = array(
			'NRVENDAREST' => $nrvendarest,
			'NRCOMANDA' => $nrcomanda,
			'CDFILIAL' => $cdfilial
		);
		return $this->connection->fetchAll(\Util\Query::GET_ORDER_ITEMS, $params);
	}
	
	public function saveCashBack($nrcomanda, $nrvendarest, $cdfilial, $cashBack){
		$params = array(
			'NRCOMANDA' => $nrcomanda,
			'NRVENDAREST' => $nrvendarest,
			'CDFILIAL' => $cdfilial,
			'VRDESCFID' => $cashBack
		);
		return $this->connection->executeQuery(\Util\Query::SAVE_CASHBACK, $params);
	}
	
	public function updateItemDiscount($nrcomanda, $nrvendarest, $cdfilial, $cdproduto, $nrprodcomven, $discount){
		$params = array(
			'NRCOMANDA' => $nrcomanda,
			'NRVENDAREST' => $nrvendarest,
			'CDFILIAL' => $cdfilial,
			'CDPRODUTO' => $cdproduto,
			'NRPRODCOMVEN' => $nrprodcomven,
			'VRDESCCOMVEN' => $discount
		);
		return $this->connection->executeQuery(\Util\Query::UPDATE_ITEM_DISCOUNT, $params);
	}
	
	public function getOrderCashback($cdfilial, $nrvendarest, $nrcomanda){
		$params = array(
			'NRVENDAREST' => $nrvendarest,
			'CDFILIAL' => $cdfilial,
			'NRCOMANDA' => $nrcomanda
		);
		return $this->connection->fetchAssoc(\Util\Query::GET_ORDER_CASHBACK, $params); 
	}
	
	public function updateConsumerVendarest($clientCode, $consumerCode, $nrvendarest, $cdfilial){
		$params = array(
			'CDCLIENTE' => $clientCode,
			'CDCONSUMIDOR' => $consumerCode,
			'NRVENDAREST' => $nrvendarest,
			'CDFILIAL' => $cdfilial
		);
		return $this->connection->executeQuery(\Util\Query::UPDATE_CONSUMER_VENDAREST, $params);
	}
	
	public function getQuantItens($subsidiaryCode, $vendarestSeq, $comandavenSeq){
		$params = array(
			'CDFILIAL' => $subsidiaryCode,
			'NRVENDAREST' => $vendarestSeq,
			'NRCOMANDA' => $comandavenSeq	
		);
		
		$quantItens = $this->connection->fetchAssoc(\Util\Query::GET_QUANT_ITENS, $params);
		
		return empty($quantItens) ? 1 : $quantItens['QUANT'] + 1;
	}
	
	public function getOrderDscomanda($dscomanda, $subsidiaryCode){
		$dscomanda = str_pad($dscomanda, 10, 0, STR_PAD_LEFT);
		$params = array(
			'CDFILIAL' => $subsidiaryCode,
			'DSCOMANDA' => $dscomanda
		);
		
		return $this->connection->fetchAssoc(\Util\Query::GET_ORDER_DSCOMANDA, $params);
	}
	
	public function getOrderById($taaSaleId){
		$params = array(
			'NRCOMANDAEXT' => $taaSaleId
		);
		return $this->connection->fetchAssoc(\Util\Query::GET_ORDER_TAA_ID, $params);
	}
}